package AIspace.XMLReader;

import java.util.ArrayList;
/**
 * @author Joseph Roy Santos
 *
 * Class that represents a single XML "block" (text enclosed by tags)
 * and provides accessors to easily read it.
 */
public class XMLBlock {
	
	/** stores the text between open and close tags */
	private String text;
	
	/** stores the tag name/name of the block */
	private String tagName;
	
	/** stores an arraylist of pair corresponding to the property - value 
	 * values in the node
	 */
	private ArrayList<Pair> properties;
	
	/** stores an arraylist corresponding to the child nodes of this block
	 * This is public for traversal reasons. It is hoped that eventually this
	 * could be made private
	 */
	public ArrayList<XMLBlock> childBlocks;

	/** constructor */	
	public XMLBlock () {
		text = "";
		tagName = "";
		properties = new ArrayList<Pair>();
		childBlocks = new ArrayList<XMLBlock>();
	}		

	/** constructor */	
	public XMLBlock (String t, String name, ArrayList<Pair> prop, ArrayList<XMLBlock> children) {
		text = t;
		tagName = name;
		properties = new ArrayList<Pair>(prop);
		childBlocks = new ArrayList<XMLBlock>(children);
	}
	
	/** copy constructor */
	public XMLBlock (XMLBlock original) {
		text = original.text;
		tagName = original.text;
		properties = new ArrayList<Pair>(original.properties);
		childBlocks = new ArrayList<XMLBlock>(original.childBlocks);
	}
	
	/** accessor. Returns text between XML open and close tags */
	public String getText() {
		return text;
	}
	
	/** accessor. Returns the tag name for this XML block */
	public String getTag() {
		return tagName;
	}
	
	/** accessor. Returns an arraylist of the property-value pairs for this block*/
	public ArrayList<Pair> getProperties() {
		return properties;
	}
	
	/** mutator. Sets the text */
	public void setText(String t) {
		text = t;
	}
	
	/** mutator. Sets the tag for this block */
	public void setTag(String tag) {
		tagName = tag;
	}

	/** adds a property - value pair to the node */	
	public void addProperty(String propertyName, String propertyValue) {
		Pair pair = new Pair();
		pair.name = propertyName;
		pair.value = propertyValue;
		properties.add(pair);
	}
	
	/** adds a child to the node */
	public void addChild(XMLBlock child) {
		childBlocks.add(child);
	}
	
	/** removes a child from the node, which has the first instance
	 * of <code>name</code> 
	 * @return true if the node is found. False otherwise.
	 * */
	public boolean removeChild(String name) {
		for (int i = 0; i < childBlocks.size(); i++) {
			if (name.equals(childBlocks.get(i).tagName)) {
				childBlocks.remove(i);
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Searches for a child with the specified tag and returns it. 
	 * If not found, null is returned. If there is more than one tag,
	 * a ArrayList<XMLBlock> is returned.
	 */
	public ArrayList<XMLBlock> searchChildTag(String tag) {
		ArrayList<XMLBlock> searchResults = new ArrayList<XMLBlock>();
		for (int i = 0; i < childBlocks.size(); i++) {
			XMLBlock currBlock = childBlocks.get(i);
			if (currBlock.tagName.equalsIgnoreCase(tag)) {
				searchResults.add(currBlock);	
			}	
		}
		if (searchResults.size() > 0) {
			return searchResults;
		}
		return null;
	}
}
