/**
 * File: GraphWindow.java
 * Aug 10, 2000
 */

package AIspace.graphToolKit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.net.URL;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import AIspace.graphToolKit.dialogs.OpenLocationDialog;
import AIspace.graphToolKit.dialogs.ProblemDialog;
import AIspace.graphToolKit.elements.Edge;
import AIspace.graphToolKit.elements.Node;
import AIspace.graphToolKit.elements.Point;
import AIspace.graphToolKit.help.HelpMenu;

/**
 * The main window class.
 */

public class GraphWindow extends JFrame implements WindowListener, ActionListener, ChangeListener{

	private static final long serialVersionUID = 1L;
	protected boolean undoBoolean;
	/** name of the file being loaded */
	protected String fileName;
	/** title of the applet */
	protected static String appletTitle;
	/** name of the applet */
	protected static String appletName;
	/** text about the applet */
	protected static String aboutText;
	/** font used in the toolbar buttons */
	protected Font toolBarFont;

	/* ITEMS FOR SAVING/RESTORING PROPERTIES */
	protected int savedLineWidth;
	protected int savedFontSize;
	protected JRadioButtonMenuItem savedFontSizeMenuItem;
	protected JRadioButtonMenuItem savedLineWidthMenuItem;

	/* MENU FONT CHECKBOXES */
	protected JRadioButtonMenuItem extraLargeFont;
	protected JRadioButtonMenuItem largeFont;
	protected JRadioButtonMenuItem mediumFont;
	protected JRadioButtonMenuItem smallFont;
	protected JRadioButtonMenuItem otherFont;

	/* MENU LINE WIDTH CHECKBOXES */
	protected JRadioButtonMenuItem wideLine;
	protected JRadioButtonMenuItem mediumLine;
	protected JRadioButtonMenuItem narrowLine;

	/* MENU PAN/ZOOM CHECKBOXES */
	protected JRadioButtonMenuItem pan;
	protected JRadioButtonMenuItem zoom;

	/* BOOLEANS RECORDING STATE VISIBILITY FOR ELEMENTS */
	protected boolean isMessagePanelShowing = true;
	protected boolean isAntiAliasingEnabled = true;
	protected boolean isButtonTextShowing = true;
	protected boolean isToolBarShowing = true;
	protected GridBagConstraints gbc;
	protected GridBagLayout gbl;
	protected JPanel canvasPanel;
	protected JPanel bottomPanel;
	protected JTabbedPane tabbedPane;
	public JScrollPane scrollPanel;
	protected JSplitPane tabandbottom;

	/** text area showing at bottom of canvas */
	protected JTextArea algoText;

	/* ELEMENTS FOR BUTTONS ON TOOLBAR */
	protected JToggleButton[] createButModes;
	protected JToggleButton[] solveButModes;
	protected ButtonGroup createModeButGroup;
	protected ButtonGroup solveModeButGroup;
	protected JToolBar toolBar;

	/* SAVE BUTTON/MODE BETWEEN MODE SWITCHES */
	/** selected button index in other (CREATE or SOLVE) main mode */
	protected int otherSavedButtonIndex = -1;

	/* ICONS FOR TOOLBAR */
	protected ImageIcon createNodeIcon = createImageIcon("images/common/newNode.png");
	protected ImageIcon createEdgeIcon = createImageIcon("images/common/newEdge2.png");
	protected ImageIcon selectIcon = createImageIcon("images/common/select.png");
	protected ImageIcon deleteIcon = createImageIcon("images/common/delete.png");
	protected ImageIcon setPropsIcon = createImageIcon("images/common/setProp.png");

	protected JPanel emptyPanel1; // empty panel
	protected JPanel emptyPanel2; // empty panel

	//protected ProblemDialog pDialog;
	
	protected GraphCanvas canvas;
	/** area that shows messages above the canvas displaying the graph */
	protected LabelCanvas messageCanvas;
	/** this application as an applet */
	protected JApplet applet;
	
	// Methods

	/**
	 * Constructor.
	 */
	public GraphWindow(JApplet parent) {
		applet = parent;
		setBackground(GraphConsts.bg);
		setForeground(GraphConsts.fg);
		setSize(600, 700);
		toolBarFont = new Font("arial", Font.PLAIN, 10);
		initializeAppletInfo();
		setTitle(appletTitle);

		addWindowListener(this);

		// Place the widgets
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;

		UIManager.put("TabbedPane.selected", Color.white);

		// solve/create tabs
		tabbedPane = new JTabbedPane();

		// NOTE: check out cloning
		// panels passed to tabs
		emptyPanel1 = new JPanel();
		emptyPanel2 = new JPanel();

		// main canvas for graph
		canvasPanel = new JPanel();
		canvasPanel.setBackground(Color.white);

		// canvas where message to user is displayed
		messageCanvas = new LabelCanvas();

		getCanvas();
		canvasPanel.setLayout(gbl);

		scrollPanel.setPreferredSize(new Dimension(425, 300));

		addComponent(messageCanvas, canvasPanel, 0, 0, 1, 1, 1, 0.135);
		addComponent(scrollPanel, canvasPanel, 1, 0, 1, 1, 1, 0.865);

		tabbedPane.addTab("Create", null, canvasPanel, "Click to change to Create mode");
		tabbedPane.setSelectedIndex(0);

		// sets the create tab as initially selected tab
		tabbedPane.addTab("Solve", null, emptyPanel2, "Click to change to Solve mode");
		tabbedPane.addChangeListener(this);
		tabbedPane.setBackground(Color.lightGray);

		JPanel toolBarPanel = new JPanel(new BorderLayout());
		JPanel main = new JPanel(new BorderLayout());

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(main, BorderLayout.CENTER);

		setPromptLabel("Click on an entity to select or drag the mouse to select multiple entities.");

		setJMenuBar(createMenuBar());

		algoText = new JTextArea();
		algoText.setBorder(BorderFactory.createRaisedBevelBorder());
		algoText.setBackground(Color.lightGray);
		algoText.setFont(new Font("arial", Font.BOLD, 12));
		algoText.setEditable(false);
		// Not all appets will use this, so if they do, the applet window should
		// set it to true.
		algoText.setVisible(false);

		toolBar = new JToolBar();

		// only call one of createToolBar and createToolBarWithText
		createToolBarWithText();

		bottomPanel = new JPanel(new BorderLayout());
		tabandbottom = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		tabandbottom.setOneTouchExpandable(true);
		bottomPanel.add(algoText, BorderLayout.NORTH);
		toolBarPanel.add(toolBar, BorderLayout.WEST);

		tabandbottom.setTopComponent(tabbedPane);
		tabandbottom.setBottomComponent(bottomPanel);
		tabandbottom.setResizeWeight(.85);
		tabandbottom.setDividerLocation(.8);

		main.add(toolBarPanel, BorderLayout.NORTH);
		main.add(tabandbottom, BorderLayout.CENTER);

		messageCanvas.setPreferredSize(new Dimension(525, 50));
		centerWindow();

		// pressing enter or spacebar will call event on focused button
		// without this the default is spacebar
		UIManager.getDefaults().put("Button.focusInputMap",
				new UIDefaults.LazyInputMap(new Object[] { "ENTER", "pressed",
						"released ENTER", "released", "SPACE", "pressed",
						"released SPACE", "released" }));

		// Uncomment only if you want to run graphToolKit by itself
		// setVisible(true);
		
	}

	/** sets the applet information */
	private void initializeAppletInfo() {
		appletTitle = "Graph Drawing ToolKit Version 4.2.1";
		appletName = "graphToolKit";
		aboutText = "About this Applet\n\n" + appletTitle + "\n\n"
				+ "Special Thanks to:\n"
				+ "David Poole, Alan Mackworth, Holger Hoos,\n"
				+ "Peter Gorniak, and Cristina Conati";
	}

	/**
	 * Adds a component to the gridbag.
	 */
	protected void addComponent(JComponent c, Container p, int row, int column,
			int width, int height, double wx, double wy) {
		// set gridx and gridy
		gbc.gridx = column;
		gbc.gridy = row;

		// set gridwidth and gridheight
		gbc.gridwidth = width;
		gbc.gridheight = height;

		// set weights
		gbc.weightx = wx;
		gbc.weighty = wy;

		gbl.setConstraints(c, gbc);
		p.add(c);
	}

	protected JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();

		menuBar.add(createFileMenu());
		menuBar.add(createViewMenu());
		menuBar.add(createHelpMenu());

		return menuBar;
	}

	/**
	 * Makes the menu bar.
	 */
	protected JMenu createFileMenu() {
		JMenu m = new JMenu("File");
		m.setMnemonic(KeyEvent.VK_F);

		JMenuItem mItem = new JMenuItem("Create Random Graph");
		mItem.setActionCommand(mItem.getText());
		mItem.addActionListener(this);
		mItem.setMnemonic(KeyEvent.VK_R);
		m.add(mItem);

		mItem = new JMenuItem("Create New Graph");
		mItem.setActionCommand(mItem.getText());
		mItem.addActionListener(this);
		mItem.setMnemonic(KeyEvent.VK_N);
		m.add(mItem);

		mItem = new JMenuItem("Load Sample Problem");
		mItem.setActionCommand(mItem.getText());
		mItem.setMnemonic(KeyEvent.VK_G);
		mItem.addActionListener(this);
		m.add(mItem);

		boolean hasLocalAccess = hasLocalAccess();
		if (hasLocalAccess) {
			mItem = new JMenuItem("Load From File");
			mItem.setActionCommand(mItem.getText());
			mItem.setMnemonic(KeyEvent.VK_F);
			mItem.setDisplayedMnemonicIndex(10);
			mItem.addActionListener(this);
			m.add(mItem);
		}
		// applets can't connect to hosts on the internet other than the
		// originating host
		// so we should only allow this option when runnint as an application -
		// ry Nov2005
		mItem = new JMenuItem("Load From URL");
		mItem.setActionCommand(mItem.getText());
		mItem.setMnemonic(KeyEvent.VK_U);
		mItem.addActionListener(this);
		m.add(mItem);

		m.addSeparator();
		if (hasLocalAccess) {
			mItem = new JMenuItem("Save Problem");
			mItem.setActionCommand(mItem.getText());
			mItem.addActionListener(this);
			mItem.setMnemonic(KeyEvent.VK_S);
			m.add(mItem);
		}

		mItem = new JMenuItem("Print");
		mItem.setActionCommand(mItem.getText());
		mItem.addActionListener(this);
		mItem.setMnemonic(KeyEvent.VK_P);
		m.add(mItem);

		mItem = new JMenuItem("Quit");
		mItem.setActionCommand(mItem.getText());
		mItem.addActionListener(this);
		mItem.setMnemonic(KeyEvent.VK_Q);
		m.add(mItem);

		return m;
	}

	protected JMenu createViewMenu() {
		ButtonGroup group = new ButtonGroup();

		JMenu menu = new JMenu("View");
		menu.setMnemonic(KeyEvent.VK_V);

		JMenu fontMenu = new JMenu("Font Size");
		fontMenu.setMnemonic(KeyEvent.VK_F);
		extraLargeFont = new JRadioButtonMenuItem("Extra Large (30 pt)", false);
		extraLargeFont.addActionListener(this);
		fontMenu.add(extraLargeFont);
		group.add(extraLargeFont);
		largeFont = new JRadioButtonMenuItem("Large (20 pt)", false);
		largeFont.addActionListener(this);
		fontMenu.add(largeFont);
		group.add(largeFont);
		mediumFont = new JRadioButtonMenuItem("Medium (12 pt)", true);
		mediumFont.addActionListener(this);
		fontMenu.add(mediumFont);
		group.add(mediumFont);
		smallFont = new JRadioButtonMenuItem("Small (9 pt)", false);
		smallFont.addActionListener(this);
		fontMenu.add(smallFont);
		group.add(smallFont);
		otherFont = new JRadioButtonMenuItem("Other...", false);
		otherFont.addActionListener(this);
		fontMenu.add(otherFont);
		group.add(otherFont);
		menu.add(fontMenu);

		group = new ButtonGroup();
		JMenu line = new JMenu("Line Width");
		line.setMnemonic(KeyEvent.VK_L);
		wideLine = new JRadioButtonMenuItem("Wide (4 pixels)", false);
		wideLine.addActionListener(this);
		line.add(wideLine);
		group.add(wideLine);

		mediumLine = new JRadioButtonMenuItem("Medium (2 pixels)", false);
		mediumLine.addActionListener(this);
		line.add(mediumLine);
		group.add(mediumLine);

		narrowLine = new JRadioButtonMenuItem("Narrow (1 pixel)", true);
		narrowLine.addActionListener(this);
		line.add(narrowLine);
		group.add(narrowLine);
		menu.add(line);

		menu.addSeparator();

		JMenuItem mItem = new JMenuItem("Autoscale");
		mItem.setActionCommand(mItem.getText());
		mItem.setMnemonic(KeyEvent.VK_A);
		mItem.addActionListener(this);
		menu.add(mItem);

		group = new ButtonGroup();
		JMenu zp = new JMenu("Pan/Zoom");
		zp.setMnemonic(KeyEvent.VK_Z);
		zoom = new JRadioButtonMenuItem("Zoom", true);
		zoom.addActionListener(this);
		group.add(zoom);
		zp.add(zoom);

		pan = new JRadioButtonMenuItem("Pan", false);
		pan.addActionListener(this);
		zp.add(pan);
		group.add(pan);
		menu.add(zp);

		mItem = new JMenuItem("Reset Labels");
		mItem.setActionCommand(mItem.getText());
		mItem.setMnemonic(KeyEvent.VK_R);
		mItem.addActionListener(this);
		menu.add(mItem);

		menu.addSeparator();

		JCheckBoxMenuItem checkItem = new JCheckBoxMenuItem("Enable Anti-Aliasing", true);
		checkItem.setMnemonic(KeyEvent.VK_A);
		checkItem.setDisplayedMnemonicIndex(7);
		checkItem.addActionListener(this);
		menu.add(checkItem);

		checkItem = new JCheckBoxMenuItem("Show Message Panel", true);
		checkItem.setMnemonic(KeyEvent.VK_M);
		checkItem.addActionListener(this);
		menu.add(checkItem);

		checkItem = new JCheckBoxMenuItem("Show Button Text", true);
		checkItem.setMnemonic(KeyEvent.VK_T);
		checkItem.setDisplayedMnemonicIndex(12);
		checkItem.addActionListener(this);
		menu.add(checkItem);

		checkItem = new JCheckBoxMenuItem("Show Buttons", true);
		checkItem.setMnemonic(KeyEvent.VK_B);
		checkItem.addActionListener(this);
		menu.add(checkItem);
		return menu;
	}

	/**
	 * create and return the help menu
	 */
	protected JMenu createHelpMenu() {
		if (appletName.equals("graphToolKit") || appletName.equals("")) {
			JMenu menu = new JMenu("Help");
			menu.setMnemonic(KeyEvent.VK_H);

			JMenuItem mItem = new JMenuItem("About AIspace");
			mItem.setMnemonic(KeyEvent.VK_C);
			mItem.setActionCommand(mItem.getText());
			mItem.addActionListener(this);
			menu.add(mItem);

			return menu;
		}
		return new HelpMenu(appletName, this);
	}

	/**
	 * This is to be used by the AboutBox.java class to return the aboutText
	 * string for each applet.
	 */
	public String getAboutText() {
		return aboutText;
	}

	public JApplet getApplet() {
		return applet;
	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	public static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = GraphWindow.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		System.err.println("Couldn't find file: " + path);
		return null;
	}

	protected void createToolBar() {
		toolBar.removeAll();

		// /create tool bar
		createModeButGroup = new ButtonGroup();
		createButModes = new JToggleButton[7];
		setCreateButtons(0, createNodeIcon, "Create Node");
		setCreateButtons(1, createEdgeIcon, "Create Edge");
		setCreateButtons(2, selectIcon, "   Select   ");
		setCreateButtons(3, deleteIcon, "   Delete   ");
		setCreateButtons(4, setPropsIcon, "Set Properties");

		createToolBarLayout();
	}

	protected void createToolBarWithText() {
		toolBar.removeAll();

		// /create tool bar
		createModeButGroup = new ButtonGroup();
		createButModes = new JToggleButton[7];
		setCreateButtonsWithText(0, "Create Node", createNodeIcon, "Create Node");
		setCreateButtonsWithText(1, "Create Edge", createEdgeIcon, "Create Edge");
		setCreateButtonsWithText(2, "   Select   ", selectIcon, "Select");
		setCreateButtonsWithText(3, "   Delete   ", deleteIcon, "Delete");
		setCreateButtonsWithText(4, "Set Properties", setPropsIcon, "Set Properties");

		this.createToolBarLayout();
	}
	
	/** set the buttons in the correct layout */
	private void createToolBarLayout() {
		for (int i = 0; i <= 4; i++) {
			createModeButGroup.add(createButModes[i]);
			toolBar.add(createButModes[i]);
			createButModes[i].setFont(toolBarFont);
		}

		algoText.setText(" ");
	}

	/**
	 * create a create button in index num, with icon and toolTip
	 * @param num - the index of the button
	 * @param icon - the ImageIcon of the icon to be associated with the button
	 * @param toolTip - the string tooltip to be associated with the button
	 */
	protected void setCreateButtons(int num, ImageIcon icon, String toolTip) {
		createButModes[num] = new JToggleButton(icon);
		createButModes[num].setActionCommand(toolTip);
		createButModes[num].addActionListener(this);
		createButModes[num].setToolTipText(toolTip);
	}

	/**
	 * create a solve button in index num, with icon and toolTip
	 * @param num - the index of the button
	 * @param icon - the ImageIcon of the icon to be associated with the button
	 * @param toolTip - the string tooltip to be associated with the button
	 */
	protected void setSolveButtons(int num, ImageIcon icon, String toolTip) {
		solveButModes[num] = new JToggleButton(icon);
		solveButModes[num].setActionCommand(toolTip);
		solveButModes[num].addActionListener(this);
		solveButModes[num].setToolTipText(toolTip);
	}

	/**
	 * create a create button in index num, name buttonName, with icon and toolTip
	 * @param num - the index of the button
	 * @param buttonName - the text to show on the button
	 * @param icon - the ImageIcon of the icon to be associated with the button
	 * @param toolTip - the string tooltip to be associated with the button
	 */
	protected void setCreateButtonsWithText(int num, String buttonName, ImageIcon icon, String toolTip) {
		createButModes[num] = new JToggleButton(buttonName, icon);
		createButModes[num].addActionListener(this);
		createButModes[num].setToolTipText(toolTip);

		createButModes[num].setVerticalTextPosition(AbstractButton.BOTTOM);
		createButModes[num].setHorizontalTextPosition(AbstractButton.CENTER);
	}

	/**
	 * create a solve button in index num, name buttonName, with icon and toolTip
	 * @param num - the index of the button
	 * @param buttonName - the text to show on the button
	 * @param icon - the ImageIcon of the icon to be associated with the button
	 * @param toolTip - the string tooltip to be associated with the button
	 */
	protected void setSolveButtonsWithText(int num, String buttonName, ImageIcon icon, String toolTip) {
		solveButModes[num] = new JToggleButton(buttonName, icon);
		solveButModes[num].addActionListener(this);
		solveButModes[num].setToolTipText(toolTip);

		solveButModes[num].setVerticalTextPosition(AbstractButton.BOTTOM);
		solveButModes[num].setHorizontalTextPosition(AbstractButton.CENTER);
	}

	protected void solveToolBar() {
		toolBar.removeAll();
		// solve tool bar
		solveModeButGroup = new ButtonGroup();
		solveButModes = new JToggleButton[9];
	}

	protected void solveToolBarWithText() {
		solveToolBar();
	}

	/**
	 * Creates a random graph with n nodes.
	 */
	private Graph randomGraph(int numNodes) {
		Graph graph = new Graph(returnCanvas());
		for (int i = 0; i < numNodes; i++) {
			String label = Integer.toString((int) (200 * Math.random()));
			int x = 150 - (int) (300.0 * Math.random());
			int y = 150 - (int) (300.0 * Math.random());

			int shape = GraphConsts.OVAL;
			Point p = new Point(x, y);
			Color color = Color.black;
			if (i % 4 == 0) {
				color = Color.red;
				shape = GraphConsts.RECT;
			} else if (i % 4 == 1) {
				color = Color.blue;
				shape = GraphConsts.DIAMOND;
			} else if (i % 4 == 2) {
				color = Color.green;
				shape = GraphConsts.OVAL;
			}
			if (i == numNodes - 1)

				p = new Point(0, 0);
			Node n = new Node(graph, label, p, color, shape);
			graph.addNode(n);
		}
		for (int i = 0; i < numNodes - 1; i++) {
			String label = "Edge #" + i;
			Node start = graph.nodeAt(i);
			Node end = graph.nodeAt(numNodes - 1);
			Color color = Color.black;
			if (i % 4 == 0)
				color = Color.red;
			else if (i % 4 == 1)
				color = Color.blue;
			else if (i % 4 == 2)
				color = Color.green;
			Edge e = new Edge(graph, label, start, end, color,
					GraphConsts.DIRECTIONAL);
			graph.addEdge(e);
		}

		return graph;
	}

	// Note to sal: this use to return a GraphCanvas
	protected void getCanvas() {
		canvas = new GraphCanvas(this, false);
		canvas.setPreferredSize(new Dimension(15000, 10000));

		scrollPanel = new JScrollPane(canvas,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPanel.setPreferredSize(new Dimension(500, 400));

		scrollPanel.getHorizontalScrollBar().setMaximum(15000);
		scrollPanel.getHorizontalScrollBar().setValue(7000);

		scrollPanel.getVerticalScrollBar().setMaximum(10000);
		scrollPanel.getVerticalScrollBar().setValue(5000);
	}

	/**
	 * returns the graphcanvas
	 */
	public final GraphCanvas returnCanvas() {
		return canvas;
	}

	/**
	 * set the horizontal scroll bar to specified value
	 * 
	 * @param horzPosition -
	 *            value to move the scrollbar to
	 */
	public void setHorizontalBarPosition(int horzPosition) {
		scrollPanel.getHorizontalScrollBar().setValue(horzPosition);
	}

	/**
	 * set the vertical scroll bar to specified value
	 * 
	 * @param vertPosition -
	 *            value to move the scrollbar to
	 */
	public void setVerticalBarPosition(int vertPosition) {
		scrollPanel.getVerticalScrollBar().setValue(vertPosition);
	}

	/**
	 * Invoked when a new tab is selected.
	 */
	public void switchTab(String str) {
		int tempSaveIndex = 0;
		if (str == "Create") {
			tempSaveIndex = this.getSelectedButtonIndex(solveButModes);
			setCreate();
			if(otherSavedButtonIndex != -1) {
				actionPerformed(new ActionEvent(createButModes[otherSavedButtonIndex], 0, createButModes[otherSavedButtonIndex].getActionCommand()));
				createButModes[otherSavedButtonIndex].setSelected(true);
			}
		} else if (str == "Solve") {
			tempSaveIndex = this.getSelectedButtonIndex(createButModes);
			setSolve();
			if(otherSavedButtonIndex != -1) {
				actionPerformed(new ActionEvent(solveButModes[otherSavedButtonIndex], 0, solveButModes[otherSavedButtonIndex].getActionCommand()));
				solveButModes[otherSavedButtonIndex].setSelected(true);
			}
		}
		otherSavedButtonIndex = tempSaveIndex;
		repaint();
	}

	public int getTabIndex() {
		return tabbedPane.getSelectedIndex();
	}

	public void setPanZoom(int num) {
		if (num == 0) {
			zoom.setSelected(true);
		} else {
			pan.setSelected(true);
		}
	}

	public void setCreate() {
		returnCanvas().setMode(GraphConsts.CREATE);
		tabbedPane.setComponentAt(0, emptyPanel1);
		tabbedPane.setComponentAt(1, emptyPanel2);
		tabbedPane.setComponentAt(0, canvasPanel);
		tabbedPane.setSelectedIndex(0);
		// only call one of createToolBar and createToolBarWithText
		if (isButtonTextShowing)
			createToolBarWithText();
		else
			createToolBar();
		createButModes[2].setSelected(true);
		actionPerformed(new ActionEvent(createButModes[2], 0, createButModes[2].getActionCommand()));
	}

	public void setSolve() {
		returnCanvas().setMode(GraphConsts.SOLVE);

		tabbedPane.setComponentAt(0, emptyPanel1);
		tabbedPane.setComponentAt(1, emptyPanel2);
		tabbedPane.setComponentAt(1, canvasPanel);
		tabbedPane.setSelectedIndex(1);

		// only call one of solveToolBar and solveToolBarWithText
		if (isButtonTextShowing) {
			solveToolBarWithText();
		} else
			solveToolBar();
	}

	public void setPromptLabel(String str) {
		messageCanvas.setText(str);
	}

	public String getExactPromptLabel() {
		return messageCanvas.getExactText();
	}

	public void setPromptColor(Color col) {
		messageCanvas.setTextColor(col);
	}

	public String getPromptLabel() {
		return messageCanvas.getText();
	}

	// Save applet properties to be restored again later
	protected void saveProperties() {
		// save font properties
		savedFontSize = returnCanvas().getFont().getSize();
		if (extraLargeFont.isSelected()) {
			savedFontSizeMenuItem = extraLargeFont;
		} else if (largeFont.isSelected()) {
			savedFontSizeMenuItem = largeFont;
		} else if (mediumFont.isSelected()) {
			savedFontSizeMenuItem = mediumFont;
		} else if (smallFont.isSelected()) {
			savedFontSizeMenuItem = smallFont;
		} else if (otherFont.isSelected()) {
			savedFontSizeMenuItem = otherFont;
		}

		// save line properties
		savedLineWidth = returnCanvas().graph.getLineWidth();
		if (wideLine.isSelected()) {
			savedLineWidthMenuItem = wideLine;
		} else if (mediumLine.isSelected()) {
			savedLineWidthMenuItem = mediumLine;
		} else if (narrowLine.isSelected()) {
			savedLineWidthMenuItem = narrowLine;
		}
	}

	// Restore all saved applet properties
	protected void restoreProperties() {
		savedLineWidthMenuItem.setSelected(true);
		returnCanvas().graph.setLineWidth(savedLineWidth);
		returnCanvas().changeLineWidth();

		savedFontSizeMenuItem.setSelected(true);
		returnCanvas().changeFont(savedFontSize);

		if (returnCanvas().getMode() == GraphConsts.CREATE) {
			createButModes[0].setSelected(true);
			returnCanvas().setSubmode(GraphConsts.C_CREATE_NODE);
			setPromptLabel("Click the canvas to create a node.");
		}
	}

	/** returns true if antiAliasing is enabled */
	public boolean isAntiAliasingEnabled() {
		return isAntiAliasingEnabled;
	}

	/** returns true if button text is showing */
	public boolean isButtonTextShowing() {
		return isButtonTextShowing;
	}

	/** returns true if the applet can write to local file system */
	protected boolean hasLocalAccess() {
		java.lang.SecurityManager secMan = System.getSecurityManager();
		try {
			if (secMan != null) {
				// check if you can write to a file, which means user has
				// accepted general
				// policy, and should be allowed to read/write local files
				secMan.checkWrite("C:/temp");
			}
			return true;
		} catch (java.lang.SecurityException e) {
			// user declined security certificate, so dont allow user to
			// read/write local files
			return false;
		}
	}

	/**
	 * override this method in subclasses method to load a sample graph
	 */
	protected void loadSampleGraph() {
		saveProperties();
		new ProblemDialog(this).open();
		restoreProperties();
	}

	/**
	 * method to open an existing graph
	 */
	protected void openGraph() {
		JFileChooser load = new JFileChooser();
		load.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		load.setLocation(0, 0);

		if (load.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File file = load.getSelectedFile();
			fileName = file.getName();
			saveProperties();
			load(file);
			restoreProperties();
		}
	}

	/**
	 * override this method in subclasses method to create a new graph
	 */
	protected void createNewGraph() {
		saveProperties();
		tabbedPane.setSelectedIndex(0);
		returnCanvas().reset();
		returnCanvas().setSubmode(GraphConsts.C_CREATE_NODE);
		createButModes[0].setSelected(true);
		setPromptLabel("Click the canvas to create a node.");
		restoreProperties();
		returnCanvas().repaint();
	}

	/**
	 * method to open a graph from a location (ie url)
	 */
	protected void openLocation() {
		OpenLocationDialog openDialog = new OpenLocationDialog(this);
		if (openDialog.url != null) {
			try {
				fileName = openDialog.url.substring(openDialog.url.lastIndexOf("/") + 1);
				load(new BufferedReader(new InputStreamReader(new URL(openDialog.url).openStream())));
			} catch (Exception exc) {
				showMessage("Error", exc.toString());
			}
		}
	}

	/**
	 * method to save the current graph to a file
	 */
	protected void saveGraph() {
		//System.err.println("Override saveGraph() method from GraphWindow");
		//new MessageDialog(this).open("save title","save msg");
		//new DescriptionDialog(this);
		JFileChooser save = new JFileChooser();
		save.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		save.setLocation(0, 0);

		if (save.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			save(save.getSelectedFile());
			if (fileName != null) {
				setTitle(appletTitle + " --- " + fileName);
			}
		}
	}

	/**
	 * override this method in subclasses
	 * 
	 * @param file -
	 *            the file to be saved
	 */
	protected void save(File file) {
		System.err.println("Override save(File file) method from GraphWindow");
	}

	/** loads in a file from a directory */
	public void load(String dir, String fileName) {
		try {
			this.fileName = fileName;
			if (applet != null) {
				load(new BufferedReader(new InputStreamReader(new URL(applet.getCodeBase() + dir + fileName).openStream())));
			} else {
				load(new BufferedReader(new FileReader(dir + fileName)));
			}
		} catch (Exception e) {
			this.fileName = null;
			showMessage("Error", e.toString());
		}
	}

	/**
	 * only called for opening an existing graph from a local file
	 * 
	 * @param file - the file to be loaded
	 */
	public void load(File file) {
		try {
			this.fileName = file.getName();
			load(new BufferedReader(new FileReader(file.getPath())));
		} catch (Exception e) {
			fileName = null;
			showMessage("Error", e.toString());
		}
	}

	/**
	 * takes in a file, converts it to a single string, and passes it to the
	 * canvas to load. Needs to be Overridden in subclasses.
	 * 
	 * @param in -the bufferedReader to read the file from
	 */
	public void load(BufferedReader in) {
		System.err.println("Override load(BufferedReader in) method from GraphWindow");
	}

	public void setMessageCanvasFontSize(int size) {
		messageCanvas.setFontSize(size);
		messageCanvas.repaint();
	}
	
	/**
	 * Sets the file name.
	 * @param fileName the name of the file.
	 */
	public void setFileName(String fileName)
	{
		this.fileName = fileName;
	}

	//
	// The following implements ActionListener
	//

	/**
	 * Invoked when an action occurs.
	 */
	public void actionPerformed(ActionEvent e) {
		String arg = e.getActionCommand().trim();

		if (arg.equals("Load Sample Problem") || arg.equals("Load From File") 
				|| arg.equals("Load From URL") || arg.equals("Create New Graph") 
				|| arg.equals("Create Random Graph")) {
			// save all applet properties
			saveProperties();

			if (arg.equals("Load Sample Problem")) {
				loadSampleGraph();
			} else if (arg.equals("Load From File")) {
				openGraph();
			} else if (arg.equals("Load From URL")) {
				openLocation();
			} else if (arg.equals("Create New Graph")) {
				createNewGraph();
			} else if (arg.equals("Create Random Graph")) {
				returnCanvas().setGraph(randomGraph(10));
				returnCanvas().repaint();
			} 
			// restore all applet properties
			restoreProperties();
		} else if (arg.equals("Save Problem")) {
			saveGraph();
		} else if (arg.equals("Create Node")) {
			returnCanvas().setSubmode(GraphConsts.C_CREATE_NODE);
			setPromptLabel("Click the canvas to create a node.");
		} else if (arg.equals("Create Edge") || arg.equals("Create Arc")) {
			returnCanvas().setSubmode(GraphConsts.C_CREATE_EDGE);
			String[] namez = arg.split(" ");
			namez[1] = namez[1].toLowerCase();
			setPromptLabel("Click on a node to start creating an " +namez[1] + 
					".\nClick on another node to finish.\nYou can cancel " + namez[1] + 
					" creation by clicking on the canvas.");
		} else if (arg.equals("Select")) {
			returnCanvas().setSubmode(GraphConsts.C_SELECT);
			if (canvas.mode == GraphConsts.SOLVE)
				setPromptLabel("Click on an entity and drag the mouse to move it.");
			else
				setPromptLabel("Click on an entity to select or drag the mouse to select multiple entities.");
		} else if (arg.equals("Delete")) {
			returnCanvas().setSubmode(GraphConsts.C_DELETE);
			setPromptLabel("Click on an entity to delete.");
		} else if (arg.equals("Set Properties")) {
			returnCanvas().setSubmode(GraphConsts.C_SET_PROP);
			setPromptLabel("Click on an entity to set its properties.");
		} else if (arg.equals("Extra Large (30 pt)")) {
			returnCanvas().changeFont(30);
			algoText.setFont(new Font("Arial", Font.BOLD, 30));
		} else if (arg.equals("Large (20 pt)")) {
			returnCanvas().changeFont(20);
			algoText.setFont(new Font("Arial", Font.BOLD, 20));
		} else if (arg.equals("Medium (12 pt)")) {
			returnCanvas().changeFont(12);
			algoText.setFont(new Font("Arial", Font.BOLD, 15));
		} else if (arg.equals("Small (9 pt)")) {
			returnCanvas().changeFont(9);
			algoText.setFont(new Font("Arial", Font.BOLD, 15));
		} else if (arg.equals("Other...")) {
			returnCanvas().changeFont(-1);
		} else if (arg.equals("Wide (4 pixels)")) {
			returnCanvas().graph.setLineWidth(4);
			returnCanvas().changeLineWidth();
		} else if (arg.equals("Medium (2 pixels)")) {
			returnCanvas().graph.setLineWidth(2);
			returnCanvas().changeLineWidth();
		} else if (arg.equals("Narrow (1 pixel)")) {
			returnCanvas().graph.setLineWidth(1);
			returnCanvas().changeLineWidth();
		} else if (arg.equals("Enable Anti-Aliasing")) {
			isAntiAliasingEnabled = !isAntiAliasingEnabled;
			canvas.repaint();
		} else if (arg.equals("Show Message Panel")) {
			isMessagePanelShowing = !isMessagePanelShowing;
			messageCanvas.setVisible(isMessagePanelShowing);
			validate();
			returnCanvas().autoscale();
		} else if (arg.equals("Show Button Text")) {
			isButtonTextShowing = !isButtonTextShowing;
			int selectedIndex;
			if (tabbedPane.getSelectedIndex() == 0) {
				selectedIndex = getSelectedButtonIndex(createButModes);
				if (isButtonTextShowing) {
					createToolBarWithText();
				} else {
					createToolBar();
				}
				if(selectedIndex != -1) {
					createButModes[selectedIndex].setSelected(true);
				}
			} else {
				selectedIndex = getSelectedButtonIndex(solveButModes);
				if (isButtonTextShowing) {
					solveToolBarWithText();
				} else {
					solveToolBar();
				}
				if(selectedIndex != -1) {
					solveButModes[selectedIndex].setSelected(true);
				}
			}
		} else if (arg.equals("Show Buttons")) {
			isToolBarShowing = !isToolBarShowing;
			toolBar.setVisible(isToolBarShowing);
		} else if (arg.equals("Autoscale")) {
			returnCanvas().autoscale();
		} else if (arg.equals("Reset Labels")) {
			returnCanvas().resetLabels();
		} else if (arg.equals("Pan")) {
			returnCanvas().setRMode(GraphConsts.PAN);
			setPromptLabel("Drag the right mouse button to pan the graph.");
		} else if (arg.equals("Zoom")) {
			returnCanvas().setRMode(GraphConsts.ZOOM);
			setPromptLabel("Drag the right mouse button up to zoom in and down to zoom out.");
		} else if (arg.equals("Print")) {
			returnCanvas().print();
			returnCanvas().autoscale();
		} else if (arg.equals("Quit")) {
			setVisible(false);
			dispose();
		}
	}

	/** returns the index of the selected button for the given array of buttons */
	protected int getSelectedButtonIndex(JToggleButton[] buttons) {
		for (int i = 0; (buttons != null) && (i < buttons.length); i++) {
			if (buttons[i] != null && buttons[i].isSelected()) {
               return i;
            }
		}
		return -1;
	}

	//
	// The end of the interface ActionListener
	//

	/**
	 * Displays a dialog box.
	 */
	public final void showMessage(String title, String msg) {
		canvas.showMessage(title, msg);
	}

	/** centers the window in the screen */
	protected final void centerWindow() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Point centerPosition = new Point(screenSize.width / 2, screenSize.height / 2);
		Dimension windowSize = getSize();
		setLocation((int) centerPosition.x - windowSize.width / 2, (int) centerPosition.y - windowSize.height / 2);
	}

	/*
	 * CHANGE LISTENER
	 */
	public void stateChanged(ChangeEvent e) {
		switchTab(tabbedPane.getTitleAt(tabbedPane.getSelectedIndex()));
	}

	//
	// The following methods implement WindowListener
	//
	/**
	 * Invoked when a window has been closed as the result of calling dispose on
	 * the window.
	 */
	public void windowClosed(WindowEvent e) {
		if (applet == null)
			System.exit(0);
	}

	/**
	 * Invoked when the user attempts to close the window from the window's
	 * system menu.
	 */
	public void windowClosing(WindowEvent e) {
		canvas.disposeWindows();
		setVisible(false);
		dispose();
	}

	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}

	//
	// End of interface WindowListener
	//			
			
	public static void main(String[] args) {
		new GraphWindow(null);
	}




}