package AIspace.graphToolKit.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

import AIspace.graphToolKit.GraphWindow;

public abstract class DescriptionDialog extends BasicDialog {
	private static final long serialVersionUID = 1L;
	protected GraphWindow window;
	private boolean editable;
	
	/**
	 * 
	 * @param window - the parent graph window
	 * @param title - the title for the dialog
	 * @param editable - whether the dialog is editable or not.
	 */
	public DescriptionDialog (GraphWindow window, String title, boolean editable) {
		super(window, title, true);
		super.setOKString("Save");
		this.window = window;
		this.editable = editable; 

		setPreferredSize(new Dimension(350, 300));
		pack();
		centerWindow();
	}
	
	public void open() { 
		setVisible(true);
	}
	
	public JPanel constructTopPanel(String sd, JTextPane sEditorPane)
	{		
		JPanel top = new JPanel();

		JLabel label = new JLabel ("Short Description");
		label.setToolTipText("This is an short description of the problem to be included in xml under <SHORT> tag.");
		
		sEditorPane.setPreferredSize(new Dimension(250,100));
		sEditorPane.setText(sd);
		
		if (!editable) {
			sEditorPane.setEnabled(false);
			sEditorPane.setDisabledTextColor(Color.BLACK);
			sEditorPane.setBackground(Color.LIGHT_GRAY);
		}
		
		JScrollPane scrollPane = new JScrollPane(sEditorPane);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setPreferredSize(new Dimension (300, 70));
		
		
		top.add(label, BorderLayout.NORTH);
		top.add(scrollPane, BorderLayout.CENTER);
		top.setPreferredSize(new Dimension(350,100));
		return top;
	}
	
	public JPanel constructCenterPanel(String ld, JTextPane lEditorPane)
	{		
		JPanel center = new JPanel();
		
		JLabel label = new JLabel ("Detailed Description");
		label.setToolTipText("This is a detailed description of the problem to be included in xml under <DETAILED> tag.");
		
		lEditorPane.setPreferredSize(new Dimension(250,100));
		lEditorPane.setText(ld);
		if (!editable) {
			lEditorPane.setEnabled(false);
			lEditorPane.setDisabledTextColor(Color.BLACK);
			lEditorPane.setBackground(Color.LIGHT_GRAY);
		}
		
		JScrollPane scrollPane = new JScrollPane(lEditorPane); 
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setPreferredSize(new Dimension (300, 100));
		
		
		center.add(label, BorderLayout.NORTH);
		center.add(scrollPane, BorderLayout.CENTER);
		
		return center;
	}
	
	public JPanel constructBottomPanel()
	{
		JPanel bottom = new JPanel();
		JButton okButton = new JButton("Save");
		okButton.setActionCommand(okButton.getText());
		okButton.addActionListener(this);
		bottom.add(okButton);
		
		JButton cButton = new JButton("Cancel");
		cButton.setActionCommand(cButton.getText());
		cButton.addActionListener(this);
		bottom.add(cButton);
		
		if (!editable) {
			okButton.setEnabled(false);
			okButton.setVisible(false);
		}
			
		return bottom;
	}
	
	protected String formatString(String s) {
		String formatedDesc = new String();
		int size = (int)(400/9);
		if (s.length() >= size) {
			String tmp = s;
			int num = s.length()/size;;
			
			int p2 = 0;
			
			while ((num >=0) && tmp.length()!=0) {
				p2 = tmp.indexOf(" ", size);
				if (tmp.length()<=size || p2<0)
					p2 =tmp.length()-1;

				formatedDesc = formatedDesc.concat((tmp.substring(0,p2).trim()).concat("\n"));
	
				tmp = tmp.substring(p2+1, tmp.length()).trim();
				num--;
			}
		}
			return formatedDesc;
	}
	
	protected abstract boolean saveDescription ();
	
	protected boolean actionCancel() {
		return true;
	}

	protected boolean actionOK() {
		if(saveDescription()){
			return true;
		} else {
			return false;
		}
	}

}
