/**
 * File: EdgeDialog.java
 * May 12, 2000
 */


package AIspace.graphToolKit.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.GraphCanvas;
import AIspace.graphToolKit.GraphConsts;
import AIspace.graphToolKit.elements.Edge;
import AIspace.graphToolKit.elements.Node;


/**
 * The pop up dialog for edges.
 */
public class EdgeDialog extends BasicDialog  {

	//
	// Fields
	//

	private JComboBox colors;
	private JComboBox types;
	private GraphCanvas parent;
	private Node start, end;
	private JTextField label;
	private boolean creating;
	private Edge edge;


	//
	// Methods
	//

	/**
	 * Constructor.
	 */
	public EdgeDialog( GraphCanvas parent ) {
		super( (JFrame)parent.parent, "Edge Options", true );
		this.parent = parent;

		creating = true;
	  
		colors = new JComboBox();
		colors.addItem( "Black" );
		colors.addItem( "Red" );
		colors.addItem( "Blue" );
		colors.addItem( "Green" );

		types = new JComboBox();
		types.addItem( "Directional" );
		types.addItem( "Bi-directional" );
		types.addItem( "Non-directional" );

		Container contentPane = getContentPane();
		contentPane.setLayout( new GridLayout( 3, 1 ) );

		JPanel panel = new JPanel();
		panel.add( new JLabel( "Label" ) );
		label = new JTextField( "", 10 );
		label.addActionListener( this );
		panel.add( label );
		contentPane.add( panel );

		panel = new JPanel();
		panel.add( colors );
		panel.add( types );
		contentPane.add( panel );

		panel = new JPanel();
		JButton button = new JButton( "OK" );
		button.setActionCommand(button.getText());
		button.addActionListener( this );
		panel.add( button );
		
		button = new JButton( "Cancel" );
		button.setActionCommand(button.getText());
		button.addActionListener( this );
		panel.add( button );
		contentPane.add( panel );
	}


	public void open( Node start, Node end ) {
		creating = true;
		centerWindow();
		this.start = start;
		this.end = end;
	  
		setSize( 265, 150 );
		setVisible( true );
	}


	public void open( Edge e ) {
		creating = false;
		edge = e;
		start = e.start;
		end = e.end;
		label.setText( e.label[0] );
		if( e.color == Color.black ) 
			colors.setSelectedItem( "Black" );
		else if( e.color == Color.red ) 
			colors.setSelectedItem( "Red" );
		else if( e.color == Color.blue ) 
			colors.setSelectedItem( "Blue" );
		else 
			colors.setSelectedItem( "Green" );
		
		if( e.type == GraphConsts.DIRECTIONAL ) {
			types.setSelectedItem( "Directional" );
		} else if( e.type == GraphConsts.BI_DIRECTIONAL ) {
			types.setSelectedItem( "Bi-directional" );
		} else types.setSelectedItem( "Non-directional" );

		centerWindow();
		setVisible( true );
	}

	protected boolean actionOK() {
		parseDialog();
		return true;
	}
	
	protected boolean actionCancel() {
		return true;
	}

	/**
	 * Reades the values from the dialog and pass them to the parent.
	 */
	private void parseDialog() {
		Color color;
		if( colors.getSelectedItem() == "Black" ) 
			color = Color.black;
		else if( colors.getSelectedItem() == "Red" ) 
			color = Color.red;
		else if( colors.getSelectedItem() == "Blue" ) 
			color = Color.blue;
		else 
			color = Color.green;

		int type;
		if( types.getSelectedItem().equals( "Directional" ) ) {
			type = GraphConsts.DIRECTIONAL;
		} else if( types.getSelectedItem().equals( "Bi-directional" ) ) {
			type = GraphConsts.BI_DIRECTIONAL;
		} else {
			type = GraphConsts.NON_DIRECTIONAL;
		}

		if( creating ) {
			Edge e = new Edge( parent.graph, label.getText(), start, end, color, type );
			parent.graph.addEdge( e );
		} else {
			parent.graph.modifyEdge( edge, label.getText(), start, end, color, type );
		}
	}
}