/**
 * File: MessageDialog.java
 * Jun 20, 2000
 */

package AIspace.graphToolKit.dialogs;

import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;


/**
 * MessageDialog is a very simple dialog which shows a text message.
 * @author  Kevin O'Neill & Shinjiro Sueda
 * @version 1.1
 * @since JDK1.1
 */
public class MessageDialog extends BasicDialog {
	//
	// Fields
	//

	private JLabel labelMessage;
	private ActionListener actionListener = null;
	private ActionEvent event;


	/**
	 * Builds the message dialog
	 */
	public MessageDialog( JFrame parent ) {
		super(parent, true, 0);
		//set to have a grid layout
		JPanel mainPanel = new JPanel();
		mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS));

		//add the message
		labelMessage = new JLabel();
		labelMessage.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainPanel.add(labelMessage);

		//add the ok button
		JPanel panel = new JPanel();
		JButton buttonOk = new JButton("OK");
		buttonOk.addActionListener(this);
		this.getRootPane().setDefaultButton(buttonOk);
		panel.add(buttonOk);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainPanel.add(panel);

		this.getContentPane().add(mainPanel);
		event = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "msgBoxClosed" );
	}


	public void open( String title, String msg ) {
		setTitle( title );
		String label = ("<html>"+msg.replaceAll("\r\n|\n|\r", "<br>")+"</html>");
		labelMessage.setText( label );

		super.packCenterOpen();
	}
	
	protected boolean actionOK() {
		processEvent(event);
		return true;
	}
	
	protected boolean actionCancel() {
		return true;
	}

	public synchronized void addActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

	public synchronized void removeActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}

	protected void processEvent(AWTEvent evt) {
		if (evt instanceof ActionEvent) {
			processActionEvent((ActionEvent)evt);
			return;
		}
		super.processEvent(evt);
	}

	protected void processActionEvent(ActionEvent evt) {
		if (actionListener != null) {
			actionListener.actionPerformed(evt);
		}
	}
}