/**
 * File: MessageDialog.java
 * Jun 20, 2000
 */


package AIspace.graphToolKit.dialogs;

import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;


/**
 * MessageDialog is a very simple dialog which shows a text message.
 * @author  Kevin O'Neill & Shinjiro Sueda
 * @version 1.1
 * @since JDK1.1
 */
public class MessageFrame extends BasicFrame implements WindowListener {

	private JLabel labelMessage;
	private ActionListener actionListener = null;
	private ActionEvent event;


	//
	// Methods
	//

	/**
	 * Builds the message dialog
	 */
	public MessageFrame( ) {
		super();
		addWindowListener( this );
		labelMessage = new JLabel();
		JButton buttonOk = new JButton("OK");
		buttonOk.addActionListener( this );
		JPanel p = new JPanel();
		JPanel p2 = new JPanel();
		p.setLayout(new GridLayout(2, 1));
		p.add(labelMessage);
		p2.add(buttonOk);
		p.add(p2);
		getContentPane().add(p);
		event = new ActionEvent( this, ActionEvent.ACTION_PERFORMED, "msgBoxClosed" );
	}

	public void open( String title, String msg ) {
		setTitle( title );
		labelMessage.setText( msg );
		packCenterOpen();
	}
	
	protected boolean actionOK() {
		processEvent(event);
		return true;
	}
	
	protected boolean actionCancel() {
		return true;
	}

	//
	// The following methods implement WindowListener
	//
	public void windowClosing( WindowEvent e ) {
		setVisible( false );
		processEvent( event );
	}

	public void windowActivated( WindowEvent e ) {}
	public void windowClosed( WindowEvent e ) {}
	public void windowDeactivated( WindowEvent e ) {}
	public void windowDeiconified( WindowEvent e ) {}
	public void windowIconified( WindowEvent e ) {}
	public void windowOpened( WindowEvent e ) {}

	//
	// End of interface WindowListener
	//


	public synchronized void addActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

	public synchronized void removeActionListener(ActionListener l) {
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}

	protected void processEvent(AWTEvent evt) {
		if (evt instanceof ActionEvent) {
			processActionEvent((ActionEvent)evt);
			return;
		}
		super.processEvent(evt);
	}

	protected void processActionEvent(ActionEvent evt) {
		if (actionListener != null) {
			actionListener.actionPerformed(evt);
		}
	}
}