/**
* File: NodeDialog.java
* May 25, 2000
*/

package AIspace.graphToolKit.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.GraphCanvas;
import AIspace.graphToolKit.GraphConsts;
import AIspace.graphToolKit.elements.Node;
import AIspace.graphToolKit.elements.Point;

/**
 * The pop up dialog for nodes.
 */
public class NodeDialog extends BasicDialog {

	//
	// Fields
	//
	private JComboBox colors;
	private JComboBox shapes;
	private GraphCanvas canvas;
	private Point pos;
	private JTextField label;
	private boolean creating;
	private Node node;

	//
	// Methods
	//

	/**
	 * Constructor.
	 */
	public NodeDialog(GraphCanvas canvas) {
		super((JFrame) canvas.parent, "Node Options", true);
		this.canvas = canvas;
		this.creating = true;

		colors = new JComboBox();
		colors.addItem("Black");
		colors.addItem("Red");
		colors.addItem("Blue");
		colors.addItem("Green");

		shapes = new JComboBox();
		shapes.addItem("Oval");
		shapes.addItem("Rectangle");
		shapes.addItem("Diamond");

		Container contentPane = getContentPane();
		contentPane.setLayout(new GridLayout(3, 1));

		JPanel panel = new JPanel();
		panel.add(new JLabel("Label"));
		label = new JTextField("", 10);
		label.addActionListener(this);
		panel.add(label);
		contentPane.add(panel);

		panel = new JPanel();
		panel.add(colors);
		panel.add(shapes);
		contentPane.add(panel);

		panel = new JPanel();
		JButton button = new JButton("OK");
		button.setActionCommand(button.getText());
		button.addActionListener(this);
		panel.add(button);
		
		button = new JButton("Cancel");
		button.setActionCommand(button.getText());
		button.addActionListener(this);
		panel.add(button);
		contentPane.add(panel);

		pack();
		setSize(300, 300);
		setResizable(false);
	}

	public void open(Point pos) {
		this.creating = true;
		this.pos = pos;
		centerWindow();

		setVisible(true);
	}

	public void open(Node n) {
		this.creating = false;
		this.node = n;
		this.pos = n.pos;
		label.setText(n.label[0]);
		if (n.color == Color.black)
			colors.setSelectedItem("Black");
		else if (n.color == Color.red)
			colors.setSelectedItem("Red");
		else if (n.color == Color.blue)
			colors.setSelectedItem("Blue");
		else
			colors.setSelectedItem("Green");

		if (n.shape == GraphConsts.OVAL)
			shapes.setSelectedItem("Oval");
		else if (n.shape == GraphConsts.RECT)
			shapes.setSelectedItem("Rectangle");
		else
			shapes.setSelectedItem("Diamond");
		centerWindow();
		setVisible(true);
	}

	protected boolean actionOK() {
		parseDialog();
		return true;
	}
	
	protected boolean actionCancel() {
		return true;
	}

	/**
	 * Reades the values from the dialog and pass them to the parent.
	 */
	private void parseDialog() {
		Color color;
		if (colors.getSelectedItem() == "Black")
			color = Color.black;
		else if (colors.getSelectedItem() == "Red")
			color = Color.red;
		else if (colors.getSelectedItem() == "Blue")
			color = Color.blue;
		else
			color = Color.green;

		int shape;
		if (shapes.getSelectedItem() == "Oval")
			shape = GraphConsts.OVAL;
		else if (shapes.getSelectedItem() == "Rectangle") {
			shape = GraphConsts.RECT;
		} else
			shape = GraphConsts.DIAMOND;

		if (creating) {
			Node n = new Node(canvas.graph, label.getText(), pos, color, shape);
			canvas.graph.addNode(n);
		} else {
			canvas.graph.modifyNode(node, label.getText(), pos, color, shape);
		}
	}
}