/**
 * May 14, 2001
 */

package AIspace.graphToolKit.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import AIspace.graphToolKit.GraphWindow;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.LinkedHashMap;

/**
 * a dialog that allows users to choose which sample search graph to load
 * 
 * @author: Leslie Tung & Shinjiro Sueda
 * rewritten to use hashmaps, and a better overall structure - Kyle Porter
 */
public class ProblemDialog extends BasicDialog implements ListSelectionListener{
	private static final long serialVersionUID = 1L;
	protected GraphWindow window;
	protected LinkedHashMap<String, String> problemNameToFileName;
	protected LinkedHashMap<String, String> sampleProblemDescription;
	protected JList list;
	protected boolean errorOccurredOnLoad = false;
	protected String myDesc;
	protected boolean longDesc = false;
	protected MessageDialog md;
	protected BufferedReader in;
	/**
	 * construct the dialog box
	 */
	public ProblemDialog (GraphWindow window) {
		super(window, "Load Sample Problem", true);
		super.setOKString("Load");
		this.window = window;
		
		getContentPane().add( constructTopPanel(), BorderLayout.NORTH );
		getContentPane().add( constructCenterPanel(), BorderLayout.CENTER );
		getContentPane().add( constructBottomPanel(), BorderLayout.SOUTH );

		pack();
		setPreferredSize(new Dimension(400, 600));
		centerWindow();
		
	}
	
	/** open the dialog */
	public void open() {
		if(!errorOccurredOnLoad) 
			setVisible(true);
	}
	
	/** construct the top label panel */
	protected JPanel constructTopPanel() {
		JPanel top = new JPanel();
		top.add( new JLabel( "Select a sample problem" ), BorderLayout.CENTER );

		
		return top;
	}
	
	/** create the center comboBox panel */
	protected JPanel constructCenterPanel() {
		//create the hashmap to hold the problem name to file name
		problemNameToFileName = new LinkedHashMap<String, String>();
		//create the hashmap to hold the problem name and description
		sampleProblemDescription = new LinkedHashMap<String, String>();
		
		JPanel middle = new JPanel();
		displayList();
	
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(this);
		list.setLayoutOrientation(JList.VERTICAL);
		list.setSelectedIndex(0);
		
		JScrollPane sPane = new JScrollPane(list);
		sPane.setPreferredSize(new Dimension(250,200));
		sPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		middle.add( sPane, BorderLayout.CENTER);
		return middle;
	}
	
	/** construct the bottom button panel */
	protected JPanel constructBottomPanel() {
		JPanel bottom = new JPanel();
		JButton button = new JButton("Load");
		button.setActionCommand(button.getText());
		button.addActionListener(this);
		bottom.add(button);
		
		button = new JButton("Cancel");
		button.setActionCommand(button.getText());
		button.addActionListener(this);
		bottom.add(button);
		
		return bottom;
	}

	/**
	 * get the problem list to be displayed in the list
	 */
	protected boolean getProblemList() {
		try {
			String text;
			BufferedReader in = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/SampleProblems.txt")));
			
			int p1;
			int p2;
			//read in the file
			while ((text = in.readLine()) != null) {
				//trim any excess whitespace
				text = text.trim();
				//ignore empty or commented lines
				if ((text.length() > 0) && !text.substring(0, 2).equals("//")) {
					//get the first divider index
					p1 = text.indexOf(",");
					
					//get the second divider index
					p2 = text.indexOf(",", p1+1);
			
					//put the problem name (first) and the file name (second) into the hashmap)
					problemNameToFileName.put(text.substring(0, p1), text.substring(p1 + 1, p2).trim());
			
					if (text.endsWith("."))
					{
						sampleProblemDescription.put(text.substring(0, p1), text.substring(p2 + 1, text.length()).trim());
					}
					else 
						sampleProblemDescription.put(text.substring(0, p1), "");
				}
			}
			in.close();
			return true;
		} catch (Exception e) {
			window.showMessage( "Error", "Error loading problem list: "+e.getLocalizedMessage() + ".");
			return false;
		}
	}
	
	protected void displayList() {
		if( !getProblemList() ) {
			errorOccurredOnLoad = true;
			return;
		}
		
		String [] fileNameList = new String[problemNameToFileName.size()];
		Iterator<String> itr = problemNameToFileName.keySet().iterator();
		
		for (int i =0; i<problemNameToFileName.size(); i++) {
			fileNameList[i] = itr.next();
		}
		
		list = new JList (fileNameList)
		{
			private static final long serialVersionUID = 1L;

			public String getToolTipText(MouseEvent e) {
				int index = list.locationToIndex(e.getPoint());
				Object item = getModel().getElementAt(index);	
				
				Iterator<String> itr = sampleProblemDescription.keySet().iterator();
				
				String desc = null;
				while(itr.hasNext()) {
					String cur = itr.next();
					if(cur.equalsIgnoreCase(item.toString())) {
						desc = formatString(sampleProblemDescription.get(cur));
						desc = ("<html>"+ desc.replaceAll("\r\n|\n|\r", "<br>")+"</html>");
						break;
					} 
				}
				return desc;
			}
		};
	}

	/**
	 * try to load the URL file
	 */
	protected void loadProblem() {
		if (list.getSelectedValue() != null ) {
			window.setFileName(problemNameToFileName.get(list.getSelectedValue()));
			window.load(new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/saves/"+problemNameToFileName.get(list.getSelectedValue())))));
			
		} else {
			window.showMessage("Error", "Error loading file.");
		}
	}

	protected String formatString(String s) {
		String formatedDesc = new String();
		int size = 50;
		if (s.length() >= size) {
			String tmp = s;
			int num = s.length()/size;;
			
			int p2 = 0;
			
			while ((num >=0) && !tmp.equals("")) {
				p2 = tmp.indexOf(" ", size);
				if (tmp.length()<=size || p2<0)
					p2 =tmp.length()-1;

				formatedDesc = formatedDesc.concat((tmp.substring(0,p2).trim()));
				tmp = tmp.substring(p2+1, tmp.length()).trim();
				
				if (!tmp.trim().equals(""))
					formatedDesc = formatedDesc + "\n";
				
				num--;
			}
			
			formatedDesc+=".";
		}
		else
		{
			formatedDesc = s;
		}
		return formatedDesc;
	}

	protected boolean actionOK() {
		loadProblem();
		return true;
	}
	
	protected boolean actionCancel() {
		return true;
	}

	public void showMessage(String title, String msg) {
			md = new MessageDialog((JFrame) window);
			md.addActionListener(this);
			md.open(title, msg);
	}

	public void valueChanged(ListSelectionEvent e) {		
	}		
}