/**
 * File: Entity.java
 * Jun 12, 2000
 */


package AIspace.graphToolKit.elements;

import java.awt.Color;
import java.awt.Graphics;

import AIspace.graphToolKit.Graph;
import AIspace.graphToolKit.GraphConsts;


public abstract class Entity {
	//
	// Fields
	//

	/** the label of the entity, label[0] is the name */
	public String[] label;
	/** true if the entity is selected */
	public boolean isSelected;
	/** true if the entity is bold */
	public boolean isBold;
	/** the color of the entity */
	public Color color;
	/** the type of entity (GraphConsts.NODE, GraphConsts.EDGE, GraphConsts.EDGE_LABEL) */ 
	public int type;
	/** extra width for this entity */
	public int xw;
	
	/** the parent graph */
	protected Graph graph;
	//
	// Methods
	//

	/**
	 * Default Constructor
	 */
	public Entity( Graph graph ) {
		this.graph = graph;
		label = new String[2];
		label[0] = "";
		label[1] = "";
		isSelected = false;
		isBold = false;
		color = Color.black;
		type = GraphConsts.NODE;
		xw = 0;
	}

	public String getLabel() {
		return label[0];
	}

	public void setLabel( String l ) {
		label[0] = l;
	}
	
	public final Color getColor() {
		return color;
	}
	
	public final void setColor( Color col){
		color = col;
	}

	//
	// Abstract Methods
	//

	public abstract void draw( Graphics g, boolean moving );
	public abstract void drawStructure( Graphics g, boolean moving);
	public abstract boolean contains( Point p );
	public abstract boolean isInRect( Point a, Point b );
	public abstract void translate( float dx, float dy );
	public abstract void move( Point p );
	public abstract boolean equals( Entity ent );
	public abstract String toString();
}