/**
 * File: Node.java
 * May 26, 2000
 */


package AIspace.graphToolKit.elements;

public class Point {
	public float x;
	public float y;

	/** default constructor */
	public Point() {
		this.x = 0;
		this.y = 0;
	}

	/**
	 * constructor
	 * @param x - the x position of the point
	 * @param y - the y position of the point
	 */
	public Point( float x, float y ) {
		this.x = x;
		this.y = y;
	}

	/**
	 * constructor
	 * @param p - the point to create the new point at
	 */
	public Point( Point p ) {
		this.x = p.x;
		this.y = p.y;
	}

	public void translate( float dx, float dy ) {
		x += dx;
		y += dy;
	}

	public void translate( Point p ) {
		x += p.x;
		y += p.y;
	}

	public void move( float x, float y ) {
		this.x = x;
		this.y = y;
	}

	public void move( Point p ) {
		x = p.x;
		y = p.y;
	}

	/**
	 * Returns a negative number if this point is on left of the 
	 * line a -> b.  Returns a positive number if this point is on 
	 * right of a -> b.  Returns 0 if this point is on a -> b.
	 */
	public float leftOf( Point a, Point b ) {
		return (x*(a.y - b.y) - y*(a.x - b.x) + (a.x*b.y - b.x*a.y));
	}

	/**
	 * Scales this point.
	 */
	public void scale( float scale ) {
		x *= scale;
		y *= scale;
	}

	/**
	 * tests for equality of x and y points
	 * @param x - the x coord to test against
	 * @param y - the y coord to test against
	 * @return - true if both are the same
	 */
	public boolean equals( float x, float y ) {
		return (this.x == x && this.y == y);
	}

	/**
	 * tests for equality between the point
	 * @param p - the point to test against this point
	 * @return - true if x and y coords are the same
	 */
	public boolean equals( Point p ) {
		return x == p.x && y == p.y;
	}

	public String toString() {
		return (int)x + "/" + (int)y;
	}
}