package AIspace.graphToolKit.help;

// import java.awt.BorderLayout;
// import java.awt.Dimension;
// import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener; // import java.io.IOException;
import java.awt.event.KeyEvent;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem; // import javax.swing.JScrollPane;
import javax.swing.JOptionPane; // import javax.swing.event.HyperlinkEvent;
// import javax.swing.event.HyperlinkListener;

import AIspace.graphToolKit.GraphWindow;
import AIspace.graphToolKit.dialogs.AboutDialog;
// import AIspace.graphToolKit.elements.Point;

/**
 * @author Mike Pavlin and Jacek Kisynski
 * 
 * 3rd party code by Dem Pilafian {@link HelpMenu#openBrowserHelp(String)}.
 * 
 * Class for creation of standard AIspace help menu.
 */
public class HelpMenu extends JMenu implements ActionListener {

	private static final long serialVersionUID = 1L;

	private static final String aboutAIspaceURL = "http://www.aispace.org/";

	private String appletName;

	private JFrame parent;

	protected JEditorPane htmlPane;

	/**
	 * Constructs the help menu.
	 * 
	 * @param name - the unofficial name (name of package applet is in)
	 * @param parent - parent frame for the menu
	 */
	public HelpMenu(String name, JFrame parent) {
		super("Help");
		this.appletName = name;
		this.parent = parent;
		this.setMnemonic(KeyEvent.VK_H);

		/* Build help menu. */
		JMenuItem mi = new JMenuItem("Legend for Nodes/Edges");
		mi.setMnemonic(KeyEvent.VK_L);
		mi.setActionCommand(mi.getText());
		mi.addActionListener((GraphWindow) parent);
		this.add(mi);

		mi = new JMenuItem("Online Help");
		mi.setMnemonic(KeyEvent.VK_H);
		mi.addActionListener(this);
		this.add(mi);

		mi = new JMenuItem("Online Tutorials");
		mi.setMnemonic(KeyEvent.VK_T);
		mi.addActionListener(this);
		this.add(mi);

		this.addSeparator();

		mi = new JMenuItem("About AIspace");
		mi.setMnemonic(KeyEvent.VK_C);
		mi.addActionListener(this);
		this.add(mi);

		mi = new JMenuItem("About this Tool");
		mi.setMnemonic(KeyEvent.VK_A);
		mi.addActionListener(this);
		this.add(mi);
	}

	/**
	 * Bare Bones Browser Launch. Version 1.5 (December 10, 2005)<br>
	 * Supports: Mac OS X, GNU/Linux, Unix, Windows XP.<br>
	 * Public Domain Software -- Free to Use as You Like
	 * 
	 * @author Dem Pilafian (adapted by Jacek Kisynski).
	 * 
	 * @param helptype
	 */
	public void openBrowserHelp(String helptype) {
		String url = aboutAIspaceURL + this.appletName + "/help/" + helptype;
		String errMsg = "Error attempting to launch a web browser";

		String osName = System.getProperty("os.name");
		try {
			if (osName.startsWith("Mac OS")) {
				// Mac OS
				Class<?> fileMgr = Class.forName("com.apple.eio.FileManager");
				java.lang.reflect.Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[]{String.class});
				openURL.invoke(null, new Object[]{url});
			} else if (osName.startsWith("Windows"))
				// Windows
				Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
			else {
				// Assume Unix or Linux
				String[] browsers = {"firefox","opera","konqueror","epiphany","mozilla","netscape"};
				String browser = null;
				for (int count = 0; count < browsers.length && browser == null; count++)
					if (Runtime.getRuntime().exec(new String[]{"which",browsers[count]}).waitFor() == 0)
						browser = browsers[count];
				if (browser == null)
					throw new Exception("Could not find a web browser.");
				Runtime.getRuntime().exec(new String[]{browser,url});
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, errMsg + ":\n" + e.getLocalizedMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "Online Help") {
			// getTabIndex = 0 in create mode; getTabIndex=1 in solve mode
			if (((GraphWindow) this.parent).getTabIndex() == 0)
				openBrowserHelp("general.shtml#create");
			else
				openBrowserHelp("general.shtml#solve");

		} else if (e.getActionCommand() == "Online Tutorials") {
			openBrowserHelp("tutorials.shtml");
		} else if (e.getActionCommand() == "About AIspace") {
			new AboutDialog(this.parent, true);
		} else if (e.getActionCommand() == "About this Tool") {
			new AboutDialog(this.parent, false);
		}
	}

	// /**
	// * Does the opening of help files... fine if we are using an applet over the net but what to do if need to handle locally... worry about it
	// later
	// * I guess
	// *
	// */
	// public void openBrowserHelp(String title, String helptype) {
	// new MyEditor(title, aboutAIspaceURL + appletName + "/help/" + helptype);
	// }
	//
	// private class MyEditor extends JFrame implements HyperlinkListener {
	// public MyEditor(String title, String urlString) {
	// super(title);
	// try {
	// // Create the HTML viewing pane.
	// htmlPane = new JEditorPane();
	// htmlPane.addHyperlinkListener(this);
	//
	// htmlPane.setPage(urlString);
	// htmlPane.setEditable(false);
	// JScrollPane htmlView = new JScrollPane(htmlPane);
	//
	// htmlView.setMinimumSize(new Dimension(500, 500));
	//
	// getContentPane().add(htmlView, BorderLayout.CENTER);
	// setSize(900, 600); // ANDRE: I am setting the X size to 900 so that the user doesn't need to automatically resize the help to remove
	// // the horizontal scroll bar
	//
	// Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	// Point bottomRightCorner = new Point(screenSize.width, screenSize.height);
	// Dimension windowSize = getSize();
	//
	// // We will get the OS name so that we can do some more sensitive screen creation
	// int helpWindowYStart = (int) (bottomRightCorner.y - windowSize.height);
	// String OS = System.getProperty("os.name");
	// if (!OS.contains("Mac OS")) {
	// helpWindowYStart -= 30; // this will move the window up by 30 pixels. Note: this works only for a majority case of windows and
	// // linux users who
	// } // have the system tray be 1 bar in height.
	//
	// setLocation((int) (bottomRightCorner.x - windowSize.width), helpWindowYStart);// ANDRE: 30 pixels were subtracted to compensate for
	// // the system tray in windows/gnome
	//
	// setVisible(true);
	// } catch (IOException e) {
	// ((GraphWindow) parent).showMessage("Error", "Cannot access URL. You must be "
	// + "connected to the Internet in order to view help documentation.");
	// }
	// }
	//
	// public void hyperlinkUpdate(HyperlinkEvent e) {
	// if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
	// try {
	// htmlPane.setPage(e.getURL());
	// } catch (Throwable t) {
	// t.printStackTrace();
	// }
	// }
	// }
	// }
	//
	// public void actionPerformed(ActionEvent e) {
	// if (e.getActionCommand() == "Online Help") {
	// // getTabIndex = 0 in create mode; getTabIndex=1 in solve mode
	// if (((GraphWindow) parent).getTabIndex() == 0)
	// openBrowserHelp("Online Help", "general.shtml#create");
	// else
	// openBrowserHelp("Online Help", "general.shtml#solve");
	//
	// } else if (e.getActionCommand() == "Online Tutorials") {
	// openBrowserHelp("Online Tutorials", "tutorials.shtml");
	// } else if (e.getActionCommand() == "About AIspace") {
	// new AboutDialog(parent, true);
	// } else if (e.getActionCommand() == "About this Applet") {
	// new AboutDialog(parent, false);
	// }
	// }
}