/*
 * File: InlineCanvas.java
 * July 18th, 2007
 */

package AIspace.neural;

import java.awt.Container;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import AIspace.graphToolKit.GraphConsts;

/**
 * Extends NeuralCanvas to provide functionality for the inline applet.
 * 
 * @author Byron Knoll
 */
public class InlineCanvas extends NeuralCanvas {

	//
	// Fields
	//

	private static final long serialVersionUID = 1L;

	//
	// Methods
	//

	/**
	 * Default Constructor.
	 */
	public InlineCanvas(Container parent) {
		super(parent, true);
		setSize(parent.getSize());
		WIDTH = getSize().width;
		HEIGHT = getSize().height;
		mode = GraphConsts.SOLVE;
	}

	/**
	 * This method is used to initialize the popup menu.
	 * 
	 */
	public void initializePopup() {
		if (!((InlineNeuralApplet) parent).isPopupMenuOn()) {
			// Make the popup menu invisible.
			pop.setPopupSize(0, 0);
		}
	}

	@Override
	protected void popupCanvas( int x, int y ) {
		pop.removeAll();
		pop.setBorder(BorderFactory.createTitledBorder("Solve Options"));
		JMenuItem mItem;
		
		if (((InlineNeuralApplet)parent).parameterInitializationOptionsPopup
				|| ((InlineNeuralApplet)parent).learningOptionsPopup
				|| ((InlineNeuralApplet)parent).stoppingConditionsPopup)
		{
			if (((InlineNeuralApplet)parent).parameterInitializationOptionsPopup)
			{
				mItem = new JMenuItem("Parameter Initialization Options...");
				mItem.addActionListener((InlineNeuralApplet)parent);
				pop.add(mItem);
			}
			if (((InlineNeuralApplet)parent).learningOptionsPopup)
			{
				mItem = new JMenuItem("Learning Options...");
				mItem.addActionListener((InlineNeuralApplet)parent);
				pop.add(mItem);
			}
			if (((InlineNeuralApplet)parent).stoppingConditionsPopup)
			{
				mItem = new JMenuItem("Stopping Conditions...");
				mItem.addActionListener((InlineNeuralApplet)parent);
				pop.add(mItem);
			}
			pop.addSeparator();
		}

		mItem = new JMenuItem("View/Edit Examples");
		mItem.addActionListener(this);
		if (((InlineNeuralApplet)parent).examplesPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Initialize Parameters");
		mItem.addActionListener(this);
		if (((InlineNeuralApplet)parent).initializeParametersPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Step");
		mItem.addActionListener(this);
		if (((InlineNeuralApplet)parent).stepPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Step " + ((NeuralGraph) graph).getNumberOfIterations() + "X");
		mItem.addActionListener(this);
		if (!((InlineNeuralApplet)parent).stepXButton.isEnabled())
		{
			mItem.setEnabled(false);
		}
		if (((InlineNeuralApplet)parent).stepXPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Step to Target Error");
		mItem.addActionListener(this);
		if (!((InlineNeuralApplet)parent).stepToTargetErrorButton.isEnabled())
		{
			mItem.setEnabled(false);
		}
		if (((InlineNeuralApplet)parent).stepToTargetErrorPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Stop");
		mItem.addActionListener(this);
		if (!((InlineNeuralApplet)parent).stopButton.isEnabled())
		{
			mItem.setEnabled(false);
		}
		if (((InlineNeuralApplet)parent).stopPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Calculate Output");
		mItem.addActionListener(this);
		if (((InlineNeuralApplet)parent).calculateOutputPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Show Plot");
		mItem.addActionListener((InlineNeuralApplet)parent);
		if (((InlineNeuralApplet)parent).showPlotPopupOn)
			pop.add(mItem);
		mItem = new JMenuItem("Summary Statistics");
		mItem.addActionListener(this);
		if (((InlineNeuralApplet)parent).summaryStatisticsPopupOn)
			pop.add(mItem);
	
		pop.addSeparator();

		pop.add(new JLabel("View Options"));
		mItem = new JMenuItem("Autoscale");
		mItem.addActionListener(this);
		pop.add(mItem);
		JMenu zp = new JMenu("Pan/Zoom");
		ButtonGroup zpGroup = new ButtonGroup();
		JMenuItem zoom;
		JMenuItem pan;

		if (this.getRMode() == GraphConsts.ZOOM) {
			zoom = new JRadioButtonMenuItem("Zoom", true);
			pan = new JRadioButtonMenuItem("Pan", false);
		} else {
			zoom = new JRadioButtonMenuItem("Zoom", false);
			pan = new JRadioButtonMenuItem("Pan", true);
		}
		zoom.setToolTipText("Zoom into certain area of graph");
		zoom.addActionListener(this);
		zpGroup.add(zoom);
		zp.add(zoom);
		pan.setToolTipText("Pan the graph to a different area on the canvas");
		pan.addActionListener(this);
		zp.add(pan);
		zpGroup.add(pan);
		pop.add(zp);

		mItem = new JMenuItem("Reset Edge Labels");
		mItem.addActionListener(this);
		pop.add(mItem);
		
		if (((InlineNeuralApplet)parent).textRepresentationPopupOn)
		{
			// Add the text representation popup menu items.
			pop.addSeparator();
			mItem = new JMenuItem("View Prolog Code");
			mItem.setActionCommand(mItem.getText());
			mItem.addActionListener((InlineNeuralApplet)parent);
			pop.add(mItem);
			mItem = new JMenuItem("View Text Representation");
			mItem.setActionCommand(mItem.getText());
			mItem.addActionListener((InlineNeuralApplet)parent);
			pop.add(mItem);
			mItem = new JMenuItem("View XML Representation");
			mItem.setActionCommand(mItem.getText());
			mItem.addActionListener((InlineNeuralApplet)parent);
			pop.add(mItem);
		}
		
		pop.show( this, x, y );
	}

	@Override
	public boolean isAntiAliasingEnabled() {
		return true;
	}
	
	/**
	 * Make the popup menu for node
	 */
	protected void popupNode( int x, int y ) {
		popupCanvas( x, y );
	}

	/**
	 * Make the popup menu for edge
	 */
	protected void popupEdge( int x, int y ) {
		popupCanvas( x, y );
	}

	public void centerView() {
		// Do nothing... Just need to override this method.
	}
}