/*
 * File: InlineNeuralApplet.java
 * July 18th, 2007
 */

package AIspace.neural;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import AIspace.graphToolKit.GraphConsts;
import AIspace.graphToolKit.InlineApplet;
import AIspace.graphToolKit.dialogs.MessageDialog;
import AIspace.neural.dialogs.ExampleDialog;
import AIspace.neural.dialogs.InitializationOptionsDialog;
import AIspace.neural.dialogs.InputExampleFrame;
import AIspace.neural.dialogs.LearningOptionsDialog;
import AIspace.neural.dialogs.StoppingOptionsDialog;
import AIspace.neural.dialogs.TestStatisticsDialog;

/**
 * The inline neural applet.
 * 
 * @author Byron Knoll
 */
public class InlineNeuralApplet extends InlineApplet {

	// Fields: 
	private static final long serialVersionUID = 1L;
	private JPanel colorKey;
	
	// Buttons:
	public JToggleButton examplesButton;
	public JToggleButton initializeParametersButton;
	public JToggleButton stepButton;
	public JToggleButton stepXButton;
	public JToggleButton stepToTargetErrorButton;
	public JToggleButton stopButton;
	public JToggleButton calculateOutputButton;
	public JToggleButton showPlotButton;
	public JToggleButton summaryStatisticsButton;
	public JToggleButton fakeButton;

	// Parameter values:
	private boolean examplesButtonOn = false;
	private boolean initializeParametersButtonOn = false;
	private boolean stepButtonOn = false;
	private boolean stepXButtonOn = false;
	private boolean stepToTargetErrorButtonOn = false;
	private boolean stopButtonOn = false;
	private boolean calculateOutputButtonOn = false;
	private boolean showPlotButtonOn = false;
	private boolean summaryStatisticsButtonOn = false;
	
	public boolean examplesPopupOn = false;
	public boolean initializeParametersPopupOn = false;
	public boolean stepPopupOn = false;
	public boolean stepXPopupOn = false;
	public boolean stepToTargetErrorPopupOn = false;
	public boolean stopPopupOn = false;
	public boolean calculateOutputPopupOn = false;
	public boolean showPlotPopupOn = false;
	public boolean summaryStatisticsPopupOn = false;
	public boolean parameterInitializationOptionsPopup = false;
	public boolean learningOptionsPopup = false;
	public boolean stoppingConditionsPopup = false;
	private boolean showColorKey = false;
	
	// Dialog boxes:
	private StoppingOptionsDialog sod;
	private LearningOptionsDialog lod;
	private InitializationOptionsDialog iod;

	/**
	 * Constructor.
	 */
	public void init() {
		super.init();

		makeCanvas(); // Initialize the canvas.

		loadParams(); // Load the parameters.

		// If an incorrect parameter has been used, stop interface creation.
		if (incorrectParameter)
			return;
		
		// Initialize the popup menu.
		((InlineCanvas) canvas).initializePopup();

		createToolBar();

		addComponent((JComponent) messageCanvas, canvasPanel, 0, 0, 1, 1, 1, 0);
		addComponent(canvas, canvasPanel, 1, 0, 1, 1, 1, 1);

		canvas.setMinimumSize(new Dimension(10, 100));

		// Set up the message panel.
		if (!showMessagePanel) {
			messageCanvas.setVisible(false);
		} else {
			messageCanvas.setMinimumSize(new Dimension(10, 50));
			setPromptLabel("Click on a variable to split its domain.\nClick on a "
					+ "constraint to reorder its variables.\nClick on an arc to make it "
					+ "arc-consistent.");
		}

		// Set up a JTextPane which shows the current algorithm selected.
		colorKey = new JPanel();
		colorKey.setBorder(BorderFactory.createRaisedBevelBorder());
		colorKey.setMinimumSize(new Dimension(100, 17));

		canvas.add(colorKey);

		// Set up the splitPane.
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(canvasPanel);
		canvasPanel.addComponentListener(this);

		splitPane.setBottomComponent(colorKey);
		splitPane.setResizeWeight(1);
		splitPane.setDividerSize(0);
		splitPane.setEnabled(false);
		
		
		if (!showColorKey) {
			// Hide the color key.
			colorKey.setVisible(false);
		}
		else
		{
			colorKey.add(new JLabel ("Color Key:"));
			colorKey.add(new Legend());
			splitPane.setResizeWeight(1);
			colorKey.setMinimumSize(new Dimension(30,30));
		}

		// Adjust the size of the panel depending on whether buttons are visible.
		if (showButtons) {
			splitPane.setPreferredSize(new Dimension(this.getSize().width - 10, this
					.getSize().height - 75));
		} else {
			splitPane.setPreferredSize(new Dimension(this.getSize().width - 10, this
					.getSize().height - 10));
			toolBarPanel.setVisible(false);
		}

		// Set up the borders based on the showBorder parameter.
		colorKey.setBorder(null);
		if (showBorder) {
			canvas.setBorder(BorderFactory.createLineBorder(Color.black));
			splitPane.setBorder(BorderFactory.createLineBorder(Color.black));
		} else {
			canvas.setBorder(null);
			splitPane.setBorder(null);
		}

		// Finish setting up the main panel.
		mainPanel.add(splitPane, BorderLayout.CENTER);
		mainPanel.setBackground(Color.white);
		mainPanel.setBorder(null);
		getContentPane().add(mainPanel, BorderLayout.CENTER);

		setPromptLabel("Click on an entity to view its properties.");
		setVisible(true);
	}
	
	/**
	 * Creates the button toolbar.
	 */
	public void createToolBar() {
		toolBarPanel = new JPanel(new BorderLayout());
		JToolBar toolBar = new JToolBar();
		ButtonGroup buttonGroup = new ButtonGroup();
		buttons = new JToggleButton[10];
		int index = 0;
		
		setSolveButtonsWithText(index, "View/Edit Examples",
				createImageIcon("images/neural/Edit24.gif"), "View/Edit Examples");
		examplesButton = buttons[index++];
		if (!examplesButtonOn) {
			examplesButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "Initialize Parameters",
				createImageIcon("images/neural/randomPara.png"), "Initialize Parameters");
		initializeParametersButton = buttons[index++];
		if (!initializeParametersButtonOn) {
			initializeParametersButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "    Step    ",
				createImageIcon("images/neural/step.png"), "Step");
		stepButton = buttons[index++];
		if (!stepButtonOn) {
			stepButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "  Step " + 
				((NeuralGraph) canvas.graph).getNumberOfIterations() + "X  ",
				createImageIcon("images/common/finestep.png"), "Step " + 
				((NeuralGraph) canvas.graph).getNumberOfIterations() + "X");
		stepXButton = buttons[index++];
		if (!stepXButtonOn) {
			stepXButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "Step to Target Error",
				createImageIcon("images/neural/stepTarget.png"), "Step to Target Error");
		stepToTargetErrorButton = buttons[index++];
		if (!stepToTargetErrorButtonOn) {
			stepToTargetErrorButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "    Stop    ",
				createImageIcon("images/neural/stop.gif"), "Stop");
		stopButton = buttons[index++];
		stopButton.setEnabled(false);
		if (!stopButtonOn) {
			stopButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "Calculate Output",
				createImageIcon("images/neural/quiz.gif"), "Calculate Output");
		calculateOutputButton = buttons[index++];
		if (!calculateOutputButtonOn) {
			calculateOutputButton.setVisible(false);
		}
		
		setSolveButtonsWithText(index, "Show Plot",
				createImageIcon("images/neural/showPlot.png"), "Show Plot");
		showPlotButton = buttons[index++];
		if (!showPlotButtonOn) {
			showPlotButton.setVisible(false);
		}
		
		setSolveButtonsWithText(index, "Summary Statistics",
				createImageIcon("images/neural/History24.gif"), "Summary Statistics");
		summaryStatisticsButton = buttons[index++];
		if (!summaryStatisticsButtonOn) {
			summaryStatisticsButton.setVisible(false);
		}

		setSolveButtonsWithText(index, "Fake", createImageIcon("images/common/finestep.png"),
				"Fake"); // This button will never be visible. It is just
		// used to make sure no buttons to appear.
		fakeButton = buttons[index++];

		Font toolFont = new Font("arial", Font.PLAIN, 10);
		for (int i = 0; i < index; i++) {
			buttonGroup.add(buttons[i]);
			toolBar.add(buttons[i]);
			buttons[i].setFont(toolFont);
		}
		buttons[index - 1].setVisible(false); // Don't show the fake button.
		toolBarPanel.add(toolBar, BorderLayout.CENTER);
		mainPanel.add(toolBarPanel, BorderLayout.NORTH);
	}
	
	/**
	 * Initializes the canvas properties.
	 */
	protected void makeCanvas() {
		canvas = new InlineCanvas(this);
		canvas.WIDTH = this.getSize().width;
		canvas.HEIGHT = this.getSize().height;
		canvas.setPreferredSize(new Dimension(canvas.WIDTH, canvas.HEIGHT));
		// Set the applet to solve mode.
		canvas.setMode(GraphConsts.SOLVE);
	}

	protected void loadParams() {
		super.loadParams();
		String param;
		param = getParameter("viewEditExamplesButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				examplesButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("viewEditExamplesButton", param);
		}

		param = getParameter("initializeParametersButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				initializeParametersButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("initializeParametersButton", param);
		}

		param = getParameter("stepButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepButton", param);
		}

		param = getParameter("stepXButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepXButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepXButton", param);
		}

		param = getParameter("stepToTargetErrorButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepToTargetErrorButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepToTargetErrorButton", param);
		}

		param = getParameter("stopButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stopButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stopButton", param);
		}

		param = getParameter("calculateOutputButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				calculateOutputButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("calculateOutputButton", param);
		}

		param = getParameter("showPlotButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				showPlotButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("showPlotButton", param);
		}

		param = getParameter("summaryStatisticsButton");
		if (param != null)
		{
			if (param.equals("on"))
			{
				summaryStatisticsButtonOn = true;
				showButtons = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("summaryStatisticsButton", param);
		}
		
		param = getParameter("viewEditExamplesPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				examplesPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("viewEditExamplesPopup", param);
		}
		
		param = getParameter("initializeParametersPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				initializeParametersPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("initializeParametersPopup", param);
		}
		
		param = getParameter("stepPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepPopup", param);
		}
			
		param = getParameter("stepXPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepXPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepXPopup", param);
		}
			
		param = getParameter("stepToTargetErrorPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stepToTargetErrorPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stepToTargetErrorPopup", param);
		}

		param = getParameter("stopPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stopPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stopPopup", param);
		}

		param = getParameter("calculateOutputPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				calculateOutputPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("calculateOutputPopup", param);
		}

		param = getParameter("showPlotPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				showPlotPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("showPlotPopup", param);
		}

		param = getParameter("summaryStatisticsPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				summaryStatisticsPopupOn = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("summaryStatisticsPopup", param);
		}
		
		param = getParameter("showColorKey");
		if (param != null)
		{
			if (param.equals("on"))
			{
				showColorKey = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("showColorKey", param);
		}
		
		param = getParameter("showParameters");
		if (param != null)
		{
			if (param.equals("off"))
			{
				((NeuralGraph) canvas.graph).setShowParam(false);
			}
			else if (!param.equals("on"))
				incorrectParameter("showParameters", param);
		}

		param = getParameter("parameterInitializationOptionsPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				parameterInitializationOptionsPopup = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("parameterInitializationOptionsPopup", param);
		}
		
		param = getParameter("learningOptionsPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				learningOptionsPopup = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("learningOptionsPopup", param);
		}
		
		param = getParameter("stoppingConditionsPopup");
		if (param != null)
		{
			if (param.equals("on"))
			{
				stoppingConditionsPopup = true;
			}
			else if (!param.equals("off"))
				incorrectParameter("stoppingConditionsPopup", param);
		}
		
		param = getParameter("normalizeInputs");
		if (param != null)
		{
			if (param.equals("on"))
			{
				NeuralGraph g = (NeuralGraph) canvas.graph;
				ExampleList ex = g.getExampleList();
				g.standardized = true;
				if ((ex != null || g.getExampleList().getTrainingArrayList().size() > 0)
					&& (g.getInputNodes().size() > 0 && g.getOutputNodes().size() > 0)) {
					g.initBackProp();
				}
			}
			else if (!param.equals("off"))
				incorrectParameter("normalizeInputs", param);
		}
		
		try
		{
			param = getParameter("iterations");
			if (param != null)
			{
				int value = Integer.parseInt(param);
				if (value < 1) value = 50;
				((NeuralGraph)canvas.graph).setNumberOfIterations(value);
			}
		}
		catch (NumberFormatException e) {
			incorrectParameter("iterations", param);
		}
		
		param = getParameter("speed");
		if (param != null) {
			if (param.equals("veryFast"))
				((NeuralGraph) canvas.graph).dt = 0;
			else if (param.equals("fast"))
				((NeuralGraph) canvas.graph).dt = 100;
			else if (param.equals("medium"))
				((NeuralGraph) canvas.graph).dt = 500;
			else if (param.equals("slow"))
				((NeuralGraph) canvas.graph).dt = 1000;
			else
				incorrectParameter("speed", param);
		}
	}

	/**
	 * Loads a preset graph.
	 */
	public void load(String filename) {
		try {
			((NeuralGraph)canvas.graph).disposeWindows();
			String line;
			StringBuffer allText = new StringBuffer();
			BufferedReader in = new BufferedReader(new InputStreamReader(new URL( filename ).openStream()));

			while ((line = in.readLine()) != null) {
				allText.append(line).append("\n");
			}
			in.close();

			ExampleList example = new ExampleList();
			String errorMessage = example.parseString(allText.toString());
			if ((errorMessage.length() > 0) && (!errorMessage.equals("OK"))) {
				new MessageDialog(null).open("Error Loading File", errorMessage);
				return;
			}
			resetCanvas();
			((NeuralCanvas) canvas).parseXML(allText.toString());
			((NeuralGraph) ((NeuralCanvas) canvas).graph).setInputOutputNodes(true);
			if (canvas.getMode()==GraphConsts.SOLVE)
			{
				((NeuralGraph) canvas.graph).setTotalErrorAfterInitialization();
			}
			if (autoscale) {
				canvas.autoscale();
			}
			((NeuralGraph)canvas.graph).repaint();
		} catch (Exception e) {
			e.printStackTrace(System.out);
			super.setPromptLabel("Error" + e.toString());
		}
	}
	
	/** clear any existing graph from the canvas */
	protected void resetCanvas() {
		int speed = ((NeuralGraph) canvas.graph).dt;
		canvas.reset();
		((NeuralGraph) canvas.graph).dt = speed;
	}
	
	/**
	 * * @param bool
	 * sets stepX, step to target to bool and stop to ! bool
	 */
	public void enableButtons(boolean bool) {
		stepXButton.setEnabled(bool);
		stepToTargetErrorButton.setEnabled(bool);
		stopButton.setEnabled(!bool);
	}

	public void actionPerformed( ActionEvent e ) {
		String arg = e.getActionCommand();
		if( arg.equals( "Autoscale" ) ) {
			canvas.autoscale();
		} else if( arg.equals( "Reset Edge Labels" ) ) {
			canvas.resetLabels();
		} else if( arg.equals( "Pan" ) ) {
			canvas.setRMode( GraphConsts.PAN );
		} else if( arg.equals( "Zoom" ) ) {
			canvas.setRMode( GraphConsts.ZOOM );
		} else if( arg.equals( "Step" ) ) {
			((NeuralGraph)canvas.graph).step();
			fakeButton.setSelected(true);
			canvas.repaint();
		} else if( arg.equals( "Step "+((NeuralGraph)canvas.graph).getNumberOfIterations()+"X" )) {
			((NeuralGraph)canvas.graph).autoStep();
			stopButton.setEnabled(true);
		} else if( arg.equals( "Step to Target Error" ) ) {
			((NeuralGraph)canvas.graph).runUntilFinished();
			stopButton.setEnabled(true);
		} else if( arg.equals( "Stop" ) ) {
			((NeuralGraph)canvas.graph).stop();
			stopButton.setEnabled(false);
			fakeButton.setSelected(true);
		} else if( arg.equals( "Initialize Parameters" ) ) {
			((NeuralGraph)canvas.graph).initializeParaValues();
			fakeButton.setSelected(true);
		} else if( arg.equals( "Calculate Output" ) ) {
			new InputExampleFrame((NeuralGraph)canvas.graph, "Calculate Output Values", false);
			fakeButton.setSelected(true);
		} else if( arg.equals( "Show Plot" ) ) {
			((NeuralGraph)canvas.graph).showPlot();
			fakeButton.setSelected(true);
		} else if (arg.equals("Summary Statistics")) {
			new TestStatisticsDialog(null, (NeuralGraph) canvas.graph);
			fakeButton.setSelected(true);
		} else if (arg.equals("View/Edit Examples")) {
			fakeButton.setSelected(true);
			if (validGraph()) {
				new ExampleDialog(this, (NeuralGraph) canvas.graph, ((NeuralGraph) canvas.graph).getExampleList());
			} else {
				canvas.showMessage("View/Edit Examples", "At least one input and output node are needed to manipulate examples.");
			}
		} else if (arg.equals("View Prolog Code")) {
			((NeuralCanvas) canvas).openPrologTextRep();
		} else if (arg.equals("View Text Representation")) {
			((NeuralCanvas) canvas).openTextRep(false);
		} else if (arg.equals("View XML Representation")) {
			((NeuralCanvas) canvas).openXMLTextRep(false);
		} else if (arg.equals("Parameter Initialization Options...")) {
			if (iod == null) {
				iod = new InitializationOptionsDialog(null, (NeuralGraph) canvas.graph, true);
			} else
				iod.open();
			return;
		} 
		else if (arg.equals("Learning Options...")) {
			if (lod == null) {
				lod = new LearningOptionsDialog(null, (NeuralGraph) canvas.graph);
			} else
				lod.open();
			return;
		}
		else if (arg.equals("Stopping Conditions...")) {
			if (sod == null) {
				sod = new StoppingOptionsDialog(null, (NeuralGraph) canvas.graph, "Stopping Conditions");
			} else
				sod.open();

			stepXButton.setText("Step " + ((NeuralGraph) canvas.graph).getNumberOfIterations() + "X");
			stepXButton.setToolTipText("Step " + ((NeuralGraph) canvas.graph).getNumberOfIterations() + "X");
			stepXButton.setActionCommand(stepXButton.getText());

			return;
		}
	}
	
	/**
	 * Helper function for ActionPerformed that checks if the canvas has
	 * a valid graph. If it does, return true, else return false.
	 * The check just looks to see if there is an input node. This
	 * is because input nodes only exist if there is at least 2 nodes
	 * and they are connected
	 */
	private boolean validGraph() {
		NeuralGraph g = (NeuralGraph) canvas.graph;
		if (g.getInputNodes().size() > 0 && g.getOutputNodes().size() > 0) {
			return true;
		}
		return false;
	}
	
	/*
	 * The following are convenience methods that can be called from a webpage:
	 */

	public void viewEditExamples() {
		if (examplesButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(examplesButton, 0,
				"View/Edit Examples"));
	}

	public void initializeParameters() {
		if (initializeParametersButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(
				initializeParametersButton, 0, "Initialize Parameters"));
	}

	public void step() {
		if (stepButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(stepButton, 0,
				"Step"));
	}

	public void stepX() {
		if (stepXButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(stepXButton, 0, 
				"Step " + ((NeuralGraph) canvas.graph).getNumberOfIterations() + "X"));
	}

	public void stepToTargetError() {
		if (stepToTargetErrorButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(stepToTargetErrorButton,
				0,"Step to Target Error"));
	}

	public void stop() {
		if (stopButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(stopButton, 0, "Stop"));
	}
	
	public void calculateOutput() {
		if (calculateOutputButton.isEnabled())
			((InlineCanvas) canvas).actionPerformed(new ActionEvent(calculateOutputButton, 
				0, "Calculate Output"));
	}
	
	public void showPlot() {
		if (showPlotButton.isEnabled())
			actionPerformed(new ActionEvent(showPlotButton, 0, "Show Plot"));
	}
	
	public void summaryStatistics() {
		if (summaryStatisticsButton.isEnabled())
			actionPerformed(new ActionEvent(summaryStatisticsButton, 0, "Summary Statistics"));
	}

	public void autoscale() {
		((InlineCanvas) canvas).actionPerformed(new ActionEvent(fakeButton, 0,
				"Autoscale"));
	}
}