/**
 * NeuralCanvas.java
 *  Updated: Oct 15, 2003
 * Original: Aug 21, 2000
 */

package AIspace.neural;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import AIspace.graphToolKit.GraphCanvas;
import AIspace.graphToolKit.GraphConsts;
import AIspace.graphToolKit.dialogs.MessageDialog;
import AIspace.graphToolKit.elements.Entity;
import AIspace.graphToolKit.elements.Point;
import AIspace.neural.dialogs.CalculateOutputDialog;
import AIspace.neural.dialogs.EdgeDialog;
import AIspace.neural.dialogs.ExampleDialog;
import AIspace.neural.dialogs.NeuralTextFrame;
import AIspace.neural.dialogs.NodeDialog;
import AIspace.neural.dialogs.TestStatisticsDialog;
import AIspace.neural.elements.NeuralEdge;
import AIspace.neural.elements.NeuralNode;

/**
 * NeuralCanvas subclasses CIGraphCanvas, implementing functionality for
 * the graph canvas which is specific to the search applet.
 *
 * This is the main class for handling events from the graph window.
 *
 * @author  Nicole Arksey,Leslie Tung, Shinjiro Sueda
 * @version 1.0
 * @since JDK1.1
 */
public class NeuralCanvas extends GraphCanvas {

	private NodeDialog nd;
	private Container window;

	/**
	* Initializes a new NeuralCanvas
	*
	* @param graph_in        The NeuralGraph to appear in the canvas
	* @param frame_in        The frame in which the canvas appears
	*/
	public NeuralCanvas(Container parent, boolean inline) {
		super(parent, inline);
		window = parent;
		graph = new NeuralGraph(this);
		// Make the popup menu into a heavy weight component so that it displays on top of
		// the color bar.
		pop.setLightWeightPopupEnabled(false);
	}

	protected void popupCanvas(int x, int y) {
		pop.removeAll();

		JMenuItem mItem;

		if (mode == GraphConsts.CREATE) {
			pop.setBorder(BorderFactory.createTitledBorder("Create Options"));

			mItem = new JMenuItem("Create Node");
			//mItem.setActionCommand( mItem.getLabel() );
			mItem.addActionListener(this);
			pop.add(mItem);

			mItem = new JMenuItem("View/Edit Examples");
			//mItem.setActionCommand( mItem.getLabel() );
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);

		} else {
			pop.setBorder(BorderFactory.createTitledBorder("Solve Options"));
			mItem = new JMenuItem("Initialize Parameters");
			mItem.addActionListener(this);
			pop.add(mItem);
			mItem = new JMenuItem("Step");

			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem = new JMenuItem("Step " + ((NeuralGraph) graph).getNumberOfIterations() + "X");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem.setEnabled(!((NeuralWindow)window).isStopEnabled());
			mItem = new JMenuItem("Step to Target Error");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem.setEnabled(!((NeuralWindow)window).isStopEnabled());

			mItem = new JMenuItem("Stop");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem.setEnabled(((NeuralWindow)window).isStopEnabled());

			mItem = new JMenuItem("Calculate Output");

			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem = new JMenuItem("Show Plot");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem = new JMenuItem("Summary Statistics");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
			mItem = new JMenuItem("View/Edit Examples");
			mItem.addActionListener((NeuralWindow)window);
			pop.add(mItem);
		}
		pop.addSeparator();
		pop.add(new JLabel("View Options"));
		mItem = new JMenuItem("Autoscale");
		mItem.addActionListener((NeuralWindow)window);
		pop.add(mItem);
		JMenu zp = new JMenu("Pan/Zoom");
		ButtonGroup zpGroup = new ButtonGroup();
		JMenuItem zoom;
		JMenuItem pan;

		if (this.getRMode() == GraphConsts.ZOOM) {
			zoom = new JRadioButtonMenuItem("Zoom", true);
			pan = new JRadioButtonMenuItem("Pan", false);
		} else {
			zoom = new JRadioButtonMenuItem("Zoom", false);
			pan = new JRadioButtonMenuItem("Pan", true);
		}
		zoom.setToolTipText("Zoom into certain area of graph");
		zoom.addActionListener((NeuralWindow)window);
		zpGroup.add(zoom);
		zp.add(zoom);
		pan.setToolTipText("Pan the graph to a different area on the canvas");
		pan.addActionListener((NeuralWindow)window);
		zp.add(pan);
		zpGroup.add(pan);
		pop.add(zp);

		mItem = new JMenuItem("Reset Labels");
		mItem.addActionListener(this);
		pop.add(mItem);

		pop.show(this, x, y);
	}

	/**
	 * Make the popup menu for node
	 */
	protected void popupNode(int x, int y) {
		pop.setBorder(BorderFactory.createTitledBorder("Node Options"));
		pop.removeAll();

		JMenuItem mItem;

		if (mode == GraphConsts.CREATE) {
			mItem = new JMenuItem("Create Edge");
			mItem.addActionListener(this);
			pop.add(mItem);

			mItem = new JMenuItem("Delete Node");
			mItem.addActionListener(this);
			pop.add(mItem);
			mItem = new JMenuItem("Set Properties of Node");
			mItem.addActionListener(this);
			pop.add(mItem);
			pop.show(this, x, y);
		}
	}

	/**
	 * Make the popup menu for edge
	 */
	protected void popupEdge(int x, int y) {
		pop.removeAll();
		pop.setBorder(BorderFactory.createTitledBorder("Edge Options"));
		JMenuItem mItem;

		if (mode == GraphConsts.CREATE) {
			mItem = new JMenuItem("Set Properties of Edge");
			mItem.addActionListener(this);
			pop.add(mItem);

			mItem = new JMenuItem("Delete Edge");
			mItem.addActionListener(this);
			pop.add(mItem);
			pop.show(this, x, y);
		}
	}

	public void reset() {
		double tempBound = ((NeuralGraph)graph).getInitParamBound();
		double tempValue = ((NeuralGraph)graph).getInitParamValue();
		boolean tempType = ((NeuralGraph)graph).getInitParamType();
		super.reset();
		graph = new NeuralGraph(this);
		((NeuralGraph)graph).setInitParamBound(tempBound);
		((NeuralGraph)graph).setInitParamValue(tempValue);
		((NeuralGraph)graph).setInitParamType(tempType);
	}

	public int print(Graphics g, PageFormat pf, int pi) throws PrinterException {
		if (pi >= 1) {
			return Printable.NO_SUCH_PAGE;
		}
		Point min = graph.getBound()[0];
		Point max = graph.getBound()[1];

		//find middle of graph
		float midX = ((Math.abs(max.x - min.x)) / 2) + min.x;
		float midY = ((Math.abs(max.y - min.y)) / 2) + min.y;

		float graphScale = graph.getScale();
		zoomForPrinting();
		drawPrint((Graphics2D) g);

		graph.setScale(graphScale);
		moveGraphToPoint(new Point(midX, midY));

		return Printable.PAGE_EXISTS;
	}

	/**
	 * Called by the mouse functions.  This method should be overridden in
	 * the subclasses of this class.
	 */
	protected void createNode(Entity ent, Point p) {
		if (ent == null || (ent.type != GraphConsts.NODE)) {
			int graphIndex = graph.getNextIndex();
			NeuralNode node = new NeuralNode((NeuralGraph) graph);
			node.pos = p;
			if (!inline)
				nd = new NodeDialog((JFrame) parent, (NeuralGraph) graph, graphIndex, "Node Properties Dialog", true, node);
			else
				nd = new NodeDialog(null, (NeuralGraph) graph, graphIndex, "Node Properties", true, node);

			if (!nd.isCancelled()) {
				graph.addNode(node);
				((NeuralGraph) graph).setIOParameters();
			}
			repaint();
		}
	}

	/**
	 * Called by the mouse functions.  This method should be overridden in
	 * the subclasses of this class.
	 */
	protected void createEdge(Entity ent, Point p) {
		if (makingEdge == YES) {
			if (ent == null || ent.type != GraphConsts.NODE) {
				makingEdge = NO;
			} else if (ent.type == GraphConsts.NODE) {
				makingEdge = DONE;
				to = (NeuralNode) ent;
				if (!to.equals(from)) {
					((NeuralNode) from).addChild(to);
					((NeuralNode) to).addParent(from);
					NeuralEdge edge = new NeuralEdge((NeuralGraph) graph, from, to);
					edge.setLabel(((NeuralGraph) graph).showParam);
					graph.addEdge(edge);
					
					// Prevent a cycle from being created.
					if (((NeuralGraph) graph).cycleCheck()==true)
					{
						((NeuralGraph) graph).select(edge);
						((NeuralGraph) graph).deleteSelected();
					}
					
					((NeuralGraph) graph).setInputOutputNodes(true);
				}
				makingEdge = NO;
			}
			if (tmpMode != -1)
				submode = tmpMode;
			tmpMode = -1;
		} else if (ent != null && ent.type == GraphConsts.NODE) {
			from = (NeuralNode) ent;
			makingEdge = YES;
			mMovedPos.move(p);
		}
	}

	/**
	 * Called by the mouse functions.
	 */
	protected void setEntProperties(Entity ent) {
		if (ent != null) {
			pause();
			if (inline)
			{
				if (ent.type == GraphConsts.NODE) {
					new NodeDialog(null, (NeuralGraph) graph, ((NeuralNode) ent).index, "Node Properties Dialog", true, (NeuralNode) ent);
					graph.updateNodeSize((NeuralNode) ent);
				} else if (ent.type == GraphConsts.EDGE) {
					new EdgeDialog(null, (NeuralEdge) ent);
				}
			}
			else
			{
				if (ent.type == GraphConsts.NODE) {
					new NodeDialog((JFrame) parent, (NeuralGraph) graph, ((NeuralNode) ent).index, "Node Properties Dialog", true, (NeuralNode) ent);
					graph.updateNodeSize((NeuralNode) ent);
				} else if (ent.type == GraphConsts.EDGE) {
					new EdgeDialog((JFrame) parent, (NeuralEdge) ent);
				}
			}
			unpause();
		}
	}
	
	public String parseText(String all) {
		String errMsg = ((NeuralGraph) graph).parseText(all);
		if (errMsg.equals("OK")) {
			repaint();
			autoscale();
		}
		return errMsg;
	}

	public String parseXML(String all) {
		String errMsg = ((NeuralGraph) graph).parseXMLText(all);
		if (errMsg.equals("OK")) {
			repaint();
		}
		return errMsg;
	}

	/**
	 * @param modify - true if the representation can be modified
	 */
	public void openTextRep(boolean modify) {
		new NeuralTextFrame(this, ((NeuralGraph) graph).generateTextRep(), "Text Representation of the Graph", false, modify);
	}

	/**
	 * @param modify - true if the representation can be modified
	 */
	public void openXMLTextRep(boolean modify) {
		new NeuralTextFrame(this, ((NeuralGraph) graph).generateXMLTextRep(), "XML Text Representation of the Graph", true, modify);
	}
	
	public void openPrologTextRep() {
		new NeuralTextFrame(this, ((NeuralGraph) graph).generatePrologTextRep(), "Prolog Code for the Graph", false, false);
	}
	
	public boolean updateGraphFromText(String allText, boolean isXML) {
		NeuralGraph old = (NeuralGraph) graph;
		graph = new NeuralGraph(this);
		String errMessage;
		if(isXML)
			errMessage = ((NeuralGraph) graph).parseXMLText(allText);
		else
			errMessage = ((NeuralGraph) graph).parseText(allText);
		
		if (errMessage.equals("OK")) {
			((NeuralGraph) graph).setInputOutputNodes(true);
			repaint();
			return true;
		}
		graph = old;
		showMessage("Error", errMessage);
		return false;
	}

	public void mouseClicked(MouseEvent e) {
		super.mouseClicked(e);
		Entity ent = graph.searchEntities(new Point(e.getX(), e.getY()));
		if (e.getModifiers() == MouseEvent.BUTTON3_MASK)
			return;
		if (mode == GraphConsts.SOLVE) {
			if (ent != null) {
				pause();
				if (inline)
				{
					if (ent.type == GraphConsts.NODE) {
						new NodeDialog(null, (NeuralGraph) graph, ((NeuralNode) ent).index, "Node Properties Dialog", false, (NeuralNode) ent);
					} else if (ent.type == GraphConsts.EDGE) {
						new EdgeDialog(null, ((NeuralEdge) ent));
					}
				}
				else
				{
					if (ent.type == GraphConsts.NODE) {
						new NodeDialog((JFrame) parent, (NeuralGraph) graph, ((NeuralNode) ent).index, "Node Properties Dialog", false, (NeuralNode) ent);
					} else if (ent.type == GraphConsts.EDGE) {
						new EdgeDialog((JFrame) parent, ((NeuralEdge) ent));
					}
				}
				unpause();
			}
		}
	}

	protected void aPerformed(ActionEvent e) {
		super.aPerformed(e);

		String arg = e.getActionCommand();
		if (arg.equals("Step")) {
			((NeuralGraph) graph).step();
			repaint();
		} else if (arg.equals("Step " + ((NeuralGraph) graph).getNumberOfIterations() + "X")) {
			((NeuralGraph) graph).autoStep();
			repaint();
		} else if (arg.equals("Step to Target Error")) {
			((NeuralGraph) graph).runUntilFinished();
		} else if (arg.equals("Stop")) {
			((NeuralGraph) graph).stop();
		} else if (arg.equals("Calculate Output")) {
			pause();
			if (!inline) {
				new CalculateOutputDialog((NeuralWindow) parent, (NeuralGraph) graph);
			} else {
				new CalculateOutputDialog(null, (NeuralGraph) graph);
			}
			unpause();
		} else if (arg.equals("Initialize Parameters")) {
			((NeuralGraph) graph).initializeParaValues();
			setPromptLabel("Initialized Parameters");
		} else if (arg.equals("View/Edit Examples")) {
			pause();
			new ExampleDialog(parent, (NeuralGraph) graph, ((NeuralGraph) graph).getExampleList());
			unpause();
		} else if (arg.equals("Summary Statistics")) {
			pause();
			if (inline)
				new TestStatisticsDialog(null, (NeuralGraph) graph);
			else
				new TestStatisticsDialog((NeuralWindow) parent, (NeuralGraph) graph);
			unpause();
		}
		graph.deselectAll();
	}
}