/**
 * PlotFrame.java
 * Aug 15, 2000
 */


package AIspace.neural;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;


/**
 * Plots number of red edges vs number of steps taken.
 * @author Shinjiro Sueda
 */
public class PlotFrame extends JFrame
   implements ActionListener, WindowListener, ComponentListener, ItemListener {

	private NeuralGraph graph;
	private PlotCanvas canvas;
	private GridBagConstraints gbc;
	private GridBagLayout gbl;
	private JCheckBox log;
	public  JButton auto;
	public  JButton run;
	public  JButton stop;


	//
	// Methods
	//

	public PlotFrame( NeuralGraph graph )  {
		super( "Total Error Plot" );
		addWindowListener( this );
		addComponentListener( this );
		setSize( 600, 400 );
		this.graph = graph;

		// Place the widgets
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;

		canvas = new PlotCanvas();

		JButton step = new JButton( "Step" );
		step.addActionListener( this );
		auto = new JButton( "Step "+ graph.getNumberOfIterations()+"X");
		auto.addActionListener( this );
		run = new JButton( "Step To Target Error" );
		run.addActionListener (this);
		stop = new JButton( "Stop" );
		stop.addActionListener( this );
		stop.setEnabled( false );
		JButton initialize = new JButton( "Initialize Parameters" );
		initialize.addActionListener( this );
		JPanel middle = new JPanel();
		middle.add( initialize );
		middle.add( step );
		middle.add( auto );
		middle.add (run);
		middle.add( stop );
     
		
		log = new JCheckBox( "Logarithmic Scale" );
		log.addActionListener( this );
		JButton close = new JButton( "Close" );
		close.addActionListener( this );
		JButton clear = new JButton( "Clear" );
		clear.addActionListener( this );
		JButton print = new JButton( "Print" );
		print.addActionListener( this );
		JPanel bottom = new JPanel();
		bottom.add( log );
		bottom.add( close );
		bottom.add( clear );
		bottom.add( print );

		getContentPane().setLayout( gbl );
		addComponent( canvas, this, 0, 0, 1, 1, 1, 1 );
		addComponent( middle, this, 1, 0, 1, 1, 0, 0 );
		addComponent( bottom, this, 2, 0, 1, 1, 0, 0 );
		setVisible( true );

		canvas.resetSteps( (int)(100*graph.totalError()) );
   }


   	/**
	 * Adds a PlotCanvas to the gridbag.
	 */
	private void addComponent(PlotCanvas c, PlotFrame p, int row, int column,  int width, int height, double wx, double wy){
		// set gridx and gridy
		gbc.gridx = column;
		gbc.gridy = row;
		
		// set gridwidth and gridheight
		gbc.gridwidth = width;
		gbc.gridheight = height;

		// set weights
		gbc.weightx = wx;
		gbc.weighty = wy;

		gbl.setConstraints( c, gbc );
		p.getContentPane().add( c );
	}


	/**
	 * Adds a component to the gridbag.
	 */
	private void addComponent( JComponent c, PlotFrame p, int row, int column, int width, int height, double wx, double wy ) {
		// set gridx and gridy
		gbc.gridx = column;
		gbc.gridy = row;

		// set gridwidth and gridheight
		gbc.gridwidth = width;
		gbc.gridheight = height;

		// set weights
		gbc.weightx = wx;
		gbc.weighty = wy;

		gbl.setConstraints( c, gbc );
		p.getContentPane().add( c );
	}


	/** open the plotfram and redraw the graph */
	public void open() {
		canvas.redraw( graph.getPlotVals(), graph.getTestPlotVals() );
		setVisible( true );
	}


	/** clear the graph plot points and the canvas */
	public void clear() {
		graph.clearPlotPts();
		canvas.clear();
	}

	/** repaint the canvas */
	public void repaint() {
		canvas.paint( graph.getPlotVals(), graph.getTestPlotVals() );
	}


	public void actionPerformed( ActionEvent e ) {
		String arg = e.getActionCommand();
		if( arg.equals( "Logarithmic Scale" ) ) {
			canvas.useLogScale( log.isSelected() );
		} else if( arg.equals( "Step" ) ) {
			graph.step();
			canvas.redraw( graph.getPlotVals(), graph.getTestPlotVals() );
			graph.canvas.repaint();
		} else if( arg.equals( "Step "+graph.getNumberOfIterations()+"X" ) ) {
			graph.autoStep();
		} else if (arg.equals( "Step To Target Error" ) ) {
			 graph.runUntilFinished();
		} else if( arg.equals( "Stop" ) ) {
			graph.stop();
		} else if( arg.equals( "Initialize Parameters" ) ) {
			graph.initializeParaValues();
			((NeuralGraph) graph).setTotalErrorAfterInitialization();
			canvas.redraw( graph.getPlotVals(), graph.getTestPlotVals() );
		} else if( arg.equals( "Redraw" ) ) {
			canvas.redraw( graph.getPlotVals(), graph.getTestPlotVals() );
		} else if( arg.equals( "Close" ) ) {
			setVisible( false );
		} else if( arg.equals( "Clear" ) ) {
			clear();
		} else if( arg.equals( "Print" ) ) {
			try {
				canvas.print( this );
			} catch( Exception exc ) {}
		}
	}


	/*
	 * ITEM LISTENER INTERFACE
	 */
	public void itemStateChanged( ItemEvent e ) {
		String arg = e.getItem().toString();
		if( arg.equals( "Logarithmic Scale" ) ) {
			canvas.useLogScale( log.isSelected() );
		}
	}

	/*
	 * WINDOW LISTENER INTERFACE
	 */
	public void windowClosing( WindowEvent e ) {
		setVisible(false);
	}

	public void windowActivated( WindowEvent e ) {}
	public void windowClosed( WindowEvent e ) {}
	public void windowDeactivated( WindowEvent e ) {}
	public void windowDeiconified( WindowEvent e ) {}
	public void windowIconified( WindowEvent e ) {}
	public void windowOpened( WindowEvent e ) {}

	/*
	 * COMPONENT LISTENER INTERFACE
	 */
	public void componentResized( ComponentEvent e ) {
		canvas.resetSteps();
		repaint();
	}

	public void componentHidden( ComponentEvent e ) {}
	public void componentMoved( ComponentEvent e ) {}
	public void componentShown( ComponentEvent e ) {}
}
