package AIspace.neural.dialogs;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.graphToolKit.elements.Node;
import AIspace.neural.NeuralGraph;
import AIspace.neural.elements.NeuralNode;

/**
 * Dialog which displays the values of nodes in the network after the user has entered 
 * values for the input nodes.
 * @author Byron Knoll
 */
public class CalculateOutputDialog extends BasicDialog {

	private static final long serialVersionUID = 1L;
	
	private JLabel labelInputs[];
	private JTextField textInputs[];
	private Integer indexInputs[];
	private JLabel labelHidden[];
	private JTextField textHidden[];
	private Integer indexHidden[];
	private JLabel labelOutputs[];
	private JTextField textOutputs[];
	private Integer indexOutputs[];
	private JButton button1;
	private JButton button2;
	private JLabel labelError;
	private JPanel p;

	private NeuralGraph graph;
	private int numInputs;
	private int numHidden;
	private int numOutputs;
	private boolean displayHidden;
	
	public CalculateOutputDialog (JFrame parent, NeuralGraph g)
	{
		super(parent, "Calculate Output", true);
	
		graph = g;

		numInputs = numOutputs = numHidden = 0;
		NeuralNode tmpNode;
		Iterator<Node> nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			if ((tmpNode.getNumParentNodes() == 0) && (tmpNode.getNumChildrenNodes() > 0)) {
				numInputs++;
			}
			else if ((tmpNode.getNumChildrenNodes() == 0) && (tmpNode.getNumParentNodes() > 0)) {
				numOutputs++;
			}
			else if ((tmpNode.getNumChildrenNodes() > 0) && (tmpNode.getNumParentNodes() > 0)) {
				numHidden++;
			}
		}
		
		// Only display the value of hidden nodes if there are less than 20 (to reduce clutter)
		displayHidden = true;
		if (numHidden > 20)
		{
			displayHidden = false;
		}

		if (numInputs > 0)
		{
			labelInputs = new JLabel[numInputs];
			textInputs = new JTextField[numInputs];
			indexInputs = new Integer[numInputs];
		}
		
		if (numHidden > 0 && displayHidden)
		{
			labelHidden = new JLabel[numHidden];
			textHidden = new JTextField[numHidden];
			indexHidden = new Integer[numHidden];
		}
				
		if (numOutputs > 0)
		{
			labelOutputs = new JLabel[numOutputs];
			textOutputs = new JTextField[numOutputs];
			indexOutputs = new Integer[numOutputs];
		}

		int countInput = 0;
		int countHidden = 0;
		int countOutput = 0;
		int numNodes = 0;
		String nodeName;
		
		nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			nodeName = tmpNode.getLabel();
			if (nodeName.equals(""))
			{
				nodeName = new String("Node " + tmpNode.getIndex());
			}

			if ((tmpNode.getNumParentNodes() == 0) && (tmpNode.getNumChildrenNodes() > 0))
			{ // The node is an input node.
				labelInputs[countInput] = new JLabel(nodeName);
				textInputs[countInput]  = new JTextField("", 10);
				indexInputs[countInput] = new Integer(tmpNode.getIndex());
				numNodes++;
				countInput++;
			} 
			else if ((tmpNode.getNumChildrenNodes() == 0) && (tmpNode.getNumParentNodes() > 0))
			{ // The node is an output node.
				labelOutputs[countOutput] = new JLabel(nodeName);
				textOutputs[countOutput]  = new JTextField("", 10);
				indexOutputs[countOutput] = new Integer(numNodes);
				countOutput++;
				numNodes++;
			}
			else if ((tmpNode.getNumChildrenNodes() > 0)&&(tmpNode.getNumParentNodes() > 0)&& displayHidden)
			{ // The node is a hidden node.
				labelHidden[countHidden] = new JLabel(nodeName);
				textHidden[countHidden]  = new JTextField("", 10);
				indexHidden[countHidden] = new Integer(numNodes);
				countHidden++;
				numNodes++;
			}
		}

		button1 = new JButton("Calculate");
		button1.addActionListener(this);
		button2 = new JButton("Cancel");
		button2.addActionListener(this);
		labelError = new JLabel("");
		p = new JPanel();
		if (displayHidden)
		{
			p.setLayout(new GridLayout(numInputs + numHidden + numOutputs + 4, 2));
		}
		else
		{
			p.setLayout(new GridLayout(numInputs + numOutputs + 4, 2));
		}

		p.add(new JLabel("Input Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numInputs; i++)
		{
			p.add(labelInputs[i]);
			p.add(textInputs[i]);
		}
		
		if (displayHidden)
		{
			p.add(new JLabel("Hidden Values:"));
			p.add(new JLabel(""));
			for (int i = 0; i < this.numHidden; i++)
			{
				p.add(labelHidden[i]);
				p.add(textHidden[i]);
				textHidden[i].setText("");
				textHidden[i].setEditable(false);
			}
		}

		p.add(new JLabel("Output Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numOutputs; i++)
		{
			p.add(labelOutputs[i]);
			p.add(textOutputs[i]);
			textOutputs[i].setText("");
			textOutputs[i].setEditable(false);
		}
		p.add(button1);
		p.add(button2);
		getContentPane().add(p);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}
	
	public void calculateOutputs()
	{
		try
		{
			String values[] = new String[numInputs];
			for (int i = 0; i < numInputs; i++)
			{
				values[i] = String.valueOf(Double.valueOf(textInputs[i].getText()));
			}

			double value;
			
			if (displayHidden)
			{
				// Find the hidden nodes.
				Iterator<Node> nodeItr = graph.getNodes();
				NeuralNode tmpNode;
				ArrayList<NeuralNode> hiddenNodes = new ArrayList<NeuralNode>();
				while (nodeItr.hasNext())
				{
					tmpNode = (NeuralNode)nodeItr.next();
					if ((tmpNode.getNumChildrenNodes() > 0)&&(tmpNode.getNumParentNodes() > 0))
						hiddenNodes.add(tmpNode);
				}
				
				for (int i = 0; i < numHidden; i++)
				{
					value = Double.valueOf(graph.nodeValue(values, hiddenNodes.get(i)));
					textHidden[i].setText((new Double(value)).toString());
					textHidden[i].setForeground(Color.red);
				}
			}
			
			for (int i = 0; i < numOutputs; i++)
			{
				value = Double.valueOf(graph.outputValue(values, i));
				textOutputs[i].setText((new Double(value)).toString());
				textOutputs[i].setForeground(Color.red);
			}
		}
		catch (NumberFormatException e)
		{
			labelError.setText("Error: invalid number format.");
		}
	}
	
	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return true;
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	
	public void actionPerformed (ActionEvent e) {
		super.actionPerformed(e);
		if (e.getActionCommand() == "Calculate") {
			labelError.setText("");
			calculateOutputs();
		}
	}
}