package AIspace.neural.dialogs;

import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.neural.elements.NeuralEdge;

/**
 * @author Joseph Roy Santos
 * Dialog to change the edge weights
 */
public class EdgeDialog extends BasicDialog {
	private JLabel labelParameters;
	private JTextField textParameters;
	private JButton buttonOk;
	private JButton buttonCancel;
	private JLabel labelError;
	private JPanel p;
	
	private NeuralEdge edge;
	
	//
	// Methods
	//
	
	/**
	 * constructs the dialog box
	 * @param parent - the SearchWindow
	 */
	public EdgeDialog( JFrame parent, NeuralEdge edge) {
		super(parent, "Edge Parameter Dialog", true);
		this.edge = edge;

		labelParameters = new JLabel("Set Weight for Edge");
		textParameters = new JTextField();
		textParameters.setText((new Double(edge.getCurrentParaValue())).toString());

		buttonOk = new JButton("OK");
		buttonOk.addActionListener(this);
		buttonCancel = new JButton("Cancel");
		buttonCancel.addActionListener(this);
		labelError = new JLabel("");
		p = new JPanel();
		p.setLayout(new GridLayout(2, 2));

		p.add(labelParameters);
		p.add(textParameters);
	 
		
		p.add(buttonOk);
		p.add(buttonCancel);
		getContentPane().add(p);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}

	/**
	 * set the node name and parameters in the node object and in the edges
	 * directed to it
	 */
	public boolean setProperties() {
		try {
			double paraValues = Double.parseDouble(textParameters.getText());
		  
			edge.setInitialParaValue(paraValues);
			edge.setCurrentParaValue(paraValues);
			
			return true;
		} catch (NumberFormatException e) {
			labelError.setText("Error: Parameters have invalid real number format!");
			return false;
		}
	}
	
	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return setProperties();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}
}