/** ExampleDialog.java
 * 
 * This dialog class displays a window for manipulating test and training set examples in the decision tree learning
 * and neural networks applets.  Examples can be added, removed, and moved back and forth between the training and test 
 * sets.
 * 
 * @author Wesley Coelho
 * 
 */


package AIspace.neural.dialogs;


import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Hashtable;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import AIspace.graphToolKit.GraphWindow;
import AIspace.neural.ExampleList;
import AIspace.neural.InlineNeuralApplet;
import AIspace.neural.NeuralGraph;
import AIspace.neural.NeuralWindow;

public class ExampleDialog extends JDialog implements ActionListener, WindowListener, ItemListener{
	
	//Member Fields
	
	private JComboBox choiceComboBox;
	private JList targetList;
	protected Container window;
	protected ExampleList exampleList;
	protected JList trainingList;
	protected JList testList;
	protected DefaultListModel trainingModel;
	protected DefaultListModel testModel;
	protected DefaultListModel targetModel;
	private NeuralGraph graph;
	
	protected Font font;
	
	protected GridBagLayout gridBagLayout;
	protected GridBagConstraints gridBagConstraints;
	private ImageIcon rightArrowIcon;
	private ImageIcon leftArrowIcon;
	private JLabel trainingLabel;
	private JLabel testLabel;
	
	/** true if the window container is inline, false otherwise */
	private boolean inline;
	
	/**
	 * Creates and displays a new ExampleDialog.
	 * PRE: The exampleList must have parameters specified.
	 * 
	 * @param parent		The parent window that is creating this dialog
	 * @param exampleList	The exampleList class that serves as a data source for this dialog
	 * 
	 */
	public ExampleDialog(Container parent, NeuralGraph graph,ExampleList exampleList)
	{
		super((parent instanceof NeuralWindow ? (NeuralWindow)parent : null), "Edit Data Set Examples", false);
		this.window = parent;
		this.graph = graph;
		this.exampleList = exampleList;	
		
		leftArrowIcon = createImageIcon("images/neural/arrowRight.png");
		rightArrowIcon = createImageIcon("images/neural/arrowLeft.png");
		gridBagLayout = new GridBagLayout();
		gridBagConstraints = new GridBagConstraints();
      	
		gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		
		if (window instanceof NeuralWindow)
			inline = false;
		else
			inline = true;
		
		if (!inline)
			font = new Font("monospaced", Font.PLAIN, ((NeuralWindow)parent).returnCanvas().getFontSize());
		else
			font = new Font("monospaced", Font.PLAIN, ((InlineNeuralApplet)parent).getCanvas().getFontSize());
		
		targetList = new JList();
		targetList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
	
		targetModel = new DefaultListModel();
		
		targetList.setModel(targetModel);
		addWindowListener(this);	
				
		buildDialog();
		populateLists();
	}
	
	/**
	 * 
	 * Creates and arranges the components of this dialog.
	 * 
	 */
	private void buildDialog()
	{
		JPanel mainPanel = new JPanel();
		
		addComponent(makeTrainingPanel(), mainPanel,		0, 0, 1, 1, 0.5, 0.5);
		addComponent(makeCenterPanel(), mainPanel,			0, 1, 1, 1, 0, 0);				
		addComponent(makeTestPanel(), mainPanel,			0, 2, 1, 1, 0.5, 0.5);
		addComponent(makeBottomPanel(), mainPanel,			1, 2, 1, 1, 0, 0);
		
		mainPanel.setLayout(gridBagLayout);
		mainPanel.setPreferredSize(new Dimension(1000, 270));
		
		getContentPane().add(mainPanel);
		pack();
		centerWindow();
		setVisible(true);
		
	}
	
	/** updates the training label */
	private void updateTrainingLabel(String label) {
		trainingLabel.setText(label);
	}
	
	/** updates the test label */
	private void updateTestLabel(String label) {
		testLabel.setText(label);
	}
	
	/** 
	 * Constructs the left-side panel for the training set
	 */
	private JPanel makeTrainingPanel()
	{
		JPanel trainingPanel = new JPanel(new BorderLayout()); 
		trainingLabel = new JLabel("Training Examples (" + exampleList.getTrainingArrayList().size() + ")");
		trainingModel = new DefaultListModel();
		trainingList = new JList();
		trainingList.setModel(trainingModel);
		trainingList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		trainingList.setFont(font);

		JScrollPane listScroller = new JScrollPane(trainingList);
		
		JPanel trainingPanelButtonPanel = makeTrainingButtonPanel();
		trainingPanel.add(BorderLayout.NORTH, trainingLabel);		
		trainingPanel.add(BorderLayout.CENTER, listScroller);
		trainingPanel.add(BorderLayout.SOUTH, trainingPanelButtonPanel);
		
		return trainingPanel;
	}
	
	/** 
	 * Constructs the right-side panel for the training set
	 */
	private JPanel makeTestPanel()
	{
		JPanel testPanel = new JPanel(new BorderLayout()); 
		testLabel = new JLabel("Test Examples (" + exampleList.getTestArrayList().size() + ")");
	
		testList = new JList();
		testModel = new DefaultListModel();
		testList.setModel(testModel);
		testList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		testList.setFont(font);
		JScrollPane listScroller = new JScrollPane(testList);
	
		JPanel trainingPanelButtonPanel = makeTestButtonPanel();
		testPanel.add(BorderLayout.NORTH, testLabel);
		testPanel.add(BorderLayout.CENTER, listScroller);
		testPanel.add(BorderLayout.SOUTH, trainingPanelButtonPanel);
		
		return testPanel;
	}


	/** 
	 * Constructs a panel of buttons for the Training Set
	 */	
	private JPanel makeTrainingButtonPanel()
	{
		JPanel buttonPanel = new JPanel();
		JButton button = new JButton("Add New");
		button.setActionCommand("add_training");
		button.setToolTipText("Add Training Example");
		button.addActionListener(this);
		buttonPanel.add(button);
		
		button = new JButton("Remove");
		button.setActionCommand("remove_training");
		button.setToolTipText("Remove Training Examples");
		button.addActionListener(this);
		buttonPanel.add(button);	
		
		button = new JButton("Edit");
		button.setActionCommand("edit_training");
		button.setToolTipText("Edit Training Example");
		button.addActionListener(this);
		buttonPanel.add(button);
		
		String[] selectChoice =  { "Select All", "Select None",  "Select % of Examples", "Invert Selection"};
		choiceComboBox = new JComboBox(selectChoice);
		choiceComboBox.setSelectedIndex(1);
		choiceComboBox.setName("TrainingChoice");
		choiceComboBox.addActionListener( this );

		buttonPanel.add(choiceComboBox);
		
		return buttonPanel;
	}
	
	
	/** 
	 * Constructs a panel of buttons for the Test Set
	 */		
	private JPanel makeTestButtonPanel()
	{
		JPanel buttonPanel = new JPanel();
		JButton button = new JButton("Add New");
		button.setActionCommand("add_test");
		button.setToolTipText("Add Test Example");
		button.addActionListener(this);
		buttonPanel.add(button);
		
		button = new JButton("Remove");
		button.setActionCommand("remove_test");
		button.setToolTipText("Remove Test Examples");
		button.addActionListener(this);
		buttonPanel.add(button);
		
		button = new JButton("Edit");
		button.setActionCommand("edit_test");
		button.setToolTipText("Edit Test Example");
		button.addActionListener(this);
		buttonPanel.add(button);
		
		String[] selectChoice = { "Select All", "Select None",  "Select % of Examples", "Invert Selection"};
		choiceComboBox = new JComboBox(selectChoice);
		choiceComboBox.setSelectedIndex(1);
		choiceComboBox.setName("TestChoice");
		choiceComboBox.addActionListener( this );
		buttonPanel.add(choiceComboBox);
				
		return buttonPanel;		
	}

	
	/** 
	 * Constructs the close button panel at the bottom of the window
	 */	
	private JPanel makeBottomPanel()
	{
		JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT) );
		JButton closeButton = new JButton("Close Window");
		closeButton.addActionListener(this);
		bottomPanel.add(closeButton);
		
		return bottomPanel;	
	}


	/** 
	 * Constructs the center panel with the left and right arrow buttons
	 */		
	private JPanel makeCenterPanel()
	{
		JPanel centerPanel = new JPanel(new GridLayout(4,2));

		JButton rightButton = new JButton(leftArrowIcon);
		rightButton.setActionCommand("Move to Test");
		rightButton.addActionListener(this);
		
		JButton leftButton = new JButton(rightArrowIcon);
		leftButton.setActionCommand("Move to Training");
		leftButton.addActionListener(this);

		centerPanel.add(new JLabel()); //Dummy Label used as a filler
		centerPanel.add(rightButton);
		centerPanel.add(leftButton);

		return centerPanel;
	}
	
	public void changeFont(Font newFont){
		font = newFont;
		trainingList.setFont(font);
		testList.setFont(font);
		repaint();
	}
	
	/** 
	 * Fills the training and test set  lists with data from the exampleList
	 * 
	 * @see populateList()
	 */	
	private void populateLists() {
		populateList(trainingList, exampleList.getTrainingArrayList());
		populateList(testList, exampleList.getTestArrayList());
	}
		
	/** 
	 * Fills the  list passed in with the data passed in.  This algorithm was taken from the original
	 * code.  It could be upgraded for better alignment of columns if necessary.
	 * 
	 * @param 	currList	The  list to fill
	 * @param	dataList	A offsetPoint of hashtables from the exampleList that is the data for the  List
	 * 
	 * @see populateLists()
	 */	
	private void populateList(JList currList, ArrayList<Hashtable<Integer, String>> dataList) {
		DefaultListModel currModel =  (DefaultListModel) currList.getModel();
		currList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		//Build Title String, return if no data
		currModel.clear();
		String heading = new String("");
		if (dataList.size() < 1 || exampleList.getNumParameters() < 2) {
			heading = "NO DATA";
		    currModel.addElement(heading);
			return;
		}
		
		//Build heading String
		String[] parameters = exampleList.getParameters();
		heading += "   ";
		int[] colPos = new int[parameters.length]; // Record starting positions of each column
		for (int i = 0; i < exampleList.getNumParameters(); i++) {
			colPos[i] = heading.length(); //Record current position
			heading += parameters[i];
			for (int j = 0; j < 5; j++) {
				heading += " ";
			}
		}

		currModel.addElement(heading);

		//Add Example Rows      
		for (int i = 0; i < dataList.size(); i++) {
			String display = new String((i +1 )+": ");
			Hashtable<Integer, String> oneExample = dataList.get(i);
	
			for (int j = 0; j < exampleList.getNumParameters(); j++) {
				String value = oneExample.get((new Integer(j)));
				if (value != null) {
					while (display.length() < colPos[j]) {
						display += " ";
					}
					display += value;
				}
			}
			currModel.addElement(display);
		}
	}
	
	/** action listener for the buttons */
	public void actionPerformed(ActionEvent e)
	{
		String command = e.getActionCommand();
		if (command.equals("Move to Test")) {
			int[] inds = trainingList.getSelectedIndices();
			if(inds.length == 0)
			 	return;
			inds = adjustIndexes(inds);
			exampleList.moveToTest(inds);
			updateTrainingLabel("Training Examples (" + exampleList.getTrainingArrayList().size() + ")");
			updateTestLabel("Test Examples (" + exampleList.getTestArrayList().size() + ")");
			populateLists();

			//Select the moved items, which are the last items, by iterating back from the end of the list.
			for (int i = testModel.getSize() - 1; i > testModel.getSize() - inds.length - 1; i--) {
				testList.setSelectedIndex(i);
			}
			return;
		} else if (command.equals("Move to Training")) {
			//testList.deselect(0);
			int[] inds = testList.getSelectedIndices();
			if(inds.length == 0)
			 	return;
			inds = adjustIndexes(inds);
			exampleList.moveToTraining(inds);
			updateTrainingLabel("Training Examples (" + exampleList.getTrainingArrayList().size() + ")");
			updateTestLabel("Test Examples (" + exampleList.getTestArrayList().size() + ")");
			populateLists();
		
			//Select the moved items, which are the last items, by iterating back from the end of the list.
			for (int i = trainingModel.getSize() - 1; i > trainingModel.getSize() - inds.length - 1; i--) {
				trainingList.setSelectedIndex(i);
			}
			return;
		} else if(command.equals("remove_training")) {
			 int[] inds = trainingList.getSelectedIndices();
			 if(inds.length == 0)
			 	return;
			 inds = adjustIndexes(inds);
			 
			 exampleList.removeTrainingExamples(inds);
			 updateTrainingLabel("Training Examples (" + exampleList.getTrainingArrayList().size() + ")");
			 populateLists();
			return;
		} else if(command.equals("remove_test")) {
			int[] inds = testList.getSelectedIndices();
			if(inds.length == 0)
			 	return;
			inds = adjustIndexes(inds);
			exampleList.removeTestExamples(inds);
			updateTestLabel("Test Examples (" + exampleList.getTestArrayList().size() + ")");
			populateLists();
			return;
		} else if(command.equals("add_training")) {
			//Don't allow Add unless there are at least 2 parameters (one input and one output)
			//If there aren't enough parameters, open the parameter input box instead.
			if (inline)
			{
				if (exampleList.getNumParameters() < 2) {
					new ParameterInputDialog(null, exampleList);
				} else {
					new InputExampleDialog(null,this, graph, "Input Training Examples");
				}
			}
			else
			{
				if (exampleList.getNumParameters() < 2) {
					new ParameterInputDialog(((NeuralWindow)window), exampleList);
				} else {
					new InputExampleDialog(((NeuralWindow)window),this, graph, "Input Training Examples");
				}
			}
			updateTrainingLabel("Training Examples (" + exampleList.getTrainingArrayList().size() + ")");
			populateLists();
			
			return;
			
		} else if(command.equals("add_test")) {
			//Don't allow Add unless there are at least 2 parameters (one input and one output)
			//If there aren't enough parameters, open the parameter input box instead.
			if (inline)
			{
				if (exampleList.getNumParameters() < 2) {
					new ParameterInputDialog(null, exampleList);
				} else {
					new InputExampleDialog(null,this, graph, "Input Test Examples");
				}
			}
			else
			{
				if (exampleList.getNumParameters() < 2) {
					new ParameterInputDialog(((NeuralWindow)window), exampleList);
				} else {
					new InputExampleDialog(((NeuralWindow)window),this, graph, "Input Test Examples");
				}
			}
			updateTestLabel("Test Examples (" + exampleList.getTestArrayList().size() + ")");
			populateLists();
		
			return;
		} else if(command.equals("Close Window")) {
			setVisible(false);
			return;
		} else if(command.equals("edit_training")) {
			int[] indexes = trainingList.getSelectedIndices();
			if((indexes.length == 0) || (indexes[0] == 0))
			 	return;
			if (inline)
				new InputExampleDialog(null, this, graph, "Edit Training Example");
			else
				new InputExampleDialog(((NeuralWindow)window), this, graph, "Edit Training Example");
			populateLists();
			return;
		} else if(command.equals("edit_test")) {
			int[] indexes = testList.getSelectedIndices();
			if((indexes.length == 0) || (indexes[0] == 0))
			 	return;
			if (inline)
				new InputExampleDialog(null, this, graph, "Edit Test Example");
			else
				new InputExampleDialog(((NeuralWindow)window), this, graph, "Edit Test Example");
			populateLists();
			return;
		}

		String selection = (String) ((JComboBox) e.getSource()).getSelectedItem();
		String sourceName = ((JComboBox)e.getSource()).getName();
	  
		// Set the list that the user wants to select/de-select from.
		if (sourceName == "TrainingChoice") {
			targetList = trainingList;
		} else if (sourceName == "TestChoice") {
			targetList = testList;
		}
	
		if (selection == "Select % of Examples") {
			if (inline)
				new SelectExamplesDialog(null, targetList);
			else
				new SelectExamplesDialog(((NeuralWindow)window), targetList);
		} else if (selection == "Select All") {	
			selectAll(targetList);
		} else if (selection == "Select None") {
			deSelectAll(targetList);
		} else if (selection == "Invert Selection") {
			invertSelection(targetList);
		}
	}
	
	/**
	 * adjusts the array of indexes to take into account the initial row, and if it was
	 * selected which results in a negative index. this removes the negative index
	 * and returns an array without it
	 * @param indexes - the indexes to be adjusted
	 */
	private int[] adjustIndexes(int[] indexes) {
		for (int i = 0; i < indexes.length; i++) {
			indexes[i] = indexes[i] - 1;
		}
		
		//first element selected was header
		if(indexes[0] < 0) {
			int[] newIndexes = new int[indexes.length-1];
			//remove that item and advance all items by one
			for(int i = 1; i < indexes.length; i++) {
				newIndexes[i-1] = indexes[i];
			}
			indexes = newIndexes;
		}
		
		return indexes;
	}

	/**
	 * deSelects all rows of an  list.
	 * 
	 * @param	The  list to deselect all items in
	 * 
	 */
	private void deSelectAll(JList targetList) {
		targetList.clearSelection();
	}
	
	/**
	 * returns the training list of examples from the dialog
	 */
	public JList getTrainingList() {
		return trainingList;
	}

	/**
	 * returns the test list of examples from the dialog
	 */
	public JList getTestList() {
		return testList;
	}


	/**
	 * Selects all rows of an  list.
	 * 
	 * @param	The  list to select all items in
	 * 
	 */
	private void selectAll(JList targetList)
	{
		deSelectAll(targetList);
		
		DefaultListModel targetModel =  (DefaultListModel) targetList.getModel();
		targetList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		targetList.setModel(targetModel);
		int[] indices = new int[targetModel.getSize()-1];
		for (int i = 1; i < targetModel.getSize(); i++)
		{
			indices[i-1]=i;
		
		}	
	
		targetList.setSelectedIndices(indices);	
	}
	
	/**
	 * Inverts the selected rows of the passed in list.
	 * If the row was selected, it is now deselected, and vice versa.
	 * 
	 * @param	The  list to invert.
	 * 
	 */
	private void invertSelection(JList targetList) {
		DefaultListModel targetModel =  (DefaultListModel) targetList.getModel();

		ArrayList<Integer> vecInds = new ArrayList<Integer>();
		
		int[] selectedinds = targetList.getSelectedIndices();
		for (int i = 1; i < targetModel.getSize(); i++) {
			boolean flag = false;
			for(int j = 0; j< selectedinds.length; j++)
				if (selectedinds[j] == i)	flag= true;
			if(!flag)
				vecInds.add(new Integer(i)); 
		}	
		
		int[] inds = new int[vecInds.size()];
	
		for(int j = 0; j< vecInds.size();j++) {
			inds[j] = vecInds.get(j).intValue();
		} 
		targetList.setSelectedIndices(inds);
	}


	/**
	 * Listener for user input on the selection drop-down choice boxes 
	 * 
	 */
	public void itemStateChanged(ItemEvent e)
	{
		
		String selection = (String) ((JComboBox) e.getSource()).getSelectedItem();
		String sourceName = ((JComboBox)e.getSource()).getName();
	
		JList targetList = null;
	  
		// Set the list that the user wants to select/de-select from.
		if (sourceName == "TrainingChoice")
		{
			targetList = trainingList;
		}
		else if (sourceName == "TestChoice")
		{
			targetList = testList;
		}

		if (selection == "Select Percentage of Examples")
		{
			if (inline)
				new SelectExamplesDialog(null, targetList);
			else
				new SelectExamplesDialog((NeuralWindow)window, targetList);
		}
		else if (selection == "Select All")
		{	
			selectAll(targetList);
		}
		else if (selection == "Select None")
		{
			deSelectAll(targetList);
		}
		else if (selection == "Invert Selection")
		{
			invertSelection(targetList);
		}
		
	}

	/**
	 * Adds a component to the gridbag.
	 */
	private void addComponent(JComponent c, Container p, int row, int column, int width, int height, double wx, double wy)
	{
		// set gridx and gridy
		gridBagConstraints.gridx = column;
		gridBagConstraints.gridy = row;
	
		// set gridwidth and gridheight
		gridBagConstraints.gridwidth = width;
		gridBagConstraints.gridheight = height;
	
		// set weights
		gridBagConstraints.weightx = wx;
		gridBagConstraints.weighty = wy;
	
		gridBagLayout.setConstraints(c, gridBagConstraints);
		p.add(c);
	}


	/** Returns an ImageIcon, or null if the path was invalid. */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = GraphWindow.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		System.err.println("Couldn't find file: " + path);
		return null;
	}

   /**
	* Centers this window.
	*/
   private void centerWindow()
   {
	   if (!inline)
	   {
		   int midX = window.getLocationOnScreen().x + window.getSize().width/2;
		   int midY = window.getLocationOnScreen().y + window.getSize().height/2;
		   setLocation( midX - getSize().width/2, midY - getSize().height/2 );
	   }
	   else
	   {
		   // If inline, just create the dialog at (50,50).
		   setLocation(50, 50);
	   }
   }


	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}

	public void windowClosing(WindowEvent e)
	{
		//The window is hidden rather than disposed
		//because it will be opened and closed frequently
		//and may require heavy computation during construction
		setVisible(false);
	}

	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
}