/**
 * InitializationOptionsDialog.java
 * May 16, 2007
 */

package AIspace.neural.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.neural.NeuralGraph;

/**
 * Dialog box to set parameter initialization options.
 * @author Byron Knoll
 */
public class InitializationOptionsDialog extends BasicDialog implements CaretListener, ItemListener{
	private static final long serialVersionUID = 1L;
	private JTextField textValue;
	private JTextField textBound;
	private JCheckBox checkValue;
	private JCheckBox checkBound;
	private JLabel labelError;

	private NeuralGraph graph;
	
	/**
	 * Construct the dialog.
	 *
	 * @param parent - the NeuralWindow that the dialog is opened from
	 * @param graph - the NeuralGraph for the neural network to be learned
	 * @param title	- the title of this dialog
	 * @param visible - a boolean which determins if the dialog is visible
	 */
	public InitializationOptionsDialog (JFrame parent, NeuralGraph graph, boolean visible) {
		super(parent, "Parameter Initialization Options", true);
		this.graph = graph;
		
		checkValue = new JCheckBox ("Constant Parameter");
		checkValue.setSelected(!graph.getInitParamType());
		checkValue.addItemListener(this);
		JLabel labelValue = new JLabel ("Value: ");
		JPanel panelValue = new JPanel (new BorderLayout());
		panelValue.add(checkValue, BorderLayout.WEST);
		panelValue.add(labelValue, BorderLayout.EAST);
		textValue = new JTextField( graph.getInitParamValue() +"", 20 );
		textValue.setEnabled(!graph.getInitParamType());
		textValue.addCaretListener(this);
		
		checkBound = new JCheckBox ("Random Parameter");
		checkBound.setSelected(graph.getInitParamType());
		checkBound.addItemListener(this);
		JLabel labelBound = new JLabel ("Bound: ");
		JPanel panelBound = new JPanel (new BorderLayout());
		panelBound.add(checkBound, BorderLayout.WEST);
		panelBound.add(labelBound, BorderLayout.EAST);
		textBound = new JTextField( graph.getInitParamBound() + "", 20 );
		textBound.setEnabled(graph.getInitParamType());
		textBound.addCaretListener(this);
		
		JButton buttonOk = new JButton("OK");
		buttonOk.addActionListener(this);
		JButton buttonCancel = new JButton("Cancel");
		buttonCancel.addActionListener(this);
		buttonCancel.setPreferredSize(new Dimension(50, buttonCancel.getHeight()));

		labelError = new JLabel("", JLabel.CENTER);
		JPanel panel = new JPanel(new GridLayout(3, 2));
		panel.add(panelValue);
		panel.add(textValue);
		panel.add(panelBound);
		panel.add( textBound );
		panel.add(buttonOk);
		panel.add(buttonCancel);
		getContentPane().add(panel);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(visible);
	}
	
	public void open() {
		textBound.setText( graph.getInitParamValue() + "");
		textBound.setText( graph.getInitParamBound() + "");
		pack();
		setVisible(true);
	}

	/**
	 * Set the parameter values.
	 */
	public boolean setProperties() {
		try {
			graph.setInitParamValue(Double.parseDouble( textValue.getText() ));
			double bound = Double.parseDouble( textBound.getText() );
			if( bound <= 0 ) {
				labelError.setText( "Error: parameter bound must be positive." );
				return false;
			}
			graph.setInitParamBound( bound );
			graph.setInitParamType(checkBound.isSelected());
			return true;
		} catch (NumberFormatException e) {
			labelError.setText("Error: Invalid number format.");
			return false;
		}
	}

	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return setProperties();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	public void caretUpdate(CaretEvent e) {
		labelError.setText("");
	}

	public void itemStateChanged (ItemEvent e) {
		if (e.getSource()==checkValue)
		{
			checkBound.removeActionListener(this);
			checkBound.setSelected(!checkValue.isSelected());
			textBound.setEnabled(!checkValue.isSelected());
			checkBound.addActionListener(this);
		}
		else if (e.getSource()==checkBound)
		{
			checkValue.removeActionListener(this);
			checkValue.setSelected(!checkBound.isSelected());
			textValue.setEnabled(!checkBound.isSelected());
			checkValue.addActionListener(this);
		}
	}
}