/**
 * InputExampleDialog.java
 * Jul 19, 2000
 */


package AIspace.neural.dialogs;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.graphToolKit.elements.Node;
import AIspace.neural.NeuralGraph;
import AIspace.neural.elements.NeuralNode;


/**
 * @author Leslie Tung & Shinjiro Sueda
 */
public class InputExampleDialog extends BasicDialog {

	//
	// Fields
	//
	
	private static final long serialVersionUID = 1L;
	
	public final static int TRAINING_ADD = 0;
	public final static int TEST_ADD = 1;
	public final static int TRAINING_EDIT = 3;
	public final static int TEST_EDIT = 4;
	
	private JLabel labelInputs[];
	private JTextField textInputs[];
	private Integer indexInputs[];
	private JLabel labelOutputs[];
	private JTextField textOutputs[];
	private Integer indexOutputs[];
	private JButton button1;
	private JButton button2;
	private JLabel labelError;
	private JPanel p;

	private NeuralGraph graph;
	private ExampleDialog dialog;
	private int numInputs;
	private int numOutputs;
	private int mode;
	
	//
	// Methods
	//
	
	public InputExampleDialog (JFrame parent, ExampleDialog dialogParent, NeuralGraph g, String title)
	{
		super(parent, title, true);
		dialog = dialogParent;
		if (title == "Input Training Examples")
  			mode = TRAINING_ADD;
		else if (title == "Input Test Examples") 
			mode = TEST_ADD;
		else if (title == "Edit Training Example")
			mode = TRAINING_EDIT;
		else if (title == "Edit Test Example")
			mode = TEST_EDIT;
	
		graph = g;

		numInputs = numOutputs = 0;
		NeuralNode tmpNode;
		Iterator<Node> nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			if ((tmpNode.getNumParentNodes() == 0) && (tmpNode.getNumChildrenNodes() > 0)) {
				numInputs++;
			}

			if ((tmpNode.getNumChildrenNodes() == 0) && (tmpNode.getNumParentNodes() > 0)) {
				numOutputs++;
			}
		}

		if (numInputs > 0)
		{
			labelInputs = new JLabel[numInputs];
			textInputs = new JTextField[numInputs];
			indexInputs = new Integer[numInputs];
		}
				
		if (numOutputs > 0)
		{
			labelOutputs = new JLabel[numOutputs];
			textOutputs = new JTextField[numOutputs];
			indexOutputs = new Integer[numOutputs];
		}

		int countInput = 0;
		int countOutput = 0;
		int numNodes =0;
		String nodeName;
		
		// get the values of the first selected example if editing
		int exampleIndex = -1;
		Hashtable oneExample = null;
		if(mode == TRAINING_EDIT) {
			exampleIndex = (dialogParent.getTrainingList().getSelectedIndices())[0];
			// Deselect the other selected examples.
			dialogParent.getTrainingList().setSelectedIndex(exampleIndex);
			oneExample = graph.getExampleList().getTrainingExampleValues(exampleIndex-1);
		} else if(mode == TEST_EDIT) {
			exampleIndex = (dialogParent.getTestList().getSelectedIndices())[0];
			// Deselect the other selected examples.
			dialogParent.getTestList().setSelectedIndex(exampleIndex);
			oneExample = graph.getExampleList().getTestExampleValues(exampleIndex-1);
		}
		
		nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			nodeName = tmpNode.getLabel();
			if (nodeName.equals(""))
			{
				nodeName = new String("Node " + tmpNode.getIndex());
			}

			if ((tmpNode.getNumParentNodes() == 0) && (tmpNode.getNumChildrenNodes() > 0))
			{
				labelInputs[countInput] = new JLabel(nodeName);
				if(exampleIndex != -1) {
				textInputs[countInput]  = new JTextField((String)oneExample.get(new Integer(countInput)), 10);
			} else {
				textInputs[countInput]  = new JTextField("", 10);
			}
				indexInputs[countInput] = new Integer(tmpNode.getIndex());
				numNodes++;
				
				countInput++;
			}

			if ((tmpNode.getNumChildrenNodes() == 0) && (tmpNode.getNumParentNodes() > 0))
			{
				labelOutputs[countOutput] = new JLabel(nodeName);
				if(exampleIndex != -1) {
					textOutputs[countOutput]  = new JTextField((String)oneExample.get(new Integer(countInput+countOutput)), 10);
			} else {
				textOutputs[countOutput]  = new JTextField("", 10);
			}
				indexOutputs[countOutput] = new Integer(numNodes);
				countOutput++;
				numNodes++;
			}
		}

		if ((mode == TRAINING_ADD) || (mode == TEST_ADD)) {
			// input example
			button1 = new JButton("Add");
		} else if((mode == TRAINING_EDIT) || (mode == TEST_EDIT)) {
			button1 = new JButton("Update");
		}
		button1.addActionListener(this);
		button2 = new JButton("Cancel");
		button2.addActionListener(this);
		labelError = new JLabel("");
		p = new JPanel();
		p.setLayout(new GridLayout(numInputs + numOutputs + 4, 2));

		p.add(new JLabel("Input Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numInputs; i++)
		{
			p.add(labelInputs[i]);
			p.add(textInputs[i]);
		}

		p.add(new JLabel("Output Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numOutputs; i++)
		{
			p.add(labelOutputs[i]);
			p.add(textOutputs[i]);
		}
		p.add(button1);
		p.add(button2);
		getContentPane().add(p);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}

	public boolean setExample()
	{
		try
		{
			Integer indices[] = new Integer[numInputs + numOutputs];
			Double values[] = new Double[numInputs + numOutputs];
		
			for (int i = 0; i < numInputs; i++)
			{
				indices[i] = new Integer(i);
				values[i] = Double.valueOf(textInputs[i].getText());
			}
			for (int i = 0; i < numOutputs; i++)
			{
				indices[i + numInputs] = new Integer(i+numInputs);
				values[i + numInputs] = Double.valueOf(textOutputs[i].getText());
			}
			if (mode == TRAINING_ADD) {
				graph.addTrainingExample(indices, values);
			} else {
				graph.addTestExample(indices, values);	
			}
			return true;
		}
		catch (NumberFormatException e)
		{
			labelError.setText(
				"Error: Invalid number format.");
			return false;
		}
	}
	
	public boolean updateExample() {
		try {
			Integer indices[] = new Integer[numInputs + numOutputs];
			String values[] = new String[numInputs + numOutputs];
			//valuesOk true if no error occurred with parameters
			boolean valuesOk = true;
		
			//get all the parameters into the array
			for (int i = 0; i < numInputs; i++) {
				indices[i] = indexInputs[i];
				values[i] = ""+Double.valueOf(textInputs[i].getText().trim());
				if(values[i].equals(""))
					valuesOk = false;
			}
			for (int i = numInputs; i < numInputs+numOutputs; i++) {
				indices[i] = indexOutputs[i-numInputs];
				values[i] = ""+Double.valueOf(textOutputs[i-numInputs].getText().trim());
				if(values[i].equals(""))
					valuesOk = false;
			}
			
			if(!valuesOk) {
				labelError.setText("Error: Please enter all parameters.");
				return false;
			}
	
			//add and remove from training or test lists
			if (mode == TRAINING_EDIT) { 
				int index = (dialog.getTrainingList().getSelectedIndices())[0];
				graph.getExampleList().updateTrainingExample(index-1, indices, values);
			} else if (mode == TEST_EDIT) { 
				int index = (dialog.getTestList().getSelectedIndices())[0];
				graph.getExampleList().updateTestExample(index-1, indices, values);
			}
	
			return true;
		} catch (NumberFormatException e) {
			labelError.setText("Error: Invalid number format.");
			pack();
			return false;
		}
	}
	
	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return true;
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	
	public void actionPerformed (ActionEvent e) {
		super.actionPerformed(e);
		if (e.getActionCommand() == "Add") {
			if (setExample()) {
				setVisible(false);
				dispose();
			}
		} else if (e.getActionCommand() == "Update") {
			if (updateExample()) {
				setVisible(false);
				dispose();
			}
		}
	}
}