/**
 * InputExampleFrame.java
 * Aug 16, 2000
 */

package AIspace.neural.dialogs;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicFrame;
import AIspace.graphToolKit.elements.Node;
import AIspace.neural.NeuralGraph;
import AIspace.neural.elements.NeuralNode;

/**
 * @author Leslie Tung & Shinjiro Sueda
 */
public class InputExampleFrame extends BasicFrame {

   //
   // Fields
   //
   
	private JLabel labelInputs[];
	private JTextField textInputs[];
	private Integer indexInputs[];
	private JLabel labelOutputs[];
	private JTextField textOutputs[];
	private Integer indexOutputs[];
	private JButton button1;
	private JButton button2;
	private JLabel labelError;
	private JPanel p;

	private NeuralGraph graph;
	private int numInputs;
	private int numOutputs;
	private boolean mode;

	
	//
	// Methods
	//
	
	/**
	 * if the boolean variable which is true, then we are inputting an example
	 * if it is false, then we are calculating the output values given the input.
	 */
	public InputExampleFrame (NeuralGraph g, String title, boolean which) {
		super(title);
		super.setOKString("Add");
		graph = g;
		mode = which;

		numInputs = numOutputs = 0;
		NeuralNode tmpNode;
		Iterator<Node> nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			if (tmpNode.getNumParentNodes() == 0) {
				numInputs++;
			}

			if (tmpNode.getNumChildrenNodes() == 0) {
				numOutputs++;
			}
		}

		if (numInputs > 0) {
			labelInputs = new JLabel[numInputs];
			textInputs = new JTextField[numInputs];
			indexInputs = new Integer[numInputs];
		}
					 
		if (numOutputs > 0) {
			labelOutputs = new JLabel[numOutputs];
			textOutputs = new JTextField[numOutputs];
			indexOutputs = new Integer[numOutputs];
		}

		int countInput = 0;
		int countOutput = 0;
		String nodeName;
		nodeItr = graph.getNodes();
		while (nodeItr.hasNext()) {
			tmpNode = (NeuralNode)nodeItr.next();
			nodeName = tmpNode.getLabel();
			if (nodeName.equals("")) {
				nodeName = new String("Node " + tmpNode.getIndex());
			}

			if (tmpNode.getNumParentNodes() == 0) {
				labelInputs[countInput] = new JLabel(nodeName);
				textInputs[countInput] = new JTextField("0", 20);
				indexInputs[countInput] = new Integer(tmpNode.getIndex());
				countInput++;
			}

			if (tmpNode.getNumChildrenNodes() == 0) {
				labelOutputs[countOutput] = new JLabel(nodeName);
				textOutputs[countOutput] = new JTextField("0", 20);
				indexOutputs[countOutput] = new Integer(tmpNode.getIndex());
				countOutput++;
			}
		}

		if (mode == true) {
			// input example
			button1 = new JButton("Add");
		} else {
			// calculate output
			button1 = new JButton("Calculate");
		}
		button1.addActionListener(this);
		button2 = new JButton("Cancel");
		button2.addActionListener(this);
		labelError = new JLabel("");
		p = new JPanel();
		p.setLayout(new GridLayout(numInputs + numOutputs + 4, 2));

		p.add(new JLabel("Input Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numInputs; i++) {
			p.add(labelInputs[i]);
			p.add(textInputs[i]);
		}

		p.add(new JLabel("Output Values:"));
		p.add(new JLabel(""));
		for (int i = 0; i < numOutputs; i++) {
			p.add(labelOutputs[i]);
			p.add(textOutputs[i]);
			if (mode == false) { // calculate outputs
				textOutputs[i].setText("");
				textOutputs[i].setEditable(false);
			}
		}
		p.add(button1);
		p.add(button2);
		add(p);
		add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}
	

	public boolean setExample() {
		try {
			Integer indices[] = new Integer[numInputs + numOutputs];
			Double values[] = new Double[numInputs + numOutputs];
		  
			for (int i = 0; i < numInputs; i++) {
				indices[i] = indexInputs[i];
				values[i] = Double.valueOf(textInputs[i].getText());
			}
			for (int i = 0; i < numOutputs; i++) {
				indices[i + numInputs] = indexOutputs[i];
				values[i + numInputs] = Double.valueOf(textOutputs[i].getText());
			}
			graph.addTrainingExample(indices, values);
			return true;
		} catch (NumberFormatException e) {
			labelError.setText("Error: Parameters have invalid real number format!");
			return false;
		}
	}

	
	public void calculateOutputs() {
		try {
			Double values[] = new Double[numInputs];
			for (int i = 0; i < numInputs; i++) {
				values[i] = Double.valueOf(textInputs[i].getText());
			}

			int currIndex;
			double value;
			for (int i = 0; i < numOutputs; i++) {
				// calculate values for each output
				currIndex = indexOutputs[i].intValue();
				if (graph.backPropagation == null) {
					graph.initBackProp();
				}
				value = graph.backPropagation.solveNodeOutput(currIndex, values);
				
				// round it to 4 decimal places
				value = Math.round(value * 10000.0) / 10000.0; 
				textOutputs[i].setText((new Double(value)).toString());
			}
		} catch (NumberFormatException e) {
			labelError.setText("Error: Parameters have invalid real number format!");
		}
	}

	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return setExample();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	public void actionPerformed (ActionEvent e) {
		super.actionPerformed(e);
		if (e.getActionCommand() == "Calculate") {
			labelError.setText("");
			calculateOutputs();
		}
	}
}