/**
 * LearningOptionsDialog.java
 * July 18, 2002
 */

package AIspace.neural.dialogs;

import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.neural.NeuralGraph;

/**
 * dialog box to set learning parameters (momentum, rate, and weight bound)
 * before starting the back propagation algorithm
 * @author Leslie Tung, Shinjiro Sueda, Joseph Roy Santos
 */
public class LearningOptionsDialog extends BasicDialog implements CaretListener {
	private JTextField textMomentum;
	private JTextField textRate;
	private JLabel labelError;

	private NeuralGraph graph;
	
	/**
	 * construct the dialog
	 *
	 * @param parent - the NeuralWindow that the dialog is opened from
	 * @param graph - the NeuralGraph for the neural network to be learned
	 * @param title	- the title of this dialog
	 */
	public LearningOptionsDialog (JFrame parent, NeuralGraph graph) {
		super(parent, "Learning Options", true);
		this.graph = graph;
		
		textMomentum = new JTextField(graph.getMomentum()+"",20);
		textMomentum.addCaretListener(this);
		
		textRate = new JTextField(graph.getLearningRate() + "", 20);
		textRate.addCaretListener(this);
		
		JButton buttonOk = new JButton("OK");
		buttonOk.addActionListener(this);
		JButton buttonCancel = new JButton("Cancel");
		buttonCancel.addActionListener(this);

		labelError = new JLabel("", JLabel.CENTER);
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(3, 2));
		panel.add(new JLabel("Learning Rate:"));
		panel.add(textRate);
		panel.add(new JLabel( "Momentum:"));
		panel.add(textMomentum);
		panel.add(buttonOk);
		panel.add(buttonCancel);
		getContentPane().add(panel);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}
	
	public void open() {
		textMomentum.setText(graph.getMomentum()+"");
		textRate.setText(graph.getLearningRate() + "");
	 
		setVisible(true);
	}

	/**
	 * set the number of iterations and the learning rate
	 */
	public boolean setProperties() {
		try {
			graph.setLearningRate(Double.parseDouble( textRate.getText() ));
			
			double momentum = Double.parseDouble(textMomentum.getText());
			if ((momentum < 0) || (momentum >= 1)) {
				labelError.setText("Error: Momentum must be a real number x, 0 < x < 1.");	
				return false;
			}
			graph.setMomentum(momentum);
 
			return true;
		} catch (NumberFormatException e) {
			labelError.setText("Error: Invalid number format.");
			return false;
		}
	}

	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return setProperties();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	public void caretUpdate(CaretEvent e) {
		labelError.setText("");
	}
}