/**
 * File: ProblemDialog.java
 * Aug 11, 2000
 */

package AIspace.neural.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import AIspace.graphToolKit.dialogs.ProblemDialog;
import AIspace.neural.NeuralWindow;

public class NeuralProblemDialog extends ProblemDialog {
	
	private static final long serialVersionUID = 1L;
	private boolean withWizard;

	/**
 	* construct the dialog box
 	*/
	public NeuralProblemDialog (NeuralWindow window, boolean withWizard) {
		super(window);
		this.withWizard = withWizard;
		if(withWizard) {
			//if using the wizard, use the other problem set and populate the list
			getContentPane().remove(1);
			getContentPane().add( constructCenterPanel(), BorderLayout.CENTER );
		}
	}
	
	protected boolean getProblemList() {
		try {
			String text;
			BufferedReader in = null;
			
			if (withWizard)
				in = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/SampleData.txt")));
			else
				in = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/SampleProblems.txt")));
			
			int p1;
			int p2;
			//read in the file
			while ((text = in.readLine()) != null) {
				//trim any excess whitespace
				text = text.trim();
				//ignore empty or commented lines
				if ((text.length() > 0) && !text.substring(0, 2).equals("//")) {
					//get the first divider index
					p1 = text.indexOf(",");
					
					//get the second divider index
					p2 = text.indexOf(",", p1+1);
			
					//put the problem name (first) and the file name (second) into the hashmap)
					problemNameToFileName.put(text.substring(0, p1), text.substring(p1 + 1, p2).trim());
			
					if (text.endsWith("."))
					{
						sampleProblemDescription.put(text.substring(0, p1), text.substring(p2 + 1, text.length()).trim());
					}
					else 
						sampleProblemDescription.put(text.substring(0, p1), "");
				}
			}
			in.close();
			return true;
		} catch (Exception e) {
			window.showMessage( "Error", "Error loading problem list: "+e.getLocalizedMessage() + ".");
			return false;
		}
	}
	
	/**
	 * try to load the URL file
	 */
	protected void loadProblem() {
		if (list.getSelectedValue() != null ) {
			window.setFileName(problemNameToFileName.get(list.getSelectedValue()));
			if(withWizard) {
				((NeuralWindow) window).loadWizard(new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/saves/"+problemNameToFileName.get(list.getSelectedValue())))));
			}
			else
			{
				window.load(new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/saves/"+problemNameToFileName.get(list.getSelectedValue())))));
			}
			
		} else {
			window.showMessage("Error", "Error loading file.");
		}
	}
}