/**
 * File: NeuralTextFrame.java
 * Jun 27, 2000
 */

package AIspace.neural.dialogs;

import AIspace.graphToolKit.dialogs.TextFrame;
import AIspace.neural.NeuralCanvas;


/**
 * TextFrame produces a frame to show the text representation of the
 * graph, and allow the user to edit the text representation. 
 *
 * If the user clicks "Update", this calls the method 'updateGraphFromText'
 * in class ConstraintWindow, and the graph is updated accordingly.
 *
 * @author  Kevin O'Neill & Shinjiro Sueda
 * @version 1.1
 * @since JDK1.1
 */
public class NeuralTextFrame extends TextFrame {
	
	private boolean isXML;

	/**
	 * constructor
	 * @param canvas - the canvas for this text representation
	 * @param textRep - the actual text rep of the canvas
	 * @param isXML - true if the text rep is in XML format
	 */
	public NeuralTextFrame(NeuralCanvas canvas, String textRep, String title, boolean isXML, boolean isEditable) {
		super(canvas, textRep, title);
		this.isXML = isXML;
		if(!isEditable) {
			display.setEditable(false);
			updateButton.setEnabled(false);
			updateButton.setVisible(false);
		}
		super.open();
	}
	
	/** "Update" actionEvent from actionPerformed */
	protected boolean updateAction() {
		if (isXML && ((NeuralCanvas) canvas).updateGraphFromText(display.getText(), true)) {
			return true;
		} else if (((NeuralCanvas) canvas).updateGraphFromText(display.getText(), false)) {
			return true;
		}
		return false;
	}
}