/**
 * NeuralWizard.java
 * July 19, 2002
 */
package AIspace.neural.dialogs;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Hashtable;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import AIspace.graphToolKit.GraphConsts;
import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.graphToolKit.elements.Point;
import AIspace.neural.ExampleList;
import AIspace.neural.NeuralCanvas;
import AIspace.neural.NeuralGraph;
import AIspace.neural.elements.NeuralEdge;
import AIspace.neural.elements.NeuralNode;

/**
 * @author Joseph Roy Santos
 * NeuralWizard implements the Wizard that auto-creates the neural net.
 * 
 */
public class NeuralWizard extends BasicDialog implements ItemListener, KeyListener {
	private JTextField hiddenLayerNumber;
	private JTextField hiddenLayerNodeNumber;
	private int[] numHiddenNodes;
	private JCheckBox nodeList[];
	private JComboBox hiddenLayerIndex;
	private DefaultComboBoxModel comboModel;
	private JCheckBox ordered[];
	private NeuralGraph nGraph;
	private ExampleList example;			//used for creating neural network
	private JLabel errLabel;
	
	/**
	 * Constructor that shows a panel for input
	 * @param graph		The parent neuralgraph
	 * @param example	A filled exampleList
	 * @param filename	The filename of the file that was loaded into example
	 */
	public NeuralWizard (JFrame parent, NeuralGraph graph, ExampleList example, String filename) {
		super(parent, "Neural Network Construction Wizard", true, 2);
		this.gbl = new GridBagLayout();
		this.gbc = new GridBagConstraints();
		this.gbc.fill = GridBagConstraints.HORIZONTAL;
		
		this.nGraph = graph;
		this.example = example;

		JPanel mainPanel = new JPanel(gbl);
		mainPanel.setBorder(new EmptyBorder(3,3,3,3));
		
		JLabel sourceName = new JLabel("Filename: " + filename, JLabel.CENTER);
		sourceName.setForeground(Color.blue);
		
		JPanel hiddenLayerNumberPanel = new JPanel(new FlowLayout(FlowLayout.LEFT,1,0));
		JLabel tempLabel = new JLabel("Create ");
		hiddenLayerNumberPanel.add(tempLabel);
		hiddenLayerNumber = new JTextField("1", 3);
		hiddenLayerNumber.setHorizontalAlignment(JTextField.RIGHT);
		hiddenLayerNumber.setName("hiddenLayerNumber");
		hiddenLayerNumber.addKeyListener(this);
		hiddenLayerNumberPanel.add(hiddenLayerNumber);
		
		tempLabel = new JLabel(" hidden layers");
		hiddenLayerNumberPanel.add(tempLabel);
		
		JPanel layerChoicePanel = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
		tempLabel = new JLabel("Hidden layer ");
		layerChoicePanel.add(tempLabel);
		
		String hiddenString = "1";
 		comboModel = new DefaultComboBoxModel(); 
		comboModel.addElement(hiddenString);
		hiddenLayerIndex = new JComboBox();
		hiddenLayerIndex.setModel(comboModel);
		hiddenLayerIndex.setSelectedIndex(0);
		hiddenLayerIndex.setName("hiddenLayerIndex");
		hiddenLayerIndex.addActionListener(this);
		layerChoicePanel.add(hiddenLayerIndex);
		
		tempLabel = new JLabel(" has ");
		layerChoicePanel.add(tempLabel);
		hiddenLayerNodeNumber = new JTextField("2");
		hiddenLayerNodeNumber.setName("hiddenLayerNodeNumber");
		hiddenLayerNodeNumber.addKeyListener(this);
		numHiddenNodes = new int[1];
		numHiddenNodes[0] = 2;
		layerChoicePanel.add(hiddenLayerNodeNumber);
		tempLabel = new JLabel(" nodes.");
		layerChoicePanel.add(tempLabel);
		
		JLabel categoryLabel = new JLabel("Choose categories as outputs:");
		nodeList = new JCheckBox[example.getNumParameters()]; 
		ordered = new JCheckBox[example.getNumParameters()]; 
		String[] paraNames = example.getParameters();
		for (int i = 0; i < nodeList.length; i++) {
			nodeList[i] = new JCheckBox(paraNames[i]);
			nodeList[i].setSelected(!example.isInputParam(i));
			nodeList[i].setName((new Integer(i)).toString());
			nodeList[i].addItemListener(this);
			ordered[i] = new JCheckBox("ordered?");
		}
	
		JPanel buttonPanel = new JPanel(new FlowLayout());
		JButton ok = new JButton("OK");
		ok.addActionListener(this);
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		buttonPanel.add(ok);
		buttonPanel.add(cancel);
		
		addComponent(sourceName, mainPanel, 0, 0, 2, 1, 2 ,1);
		addComponent(hiddenLayerNumberPanel, mainPanel, 2, 0, 2, 1, 2 ,1);
		addComponent(layerChoicePanel, mainPanel, 3, 0, 2, 1, 2 ,1);
		addComponent(categoryLabel, mainPanel, 4, 0, 2, 1, 2 ,1);
		
		int totalLength = 4+nodeList.length;
		for (int i = 0; i < nodeList.length; i++) {
			addComponent(nodeList[i], mainPanel, 5+i, 0, 1, 1, 1 ,1);
			addComponent(ordered[i], mainPanel, 5+i, 1, 1, 1, 1 ,1);
		}
		
		errLabel = new JLabel();
		errLabel.setForeground(Color.red);
		addComponent(errLabel, mainPanel, totalLength+1, 0, 2, 1, 2 ,1);
		addComponent(buttonPanel, mainPanel, totalLength+2, 0, 2, 1, 2 ,1);
		getContentPane().add(mainPanel);

		setSize(280,300);
		centerWindow();
		setVisible( true );
	}
	
	/** 2nd constructor that doesn't create a viewable panel */
	public NeuralWizard (JFrame parent, NeuralGraph graph, ExampleList example) {
		super(parent, "Neural Network Construction Wizard", true, 2);

		this.nGraph = graph;
		this.example = example; 
	
		hiddenLayerNumber = new JTextField("1");
		hiddenLayerNumber.setName("hiddenLayerNumber");
		hiddenLayerNumber.addKeyListener(this);

		String[] hidenString = {"1"};
		hiddenLayerIndex = new JComboBox(hidenString);

		numHiddenNodes = new int[1];
		numHiddenNodes[0] = 2;
	
		nodeList = new JCheckBox[example.getNumParameters()]; 
		ordered = new JCheckBox[example.getNumParameters()]; 
		String[] paraNames = example.getParameters();
		for (int i = 0; i < nodeList.length; i++) {
			nodeList[i] = new JCheckBox(paraNames[i]);
			nodeList[i].setSelected(!example.isInputParam(i));
			nodeList[i].setName((new Integer(i)).toString());
			nodeList[i].addItemListener(this);
			ordered[i] = new JCheckBox("ordered?");
		}
	
		buildGraph();
	}
   
   protected boolean actionCancel() {
	   return true;
   }
   
   protected boolean actionOK() {
	   // if ok is pressed, check to see if at least one output and
		// one input is selected. If the selections are valid, build the
		// graph, return and dispose.	
	   if (!validOutputSelection()) {
			errLabel.setText("Must select at least one output and input");
			return false;		
		}
		setCursor(GraphConsts.WAIT_CURSOR);
		buildGraph();
		setCursor(GraphConsts.DEFAULT_CURSOR);
		nGraph.setExampleList(example);

		return true;
   }
	/**
	* Implements actionListener
	*/
	public void actionPerformed (ActionEvent e) {
		super.actionPerformed(e);
		if (e.getActionCommand() == "comboBoxChanged") {
			if(hiddenLayerIndex.getSelectedIndex() != -1) {
				hiddenLayerNodeNumber.setText(""+numHiddenNodes[hiddenLayerIndex.getSelectedIndex()]);
			}
		}
	}
	
	/**
	 * Function that checks if there is at least one input and one output
	 * selected in the category fields.
	 * @return true if there is at least one input and one output, false otherwise
	 */
	public boolean validOutputSelection() {
		boolean hasInput = false;
		boolean hasOutput = false;	
		for (int i = 0; i < nodeList.length; i++) {
			hasInput = (!nodeList[i].isSelected()) || hasInput; 	
			hasOutput = (nodeList[i].isSelected()) || hasOutput; 
		}
		
		return (hasInput && hasOutput);
	}
	
	public void setExampleList(ExampleList list) {
		this.example = list;
	}
	
	/**
	 * Function to build a neural network with the appropriate exampleList 
	 * structure from non-numerical exampleList. This converts an exampleList
	 * into a form that can be learned by a neural network, and also helps
	 * the user build the neural network.
	 */
	public void buildGraph() {
		// clear the canvas
		NeuralCanvas newCanvas = (NeuralCanvas) nGraph.canvas;
		nGraph = (NeuralGraph) newCanvas.graph;
		nGraph.setExampleList(example);
		// algorithm
		// first, go through example list and note the parameters
		// that are non-numerical. List down the range.
		// second, create a new example list that represents the
		// same example set in 1-of-C encoding or a continuum depending
		// on the user.
		// third, create the neural network from the new example list
		// and information provided by the user.
		
		// Array that will store vectors of ranges for each parameter.
		// Each entry will either be a offsetPoint of strings (if the parameter
		// is categorical) or null (if the parameter is numerical)
		ArrayList<String>[] rangeArray = new ArrayList[example.getNumParameters()];
		// an array of hashtables that stores domain elements and their 
		// corresponding user-defined mappings into numerical examples.
		Hashtable[] valueMapArray = new Hashtable[example.getNumParameters()];
		// fields to determine if a category can be represented as a number
		boolean[] nonCategorical = new boolean[example.getNumParameters()];
		for (int i = 0; i < nonCategorical.length; i++) {
			nonCategorical[i] = true;	
		}
		// get the test and training sets
		ArrayList<Hashtable<Integer, String>> test = example.getTestArrayList();
		ArrayList<Hashtable<Integer, String>> training = example.getTrainingArrayList();
		// test to see WHICH parameters are categorical
		// note that this relies on having at least one example
		// in the training set. The algorithm just checks each letter
		// if the string if it is either a digit or a . symbol
		Hashtable<Integer, String> oneExample = null;

		for (int allExamples = 0; allExamples < training.size(); allExamples++) {
			oneExample = training.get(allExamples);

			for (int i = 0; i < oneExample.size(); i++) {
				String para = oneExample.get(new Integer(i));
				for (int j = 0; j < para.length(); j++) {
					if (!nonCategorical[i]) {
						break;
					}
					nonCategorical[i] = (Character.isDigit(para.charAt(j)) || para.charAt(j) == '.') && nonCategorical[i];
				}
			}
		}
		// initialize the range array for nonCategorical elements
		for (int i = 0; i < nonCategorical.length; i++) {
			valueMapArray[i] = null;
			if (nonCategorical[i]) {
				rangeArray[i] = null;

			} else {
				rangeArray[i] = new ArrayList<String>();
			}	
		}	
		// go through the exampleList and find all the categories
		// both test and training set. This depends on rangeArray[] not being null
		// if it is categorical (since it was initialized previously).
		for (int i = 0; i < training.size(); i++) {
			oneExample = training.get(i);
			for (int j = 0; j < oneExample.size(); j++) {
				if (rangeArray[j] != null) {
					String para = oneExample.get(new Integer(j));
					if (!rangeArray[j].contains(para)) {
						rangeArray[j].add(para);
					}
				}
			}
		}
		for (int i = 0; i < test.size(); i++) {
			oneExample = test.get(i);
			for (int j = 0; j < oneExample.size(); j++) {
				if (rangeArray[j] != null) {
					String para = oneExample.get(new Integer(j));
					if (!rangeArray[j].contains(para)) {
						rangeArray[j].add(para);	
					}
				}
			}
		}
		
		// create a new example list with 1-of-C encoding
		// or ordered encoding (in which case the
		// user has to be queried).
		
		// look through rangeArray and for each non-null value
		// query the user as to what value to map to the category
		// element
		for (int k = 0; k< rangeArray.length; k++) {
			if (ordered[k].isSelected()) {
				// build the rangeArray information if it does not exist
				if (rangeArray[k] == null) {
					rangeArray[k] = new ArrayList<String>();
					for (int i = 0; i < training.size(); i++) {
						oneExample = training.get(i);	
						String para = oneExample.get(new Integer(k));
						if (!rangeArray[k].contains(para)) {
							rangeArray[k].add(para);	
						}		
					}	
					for (int i = 0; i < test.size(); i++) {
						oneExample = test.get(i);	
						String para = oneExample.get(new Integer(k));
						if (!rangeArray[k].contains(para)) {
							rangeArray[k].add(para);	
						}
					}
				}
				NeuralWizardOrderedQueryDialog dial = new NeuralWizardOrderedQueryDialog((JFrame)super.getParent(), rangeArray[k], nodeList[k].getText());
				valueMapArray[k] = dial.returnValue();	
				dial.dispose();	
			}
		}
		
		// initialize a new example list
		ExampleList newExample = new ExampleList();
		// create a offsetPoint of categories. This will include 
		// all categories that need to be created. Note that
		// this should be ordered : input1, input2, ..., output1-outputX
		ArrayList<String> newPara = new ArrayList<String>();
		int counter = 0;
		// get all inputs
		// note that we put in inputs first before output. This is essential
		// to the backprop algorithm.
		for (int i = 0; i < nodeList.length; i++) {
			if (!nodeList[i].isSelected()) {
				// if rangeArray[i] is null, then the category is numerical
				// and we could just set it as an input node.
				if (rangeArray[i] == null) {
					newPara.add(nodeList[i].getText());
					counter++;	
				}
				// rangeArray[i] is not null BUT the user did not define it
				// as ordered. Thus, we put in one input node per
				// domain element.
				else if (!ordered[i].isSelected()) {
					//if there are only two attributes (binary), then only add one
					//as the other is the complement of the first
					int limit = rangeArray[i].size();
					if(rangeArray[i].size() == 2)
						limit--;
					for (int j = 0; j < limit; j++) {
						newPara.add(nodeList[i].getText()+"="+rangeArray[i].get(j));
						counter++;	
					}	
				}
				// rangeArray[i] is not null and the user defined it as ordered.
				// We just put in one input node.
				else {
					newPara.add(nodeList[i].getText());
					counter++;						
				}	
			}
		}
		int inputLimit = counter;
		// get all outputs
		for (int i = 0; i < nodeList.length; i++) {
			if (nodeList[i].isSelected()) {
				// if rangeArray[i] is null, then the category is numerical
				// and we could just set it as an output node.
				if (rangeArray[i] == null) {
					newPara.add(nodeList[i].getText());
					counter++;	
				}
				// rangeArray[i] is not null BUT the user did not define it
				// as ordered. Thus, we put in one input node per
				// domain element.
				else if (!ordered[i].isSelected()) {
					//if there are only two attributes (binary), then only add one
					//as the other is the complement of the first
					int limit = rangeArray[i].size();
					if(rangeArray[i].size() == 2)
						limit--;
					for (int j = 0; j < limit; j++) {
						newPara.add(rangeArray[i].get(j));
						counter++;	
					}	
				}
				// rangeArray[i] is not null and the user defined it as ordered.
				// We just put in one input node.
				else {
					newPara.add(nodeList[i].getText());
					counter++;						
				}	
			}
		}
		newExample.setParameters(newPara);
		// set the input and output in exampleList
		for (int i = 0; i < counter; i++) {
			if (i < inputLimit) {
				newExample.setInputParam(true, i);	
			} else {
				newExample.setInputParam(false, i);	
			}	
		}
		
		// transfer information from the old example list
		// to the new example list
		String[] values = null;
		for (int c = 0; c < training.size(); c++) {
			values = orderElements(training.get(c), newPara.size(), rangeArray, valueMapArray);
			newExample.addTrainingExample(newExample.getIndices(), values);
		}
		for (int c = 0; c < test.size(); c++) {
			values = orderElements(test.get(c), newPara.size(), rangeArray, valueMapArray);
			newExample.addTestExample(newExample.getIndices(), values);			
		}
		
		example = newExample;
		
		// build the neural graph
		
		// create the node layers
		NeuralNode node = null;
		
		// create the input node layer
		int X = -200;
		int Y = -200;
		// create a offsetPoint to keep track of
		// the previous layer
		ArrayList<NeuralNode> prevLayer = new ArrayList<NeuralNode>();
		for (int i = 0; i < inputLimit; i++) {
			node = new NeuralNode(nGraph);
			node.setLabel(newPara.get(i));
			Point p = new Point(X,Y);
			node.pos = p;
			nGraph.addNode(node);
			node.updateSize();	
			X += 400;		
			prevLayer.add(node);
		}
		X = -200;
		Y += 400;
		// create the hidden node layers
		int hiddenNumber = (new Integer(hiddenLayerNumber.getText())).intValue();
		int hiddenCounter = 1;
		for (int i = 0; i < hiddenNumber; i++) {
			// create a offsetPoint to store the
			// previous layer temporarily
			ArrayList<NeuralNode> tempList = new ArrayList<NeuralNode>();
			for (int j = 0; j < numHiddenNodes[i]; j++) {
				node = new NeuralNode(nGraph);
				node.setLabel("Hidden "+hiddenCounter);
				Point p = new Point(X,Y);
				hiddenCounter++;
				node.pos = p;
				nGraph.addNode(node);
				node.updateSize();	
				X += 400;		
				tempList.add(node);
				// draw edges from the previous layer to this node
				for (int k = 0; k < prevLayer.size(); k++) {
					NeuralNode to = node;
					NeuralNode from = prevLayer.get(k);
					if( !to.equals( from ) ) {
			   			from.addChild(to);
			   			to.addParent(from);
			   			NeuralEdge edge = new NeuralEdge(nGraph, from, to );
			   			nGraph.addEdge( edge, false );
					}
				}
			}
			X = -200;
			Y += 400;
			prevLayer = tempList;														
		}
		
		// create the output node layer
		for (int i = inputLimit; i < counter; i++) {
			node = new NeuralNode(nGraph);
			node.setLabel(newPara.get(i));
			Point p = new Point(X,Y);
			node.pos = p;
			nGraph.addNode(node);
			node.updateSize();	
			X += 400;
			// draw edges from the previous layer to this node
			for (int k = 0; k < prevLayer.size(); k++) {
				NeuralNode to = node;
				NeuralNode from = prevLayer.get(k);
				if( !to.equals( from ) ) {
		   			from.addChild(to);
		   			to.addParent(from);
		   			NeuralEdge edge = new NeuralEdge(nGraph, from, to );
		   			nGraph.addEdge( edge, false );
				}
			}				
		}
		nGraph.setExampleList(newExample);
		nGraph.setInputOutputNodes( true );	
		
		nGraph.canvas.repaint();
		nGraph.canvas.autoscale();
		return;
	}
	
	/**
	 * Helper function that creates an array of ordered input values from one example, using
	 * information from the range array (which stores whether or not a category is numerical
	 * or not), the valueMap array (which stores domain element - value mapping for ordered
	 * non-numerical data), and the length of the new array (corresponding to the number of
	 * actual categories).
	 */
	
	public String[] orderElements(Hashtable oneExample, int newLength, ArrayList[] rangeArray, Hashtable[] valueMapArray) {
		String[] val = new String[newLength];	
		int counter = 0;
		// get inputs
		for (int i = 0; i < nodeList.length; i++) {
			// this means that it is an input node
			if (!nodeList[i].isSelected()) {
				// if rangeArray[i] is null, the field is numerical, and just
				// copy the value to the correct spot.
				if (rangeArray[i] == null) {
					val[counter] = (String)oneExample.get(new Integer(i));	
					counter++;
				}
				// the field is non-numerical, but unordered. Create the same number of input
				// nodes as the number of domain elements, and put in 1.0 if the domain element
				// is true, 0.0 otherwise.	
				else if (!ordered[i].isSelected()) {
					String result = (String)oneExample.get(new Integer(i));
					//check for binary attributes
					int limit = rangeArray[i].size();
					if(rangeArray[i].size() == 2)
						limit--;
					for (int j = 0; j < limit; j++) {
						if (((String)rangeArray[i].get(j)).equals(result)) {
							val[counter] = (new Double(1.0)).toString();	
						} else {
							val[counter] = (new Double(0.0)).toString();								
						}
						counter++;	
					}						
				}
				// field is non-numerical, but ordered. Create one input node, and map
				// the domain element name key to the value in valueMapArray
				else {
					String result = (String)oneExample.get(new Integer(i));
					val[counter] = ((Double)valueMapArray[i].get(result)).toString();
					counter++;						
				}
			}		
		}
		// get outputs
		for (int i = 0; i < nodeList.length; i++) {
			// this means that it is an output node
			if (nodeList[i].isSelected()) {
				// if rangeArray[i] is null, the field is numerical, and just
				// copy the value to the correct spot
				if (rangeArray[i] == null) {
					val[counter] = (String)oneExample.get(new Integer(i));	
					counter++;
				}
				// the field is non-numerical, but unordered. Create the same number of output
				// nodes as the number of domain elements, and put in 1.0 if the domain element
				// is true, 0.0 otherwise.	
				else if (!ordered[i].isSelected()) {
					String result = (String)oneExample.get(new Integer(i));
					//check for binary attributes
					int limit = rangeArray[i].size();
					if(rangeArray[i].size() == 2)
						limit--;
					for (int j = 0; j < limit; j++) {
						if (((String)rangeArray[i].get(j)).equals(result)) {
							val[counter] = (new Double(1.0)).toString();	
						} else {
							val[counter] = (new Double(0.0)).toString();								
						}
						counter++;	
					}						
				}
				// field is non-numerical, but ordered. Create one input node, and map
				// the domain element name key to the value in valueMapArray
				else {
					String result = (String)oneExample.get(new Integer(i));
					val[counter] = ((Double)valueMapArray[i].get(result)).toString();
					counter++;
				}
			}		
		}
		// return the completed array
		return val;
	}
	
	/**
	 * Implements the itemStateChanged function of ItemListener
	 */
	public void itemStateChanged(ItemEvent e) {
		// A radio button determining the input and output nodes has been clicked.
		// This sets the exampleList flags to the correct settings.
		int index = (new Integer (((JComponent)e.getSource()).getName())).intValue();
		if (e.getStateChange() == ItemEvent.SELECTED) {
			example.setInputParam(false,index);	
		} else {
			example.setInputParam(true,index);					
		}
	}

	public void keyReleased(KeyEvent e) {
		// this checks whether the # field has a correct number set. If it is, then
		// it recreates the hiddden layer index choice box to the the appropriate number,
		// and reinitializes the array that stores the number of nodes per hidden layer,
		// copying what can be copied from the old array.
		if (((JComponent)e.getSource()).getName().equals("hiddenLayerNumber")) {
			errLabel.setText("");
			if (hiddenLayerNumber.getText() == "") {
				return;	
			}

			int hiddenNumber = 1;
			try {
				hiddenNumber = Integer.parseInt(hiddenLayerNumber.getText());	
			} catch (NumberFormatException ex) {
				errLabel.setText("Hidden Layer number must be integer");
				return;	
			}
			int[] numHidden = new int[hiddenNumber];
			comboModel.removeAllElements();
			for (int i = 0; i < hiddenNumber; i++) {	
				comboModel.addElement((new Integer(i+1)).toString());
				if (i < numHiddenNodes.length) {
					numHidden[i] = numHiddenNodes[i];	
				} else {
					numHidden[i] = 2;	
				}	
			}
			numHiddenNodes = numHidden;
			hiddenLayerIndex.repaint();		
			hiddenLayerIndex.setSelectedIndex(0);	
		}
		// this is primarily for saving the number of nodes per layer, and
		// errorchecking to see if the number is an integer
		else if (((JComponent)e.getSource()).getName().equals("hiddenLayerNodeNumber")) {
			errLabel.setText("");	
			if (hiddenLayerNodeNumber.getText() == "") {
				return;	
			}
			
			int hiddenNodeNumber = 1;
			try {
				hiddenNodeNumber = (new Integer(hiddenLayerNodeNumber.getText()).intValue());	
			} catch (NumberFormatException ex) {
				errLabel.setText("Number of nodes must be an integer");
				return;	
			}
			numHiddenNodes[hiddenLayerIndex.getSelectedIndex()] = hiddenNodeNumber;		
		}
	}
	
	public void keyTyped(KeyEvent e) {}
	public void keyPressed(KeyEvent arg0) {}
}