/**
 * NeuralWizardOrderedQueryDialog.java
 * July 19, 2002
 */

package AIspace.neural.dialogs;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;

/**
 * @author Joseph Roy Santos
 * Queries the user as to what values to map into
 * a categorical entry. This dialog does not have
 * a dispose() call because the calling function 
 * needs to get the map hashtable back (by calling
 * returnValue() ). The calling function MUST call
 * dispose() after calling returnValue().
 *
 */
public class NeuralWizardOrderedQueryDialog extends BasicDialog {
	private JLabel[] elementLabels;
	private JTextField[] elementFields;
	private JButton ok;
	private JLabel errLabel;
	private Hashtable<String, Double> map;
	private ArrayList<String> elements;

	/**
	 * Constructor. 
	 * @param parent	The parent frame of this dialog
	 * @param elements 	A offsetPoint of strings denoting the domain of a category
	 * @param name 		Name of a category
	 */
	NeuralWizardOrderedQueryDialog (JFrame parent, ArrayList<String> elements, String name) {
		super(parent, "Define Category Value Mapping", true);
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		this.map = new Hashtable<String, Double>();
		JPanel mainPanel = new JPanel(gbl);
		JLabel header = new JLabel("Input the values of these elements for "+name+" :");
		header.setForeground(Color.blue);
		addComponent(header, mainPanel, 0, 0, 2, 1, 2, 1);
		elementLabels = new JLabel[elements.size()];
		elementFields = new JTextField[elements.size()];
		gbc.fill = GridBagConstraints.HORIZONTAL;
		for (int i = 0; i < elements.size(); i++) {	
			elementLabels[i] = new JLabel(elements.get(i)+" :");
			addComponent(elementLabels[i], mainPanel, i+1, 0, 1, 1, 1, 1);
			elementFields[i] = new JTextField("");
			elementFields[i].addActionListener(this);
			addComponent(elementFields[i], mainPanel, i+1, 1, 1, 1, 1, 1);
		}
		this.elements = elements;
		errLabel = new JLabel();
		errLabel.setForeground(Color.red);
		addComponent(errLabel, mainPanel, elements.size()+1, 0, 2, 1, 2, 1);
		ok = new JButton("OK");
		ok.addActionListener(this);
		addComponent(ok, mainPanel, elements.size()+2, 0, 2, 1, 2, 1);
		
		getContentPane().add(mainPanel); 
			
		pack();
		centerWindow();
		setVisible(true);
	}
	
	protected boolean actionOK() {
		if (!setMap()) {
			errLabel.setText("Must set all fields to real numbers");
			return false;
		}
		return true;
	}
	
	protected boolean actionCancel() {
		//not triggered
		return true;
	}

	/** Implements ActionListener. */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() instanceof JTextField) {
			// this is called when a textfield gets an action. This erases the error label.
			// If adding a new if-statement, place it as an else-if so this will still catch
			// the TextField actions.	
			errLabel.setText("");
		} else {
			super.actionPerformed(e);
		}
	}
	
	/**
	 * Function that inputs key-value pairs to the map Hashtable. The key is the
	 * string name of the domain element, and the value is its associated value between
	 * 0 and 1. If any field is not valid, the hashtable is reset and this returns false.
	 * If the operation was successful, this returns true.
	 */
	private boolean setMap() {
		Double value;
		for (int i = 0; i < elementFields.length; i++) {
			try {
				value = new Double (elementFields[i].getText());
			} catch (NumberFormatException ex) {
				map = new Hashtable<String, Double>();
				return false;
			}	
			map.put(elements.get(i), value);
		}	
		return true;
	}
	
	/**
	 * Returns the map hashtable. This is typically called by functions which instantiate
	 * the dialog. Typically precedes a call to dispose().
	 */
	public Hashtable returnValue() {
		return map;
	}
}