/**
 * @author Wesley Coelho
 *
 * Prompts the user for parameter names when creating a new data set.  Parameter inputs in
 * comma delimited form are then parsed and set as the parameters in the ExampleList object.
 * By convention, the last parameter is the output value.
 * 
 * Update this comment when finished
 */

package AIspace.neural.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.util.StringTokenizer;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.graphToolKit.dialogs.MessageDialog;
import AIspace.neural.ExampleList;
import AIspace.neural.NeuralWindow;

public class ParameterInputDialog extends BasicDialog {
	private ExampleList eList;

	//Input Panel
	private JTextField promptText;
	
	/**
	 * Creates a new Dialog window for inputing parameter names when creating a new dataset.
	 * 
	 * @ param parent - The parent frame calling this constructor
	 * @ param exampleList - The exampleList object to store new parameters in
	 */
	public ParameterInputDialog(JFrame parent, ExampleList eList) {
		super(parent, "Data Set Parameter Input", true);
		this.eList	= eList;
		
		//Create the instructionPanel
		JPanel instructionPanel = new JPanel(new BorderLayout());
		instructionPanel.add(BorderLayout.NORTH, new JLabel("Please enter the parameters for the new dataset."));
		instructionPanel.add(BorderLayout.SOUTH, new JLabel("Separate each parameter with a comma and specify the output parameter last."));
		//Note: this could be done with one label but /n doesn't seem to work.			

		//Create the inputPanel
		JPanel inputPanel = new JPanel(new BorderLayout());
		inputPanel.add(BorderLayout.WEST, new JLabel("Parameters: "));
		promptText = new JTextField(50);
		inputPanel.add(BorderLayout.EAST, promptText);

		//Create the buttonPanel
		JPanel buttonPanel = new JPanel(new FlowLayout());
		JButton okButton = new JButton("OK");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);

		getContentPane().add(BorderLayout.NORTH, instructionPanel);
		getContentPane().add(BorderLayout.CENTER, inputPanel);
		getContentPane().add(BorderLayout.SOUTH, buttonPanel);
		pack();
		centerWindow();
		promptText.requestFocus();
		setVisible(true);
	}
	
	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return parseInput();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

	/**
	 * Parses the user's input.  If valid, it is sent to the
	 * parent window. (update this)
	 */
	private boolean parseInput() {
		//Will hold the extracted parameters
		ArrayList<String> parameters = new ArrayList<String>(5);
		// parses the input
		StringTokenizer tokenizer = new StringTokenizer(promptText.getText());
		while (tokenizer.hasMoreTokens()) {
			parameters.add(tokenizer.nextToken(", "));
		}

		if (parameters.size() < 2) {
			new MessageDialog((NeuralWindow) getParent()).open("Invalid Input", "You must enter at least two parameters");
			return false;
		}
		eList.setParameters(parameters);
		return true;
	}
}