/**
 * SelectExamplesDialog.java
 * 
 * Used by the ExampleDialog class to select examples.
 * This is adapted from the MovePartDialog class from the original code (MoveSidePanelPartDialog).
 * 
 * 
 */

package AIspace.neural.dialogs;

import java.awt.BorderLayout;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;


public class SelectExamplesDialog extends BasicDialog {  
	private DefaultListModel exampleModel;

	private JComboBox choiceComboBox;
	private JTextField X;

	private JLabel selectLabel = new JLabel("Select");
	private JLabel percentLabel = new JLabel("%    of the data");
	private JLabel labelError;

	private JButton okButton = new JButton("OK");
	private JButton cancelButton = new JButton("Cancel");

	private JPanel p;
	private JPanel q;
	private JPanel p1;
 
	private int numExamples;
   
	private JList exampleList;
   
	/**
	 * Creates and opens a dialog to select examples in the ExampleDialog window.
	 * 
	 * @param parent - the parent window of the dialog
	 * @param exampleList - the list of examples in the neural network
	 */
	public SelectExamplesDialog (JFrame parent, JList exampleList) {
		super(parent, "Select Data Examples", true);
		exampleModel =  (DefaultListModel) exampleList.getModel();
		this.exampleList = exampleList;
		this.exampleList.setModel(exampleModel);
			
		numExamples = exampleModel.getSize() -1 ;  
    
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
 
		String[] choiceString = {"Random", "the First", "the Last"};
		choiceComboBox =  new JComboBox(choiceString);
 
		X = new JTextField(3);

		labelError = new JLabel("");
		p = new JPanel();
		q = new JPanel();
     
		p.add(selectLabel);
		p.add(choiceComboBox);
		p.add(X);
		p.add(percentLabel); 
     

		q.add(okButton);
		q.add(cancelButton);

		p1 = new JPanel();
		p1.setLayout(new BorderLayout());
		p1.add(p, "Center");
		p1.add(q, "South");
 
		getContentPane().add(p1);
		getContentPane().add("South", labelError);
		pack();
		centerWindow();
		setVisible(true);
	}
	
	/** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		double x;
		int[] inds;
		if (parseX()) {
			x = new Double(X.getText()).doubleValue();
			int numToMove = (int)( numExamples * (x /100));
			inds = new int[numToMove];
		
			if (numToMove > 0) {
				if (choiceComboBox.getSelectedItem() == "the First") {
					for (int i = 0; i < numToMove; i++) {
						inds[i] = i;
					}
				} else if (choiceComboBox.getSelectedItem() == "the Last") {
					for (int i = 0; i < numToMove; i++) {   
						inds[i] = numExamples - numToMove + i ; 
					}
				} else if (choiceComboBox.getSelectedItem() == "Random") {	
					int n = (int)(numExamples*Math.random()); 
					inds[0] = n;
					boolean different;

					for (int i = 1; i < numToMove; i++) {   
						different = false;              
						while (! different) {
							n = (int)(numExamples*Math.random()); 
							different = true;
							for (int j = 0; j < i; j++) {
								if (inds[j] == n) {
									different = false;
									break;
								}
							}
						}
						inds[i] = n;
					}
				}
				selectItems(inds);
			} else {
				//select no items
			}
			return true;
		}
		return false;
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}
     
	/** select indicated items
	 * 
	 * @param inds - the indices of items to select
	 */
	private void selectItems(int[] inds) {
		for (int i = 0; i < inds.length; i++) {
			inds[i]= inds[i]+1;	//The + 1 corrects for the title string
		}
		exampleList.setSelectedIndices(inds);
	}

	/**
	 * Validates the percentage entered by the user.  Returns true if valid
	 */
	private boolean parseX() {
		try {
			double x = new Double(X.getText()).doubleValue();    
			if ((x < 0) || (x > 100))
				throw new NumberFormatException();   
		} catch (NumberFormatException e) {
			labelError.setText("Error: Parameter should have a real value between 0 and 100");
			return false;
		}
		
      	return true;
	}
}