/**
 * StoppingOptionsDialog.java
 * July 18, 2002
 */
package AIspace.neural.dialogs;

import java.awt.GridLayout;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import AIspace.graphToolKit.dialogs.BasicDialog;
import AIspace.neural.NeuralGraph;

/**
 * dialog box to set the number of iterations and error bound
 * before starting the back propagation algorithm
 * @author Leslie Tung, Shinjiro Sueda, Joseph Roy Santos
 */

public class StoppingOptionsDialog extends BasicDialog implements TextListener
{
   //
   // Fields
   //

   private JLabel labelIter;
   private JTextField textIter;
   private JLabel labelErr;
   private JTextField textErr;
   private JButton buttonOk;
   private JButton buttonCancel;
   private JLabel labelError;
   private JPanel p;

   private NeuralGraph graph;
   private int iterations;
   private double err;


   //
   // Methods
   //
   
   /**
    * construct the dialog
    *
    * @param paranet            the NeuralWindow
    * @param title                      the title of this dialog
    */
   public StoppingOptionsDialog (JFrame parent, NeuralGraph graph, String title) {
      super(parent, title, true);
      this.graph = graph;
      
      labelIter = new JLabel("Number Of Iterations:");
      labelErr = new JLabel( "Target Error:" );

      iterations = graph.getNumberOfIterations();
      textIter = new JTextField(iterations + "", 20);
      textIter.addActionListener(this);
      textIter.addActionListener( this );
      err = graph.getStoppingErrVal();
      textErr = new JTextField( err + "", 20 );
      textErr.addActionListener( this );
      textErr.addActionListener( this );
      
      buttonOk = new JButton("OK");
      buttonOk.addActionListener(this);
      buttonCancel = new JButton("Cancel");
      buttonCancel.addActionListener(this);
      labelError = new JLabel("", JLabel.CENTER);
      p = new JPanel();
      p.setLayout(new GridLayout(3, 2));
      p.add(labelIter);
      p.add(textIter);
      p.add(labelErr);
      p.add(textErr);
      p.add(buttonOk);
      p.add(buttonCancel);
      getContentPane().add(p);
      getContentPane().add("South", labelError);
      pack();
      centerWindow();
      setVisible(true);
   }

   /** opens this window and sets the default texts */
	public void open() {
	   iterations = graph.getNumberOfIterations();
	   textIter.setText(iterations + "");
	   err = graph.getStoppingErrVal();
	   textErr.setText( err + "" );
	   setVisible(true);
	}
   
   /**
    * set the number of iterations and the learning rate
    */
   public boolean setProperties() {
      try {
         iterations = Integer.valueOf(textIter.getText()).intValue();
         if( iterations <= 0 ) {
            labelError.setText( "Error: Number of iterations must be an integer > 0." );
            return false;
         }
         
         err = Double.valueOf( textErr.getText() ).doubleValue();
         if( err < 0 ) {
            labelError.setText( "Error: Target error value must be non-negative." );
            return false;
         }
         
         graph.setNumberOfIterations( iterations );
         graph.setStoppingErrVal( err );
         if (graph.pFrame != null) {
         	graph.pFrame.auto.setText("Step "+iterations+"X");
         }
         return true;
      } catch (NumberFormatException e) {
         labelError.setText("Error: Invalid number format!");
         return false;
      }
   }
   
   /** "OK" actionEvent from actionPerformed */
	protected boolean actionOK() {
		return setProperties();
	}
	
	/** "Cancel" actionEvent from actionPerformed */
	protected boolean actionCancel() {
		return true;
	}

   public void textValueChanged(TextEvent e) {
      if (e.getSource().equals(textIter)) {
         labelError.setText("");
      }
   }
}